@@ -59,10 +59,15 @@ class NedBeam {
 		double getYPosAt(double xpos);
 		void setTupletMarker(int tuplet_marker, double *topy, double *boty);
 		bool isBeamUp() {return m_up;}
-		void computeBeam(int stem_dir = STEM_DIR_NONE);
+		void computeBeam(int staff_voice_offs);
 		void freeChords();
 		void reconnect();
 		void saveBeam(FILE *fp);
+		NedChordOrRest *getFirst();
+		NedChordOrRest *getLast();
+		double getBeamLength();
+		bool isFirstInBeam(NedChordOrRest *chord);
+		bool isLastInBeam(NedChordOrRest *chord);
 		bool m_needed; // aux.
 	private:
 		void computeGraceBeam(NedChordOrRest *first, NedChordOrRest *last);
