@@ -24,6 +24,7 @@
 #define CHORD_OR_REST_H
 #include "config.h"
 #include "resource.h"
+#include "beam.h"
 
 #include <gtk/gtk.h>
 #include <cairo.h>
@@ -47,6 +48,9 @@
 #define TYPE_LINE3 (1 << 9)
 #define TYPE_FREE_TEXT (1 << 10)
 #define TYPE_SIGN (1 << 11)
+#define TYPE_GUITAR_CHORD (1 << 12)
+#define TYPE_SPACER (1 << 13)
+#define TYPE_GUITAR_CHORDNAME (1 << 14)
 
 
 #define LYR_CURSOR_LEFT -1
@@ -63,19 +67,23 @@
 #define WRITTEN_WEIGHT_NORMAL 0
 #define WRITTEN_WEIGHT_BOLD 1
 
+#define UNSET_STEM_HEIGHT -10000.0
+#define UNSET_SLOPE_OFFS -1000
+
 
 class NedMainWindow;
 class NedPage;
 class NedSystem;
 class NedStaff;
 class NedVoice;
-class NedBeam;
 class NedTuplet;
 class BBox;
 class NedCommandList;
 class NedNote;
 class NedFreeReplaceable;
 class NedSlur;
+class NedMeasure;
+class NedPangoCairoText;
 
 class VolumeChange {
 	public:
@@ -99,19 +107,28 @@ class NedChordOrRest {
 		NedChordOrRest(NedVoice *voice, int type, int keysig, int last_keysig, unsigned long long time); // key signature
 		~NedChordOrRest();
 		NedChordOrRest *clone(struct addr_ref_str **addrlist, bool keep_beam = false);
+		NedChordOrRest *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedVoice *p_voice);
 		void setActive();
+		void setForceBeamFlag() {m_status &= (~(STAT_BREAK_BEAM)); m_status |= STAT_FORCE_BEAM;}
+		void setBreakBeamFlag() {m_status &= (~(STAT_FORCE_BEAM)); m_status |= STAT_BREAK_BEAM;}
 		bool hasUpDir();
+		void flipStem(NedCommandList *command_list);
 		void print();
-		void draw(cairo_t *cr, bool *lyrics_present);
-		void drawlyrics(cairo_t *cr);
+		void draw(cairo_t *cr, bool *freetexts_or_lyrics_present);
+		void drawTexts(cairo_t *cr, double scale, double last_left_bbox_edge);
 		double getRealXpos() {return m_xpos;}
 		double getXPos() {if (m_tmp_xpos < 0.0) return m_xpos; return m_tmp_xpos;}
+		double getLeftBboxEdge() {return m_xpos + m_bbox.x;}
+		double getXPosAndMicroshifts() {if (m_tmp_xpos < 0.0) return m_xpos + m_micro_shift; return m_tmp_xpos + m_micro_shift;}
 		void insertNoteAt(NedNote *note, bool echo = FALSE);
 		bool removeNoteAt(int line);
 		bool noConflict(int line);
 		void testTies();
 		void reConfigure();
+		NedMeasure *m_measure;
 		NedNote *getFirstNote();
+		int getLineOfLowestNote();
+		int getLineOfFirstNote();
 		NedMainWindow *getMainWindow();
 		NedPage *getPage();
 		NedSystem *getSystem();
@@ -119,10 +136,9 @@ class NedChordOrRest {
 		NedVoice *getVoice() {return m_voice;}
 		void createLyrics(int line, char *text);
 		void setLyrics(int line, char *text);
-		void computeLyricsParams();
+		void setLyricsFromOldContext(NedMainWindow *main_window, int line, char *text);
 		void appendToLyrics(NedCommandList *command_list, int line, char *text); 
 		bool deleteLyricsLetter(NedCommandList *command_list, int line);
-		void computeFontExtentions(int line);
 		bool setLyricsCursor(int line, int pos);
 		void selectNextChord(int line, bool lyrics_required);
 		void selectPreviousChord(int line, bool lyrics_required);
@@ -131,53 +147,70 @@ class NedChordOrRest {
 		void setToRest(bool toRest);
 		int getType() {return m_type;}
 		bool isRest() {return (m_type == TYPE_REST);}
-		bool isHidden() {return m_is_hidden;}
+		bool isHidden() {return m_is_hidden || (m_status & STAT_USER_HIDDEN_REST);}
+		void setHidden() {m_is_hidden = true;}
 		void setTime(unsigned int time)  {m_time = time;}
 		unsigned int getStatus() {return m_status;}
 		void setStatus(unsigned int status) {m_status = status;}
 		unsigned int getLength() {return m_length;}
 		void setLength(int length);
 		bool trySelect (double x, double y, bool only_free_placeables);
+		bool hitStemEnd(double x, double y);
 		double computeDistFrom(double x, double y, double factor, double *ydist);
 		bool tryErease (double x, double y, bool *removed);
 		bool testXShift(double x);
+		void moveToLine(int line);
+		bool scaleStem(double y, double old_ypos);
+		void fixIntermediateValues();
+		double getCurrentStemLen() {return m_stem_height;}
+		double getOldStemLen() {return m_stemheight_copy;}
 		bool ReleaseTmpValue();
 		static int compareMidiTimes(NedChordOrRest *c1, NedChordOrRest *c2);
 		unsigned int getTime() {return m_time;}
 		unsigned int getDuration(unsigned int measure_duration = -1);
 		unsigned int determineVolume();
 		double determineTempoInverse();
+		void zoomFreeReplaceables(double zoom, double scale);
 		void determineTempoInverse(double *tempoinverse);
 		void setXPos(double pos);
+		void setXPos2(double pos);
 		void shiftX(double x);
 		void shiftY(double y);
+		void setRestsY(double y);
 		void setXPosAccordingBBox(double pos) {m_xpos = pos - m_bbox.x;}
 		unsigned int getStopTime();
 		void setBeam(NedBeam *beam) {m_beam = beam;}
 		NedBeam *getBeam() {return m_beam;}
 		void reposit(unsigned int pos);
 		void xPositNotes();
+		void collectNotesWithAccidentals(unsigned int *num_notes_with_accidentals, NedNote **note_field);
 		void sortNotes();
 		double getBeamXPos();
 		double getStemTop();
 		double getStemBottom();
-		void resetYPos() {m_ypos = - 3 * LINE_DIST/2.0;}
+		int getStemDir();
+		void resetYPos();
 		double getStemYStart();
-		void getTopBotY(int *lyrics_lines, double *topy, double *boty);
+		void getTopBotY(int *lyrics_lines, double *topy , double *boty/* , bool *topFix, bool *botFix */);
 		double getTopOfChordOrRest();
 		double getBottomOfChordOrRest();
 		bool testForTiesToDelete(NedCommandList *command_list, unsigned int dir = BREAK_TIE_BACKWARD, bool execute = false);
+		void testForDeleteableConstraints(NedCommandList *command_list);
+		static void handleOpenTies(NedCommandList *command_list, GList *pool);
 		bool isTied();
 		void tieCompleteTo(NedChordOrRest *other_chord);
 		int getDotCount() {return m_dot_count;}
 		void setDotCount(int dot_count) {m_dot_count = dot_count;}
 		NedBbox *getBBox() {return &m_bbox;}
 		void computeBbox();
+		void computeTies();
 		double getNeededSpace();
-		void setMidiTime(unsigned long long time, bool resetRestYPos = FALSE);
+		bool testMeasureSpread(int dir);
+		void setMidiTime(unsigned long long time);
 		void incrMidiTime(unsigned long long incr) {m_midi_time += incr;}
 		void decrMidiTime(unsigned long long incr); // {m_midi_time -= incr;}
 		unsigned long long getMidiTime() {return m_midi_time;}
+		bool isDown() {return m_beam ? !m_beam->isBeamUp() : !(m_status & STAT_UP);}
 		int getSorting(NedChordOrRest *other_chord);
 		void changeDuration(unsigned int new_duration, int tuplet_val);
 		NedChordOrRest *cloneWithDifferentLength(unsigned int newlen, int dotcount);
@@ -188,7 +221,13 @@ class NedChordOrRest {
 		NedChordOrRest *getPreviousChordOrRest();
 		NedChordOrRest *getFirstObjectInNextSystem();
 		int m_position;
+		int getStemDirectionOfTheMeasure(int staff_voice_offs);
 		double computeStemDist(bool thisup, NedChordOrRest *other_chord, bool otherup);
+		double getBeamOffset() {return m_beam_y_offs;} 
+		int getSlopeOffset() {return m_slope_offs;}
+		void setStemHeight(double stem_height) {m_stem_height = stem_height;}
+		void setBeamYOffset(double beam_offset) {m_beam_y_offs = beam_offset;}
+		void setBeamSlopeOffset(int beam_slope_offs) {m_slope_offs = beam_slope_offs;}
 		void saveChordOrRest(FILE *fp);
 		void changeStemDir(int dir);
 		void saveTies(FILE *fp, bool *ties_written);
@@ -198,7 +237,12 @@ class NedChordOrRest {
 		void updateVolume(int staff_nr);
 		void setOffset(char offs_array[115]);
 		int getPitchOfFirstNote();
-		double compute_microshift(NedChordOrRest *left_chord);
+		void setInternalPitch(int pitchdist);
+		void hideRests(NedCommandList *command_list, bool unhide);
+		void shiftNotes(int linedist);
+		void adjustAccidentals();
+		void resetMicroshift() {m_micro_shift = 0.0;}
+		double compute_microshift(NedChordOrRest *right_chord, int mnum, int pos);
 		double compute_y_shift(NedChordOrRest *other_chord, int dir);
 		bool hasLastTupletFlag() {return (m_tuplet_val & LAST_TUPLET_FLAG);}
 		void setLastTupletFlag(bool on) {if (on) m_tuplet_val |= LAST_TUPLET_FLAG; else m_tuplet_val &= (~(LAST_TUPLET_FLAG));}
@@ -211,16 +255,18 @@ class NedChordOrRest {
 		int getTupletVal() {return m_tuplet_val & TUPLET_VAR_MASK;}
 		int getTupletValRaw() {return m_tuplet_val;}
 		bool isUp();
-		void adjust_pointers(struct addr_ref_str *addrlist);
-		char *getLyrics(int line) {return m_lyrics[line];}
+		int getLine() {return m_line;}
+		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
+		char *getLyrics(int line);
 		void testNewMidiVolume(bool before);
-		void exportLilyPond(FILE *fp, int *last_line, unsigned int *midi_len, bool *tie, NedSlur **lily_slur, unsigned int *lyrics_map);
+		void exportLilyPond(FILE *fp, int *last_line, unsigned int *midi_len, NedSlur **lily_slur, unsigned int *lyrics_map, bool *chordname);
+		void exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len);
 		void exportLilyLyrics(FILE *fp, int line_nr, int *sil_count);
 	private:
 		void removeFreeDisplaceable(NedFreeReplaceable *freedisplaceable);
 		void addFreeDislaceable(NedFreeReplaceable *freedisplaceable);
 		void exportLilyAccents(FILE *fp);
-		void exportLilyFreePlaceables(FILE *fp, NedSlur **lily_slur);
+		void exportLilyFreePlaceables(FILE *fp, NedSlur **lily_slur, bool *chordnames);
 		GList *m_notes;
 		NedBeam *m_beam;
 		NedTuplet *m_tuplet_ptr;
@@ -235,21 +281,31 @@ class NedChordOrRest {
 		int m_line;
 		NedVoice *m_voice;
 		NedBbox m_bbox;
-		double m_end_sign_x_pos;
 		unsigned long long m_midi_time;
 		double m_tmp_xpos;
 		bool m_all_shifted;
 		double m_art_up_y_pos[MAX_ARTICULATIONS];
 		double m_art_down_y_pos[MAX_ARTICULATIONS];
 		int m_up_art_count, m_down_art_count;
-		char *m_lyrics[MAX_LYRICS_LINES];
-		cairo_text_extents_t *m_lyrics_extends[MAX_LYRICS_LINES];
-		int m_cursorpos;
+		NedPangoCairoText *m_lyrics[MAX_LYRICS_LINES];
 		GList *m_freedisp;
 		bool m_active;
+		double m_stem_end_xpos, m_stem_end_ypos;
+		double m_stem_height, m_stemheight_copy;
+		double m_beam_y_offs;
+		int m_slope_offs;
+		double m_beam_y_offs_copy;
+		double m_micro_shift;
+		int m_slope_offs_copy;
+		unsigned int m_status_copy;
+		bool m_free_texts_present, m_lyrics_present;
 	friend class NedNewLyricsCommand;
 	friend class NedDeleteLyricsCommand;
 	friend class NedFreeReplaceable;
+	friend class NedFixIntermediateCommand;
+	friend class NedMoveNoteRelativeCommand;
+	friend class NedFlipValuesCommand;
+	friend class NedRemoveConstraintsCommand;
 };
 
 #endif /* CHORD_OR_REST_H */
