@@ -54,7 +54,7 @@ void NedInsertChordOrRestCommand::unexecute(bool adjust /* = false */) {
 #endif
 	GList *inserted = g_list_find(m_voice->m_chord_or_rests, m_chord_or_rest);
 	if (inserted == NULL) {
-	  printf("voice %d (0x%x) gesucht: 0x%x\n", m_voice->getVoiceNumber(), (unsigned int)m_voice, (unsigned int)m_chord_or_rest);
+	  printf("voice %d (0x%lx) gesucht: 0x%lx\n", m_voice->getVoiceNumber(), (unsigned long)m_voice, (unsigned long)m_chord_or_rest);
 		NedResource::Abort("NedInsertChordOrRestCommand::unexecute");
 	}
 	m_voice->m_chord_or_rests = g_list_delete_link(m_voice->m_chord_or_rests, inserted); // friend !!
