@@ -47,6 +47,9 @@ NedPaperConfigDialog::NedPaperConfigDialog(GtkWindow *parent,
 	m_radio_box_A4 = gtk_radio_button_new_with_label(NULL, "A4");
 	m_radio_box_A3 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (m_radio_box_A4), "A3");
 	m_radio_box_B4 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (m_radio_box_A4), "B4");
+	m_radio_box_Letter = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (m_radio_box_A4), "Letter");
+	m_radio_box_Legal = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (m_radio_box_A4), "Legal");
+	m_radio_box_230x293 = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (m_radio_box_A4), "230x293");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_radio_box_A4), TRUE);
 	if (!strcmp(current_paper, "A3")) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_radio_box_A3), TRUE);
@@ -54,11 +57,23 @@ NedPaperConfigDialog::NedPaperConfigDialog(GtkWindow *parent,
 	else if (!strcmp(current_paper, "B4")) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_radio_box_B4), TRUE);
 	}
+	else if (!strcmp(current_paper, "Letter")) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_radio_box_Letter), TRUE);
+	}
+	else if (!strcmp(current_paper, "Legal")) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_radio_box_Legal), TRUE);
+	}
+	else if (!strcmp(current_paper, "230x293")) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_radio_box_230x293), TRUE);
+	}
 
 	paper_type_vbox = gtk_vbox_new(FALSE, 5);
 	gtk_box_pack_start (GTK_BOX (paper_type_vbox), m_radio_box_A4, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (paper_type_vbox), m_radio_box_A3, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (paper_type_vbox), m_radio_box_B4, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (paper_type_vbox), m_radio_box_Letter, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (paper_type_vbox), m_radio_box_Legal, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (paper_type_vbox), m_radio_box_230x293, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER(paper_type_frame), paper_type_vbox);
 
 	orientation_frame = gtk_frame_new(_("orientation"));
@@ -106,10 +121,20 @@ void NedPaperConfigDialog::CloseCallBack(GtkDialog *dialog, gint result, gpointe
 	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (paper_dialog->m_radio_box_B4))) {
 		paper_dialog->m_paper = "B4";
 	}
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (paper_dialog->m_radio_box_Letter))) {
+		paper_dialog->m_paper = "Letter";
+	}
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (paper_dialog->m_radio_box_Legal))) {
+		paper_dialog->m_paper = "Legal";
+	}
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (paper_dialog->m_radio_box_230x293))) {
+		paper_dialog->m_paper = "230x293";
+	}
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (paper_dialog->m_radio_box_landspace))) {
 		paper_dialog->m_portrait = FALSE;
 	}
 	gtk_widget_destroy (GTK_WIDGET(dialog));
+	gtk_widget_hide (GTK_WIDGET(dialog));
 }
 
 void NedPaperConfigDialog::getValues(bool *state, const char **paper, bool *portrait) {
