@@ -20,6 +20,8 @@
 /****************************************************************************************/
 
 #include "staffcontextdialog.h"
+#include "pangocairotext.h"
+#include "mainwindow.h"
 #include "localization.h"
 #include "resource.h"
 
@@ -37,12 +39,12 @@ _("F sharp Major; d sharp minor")
 };
 
 
-NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, int staff_count, int staff_pos, char *staff_name, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int reverb, bool muted) :
-m_octave_shift(octave_shift), m_tempo(tempo), m_pan(pan), m_chorus(chorus), m_reverb(reverb), m_volume(volume), m_channel_value(channel),
+NedStaffContextDialog::NedStaffContextDialog(GtkWindow *parent, NedMainWindow *main_window,  bool allow_delete_system, int staff_count, int staff_pos, char *staff_name, char *staff_short_name, int clef_number, int octave_shift, int key_signature_number, int numerator, int demoninator, int volume, int midi_program, int channel, int tempo, int pan, int chorus, int play_transposed, int reverb, bool muted) :
+m_octave_shift(octave_shift), m_tempo(tempo), m_pan(pan), m_chorus(chorus), m_play_transposed(play_transposed), m_reverb(reverb), m_volume(volume), m_channel_value(channel),
 m_state(false), m_numerator_value(numerator), m_denominator_value(demoninator),
 m_key_signature_number(key_signature_number), m_current_clef(clef_number), m_midi_program_number(midi_program), m_config_changed(false),
 m_delete_staff(false), m_staff_pos(staff_pos), 
-m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
+m_staff_count(staff_count), m_staff_name(staff_name), m_staff_short_name(staff_short_name), m_muted(muted)  {
 	GtkWidget *dialog;
 	GtkWidget *label_score;
 	GtkWidget *label_midi;
@@ -69,6 +71,7 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	GtkWidget *tempo_frame;
 	GtkWidget *pan_frame;
 	GtkWidget *chorus_frame;
+	GtkWidget *play_transposed_frame;
 	GtkWidget *reverb_frame;
 	GtkWidget *instrument_scroll;
 	GtkListStore *instrumend_list_store;
@@ -87,8 +90,9 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	GtkTreeViewColumn *position_num_column;
 	GtkTreePath *position_tree_path;
 	GtkObject *position_adjustment;
-	GtkWidget *position_scroll;
+	GtkWidget *position_scroll = NULL;
 	GtkWidget *staff_name_frame;
+	GtkWidget *staff_short_name_frame;
 	GtkWidget *signature_name_vbox;
 	int i;
 	char Str[128];
@@ -100,6 +104,9 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	if (staff_count > 1) {
 		gtk_dialog_add_button(GTK_DIALOG(dialog), "delete-staff", DELETE_STAFF_CMD);
 	}
+	if (allow_delete_system) {
+		gtk_dialog_add_button(GTK_DIALOG(dialog), "delete-system", DELETE_SYSTEM_CMD);
+	}
 	g_signal_connect(dialog, "response", G_CALLBACK (OnClose), (void *) this);
 
 
@@ -134,6 +141,10 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	gtk_box_pack_start(GTK_BOX(clef_hbox), octave_shift_vbox, FALSE, FALSE, 0);
 	g_signal_connect (m_octave_shift_spin_box, "value-changed", G_CALLBACK (octave_shift_change), (void *) this);
 
+	m_adjust_check_box = gtk_check_button_new_with_label(_("adjust notes"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_adjust_check_box), TRUE);
+	gtk_box_pack_start (GTK_BOX (octave_shift_vbox), m_adjust_check_box, FALSE, FALSE, 0);
+
 	if (m_newclef == MINCLEF_NUMBER) {
 		gtk_widget_set_sensitive(m_clef_bu_up, FALSE);
 	}
@@ -200,6 +211,13 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	}
 	gtk_container_add (GTK_CONTAINER(staff_name_frame), m_staffname);
 
+	staff_short_name_frame =  gtk_frame_new(_("staff short name"));
+	m_staffshortname = gtk_entry_new_with_max_length(50);
+	if (m_staff_short_name != NULL) {
+		gtk_entry_set_text(GTK_ENTRY(m_staffshortname), m_staff_short_name);
+	}
+	gtk_container_add (GTK_CONTAINER(staff_short_name_frame), m_staffshortname);
+
 
 	volume_frame = gtk_frame_new(_("volume"));
 	m_volume_scale = gtk_vscale_new_with_range(0.0, 127.0, 1.0);
@@ -208,7 +226,7 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	gtk_container_add (GTK_CONTAINER(volume_frame), m_volume_scale);
 
 	tempo_frame = gtk_frame_new(_("tempo"));
-	m_tempo_scale = gtk_vscale_new_with_range(20.0, 200.0, 1.0);
+	m_tempo_scale = gtk_vscale_new_with_range(20.0, 300.0, 1.0);
 	gtk_range_set_inverted(GTK_RANGE(m_tempo_scale), TRUE);
 	gtk_range_set_value(GTK_RANGE(m_tempo_scale), m_tempo);
 	gtk_container_add (GTK_CONTAINER(tempo_frame), m_tempo_scale);
@@ -225,6 +243,12 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	gtk_range_set_value(GTK_RANGE(m_chorus_scale), m_chorus);
 	gtk_container_add (GTK_CONTAINER(chorus_frame), m_chorus_scale);
 
+	play_transposed_frame = gtk_frame_new(_("transp."));
+	m_play_transposed_scale = gtk_vscale_new_with_range(-12.0, 12.0, 1.0);
+	gtk_range_set_inverted(GTK_RANGE(m_play_transposed_scale), TRUE);
+	gtk_range_set_value(GTK_RANGE(m_play_transposed_scale), m_play_transposed);
+	gtk_container_add (GTK_CONTAINER(play_transposed_frame), m_play_transposed_scale);
+
 	reverb_frame = gtk_frame_new(_("reverb"));
 	m_reverb_scale = gtk_vscale_new_with_range(20.0, 127.0, 1.0);
 	gtk_range_set_inverted(GTK_RANGE(m_reverb_scale), TRUE);
@@ -273,7 +297,12 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 
 		for (i = 0; i < staff_count; i++) {
 			gtk_list_store_append (position_list_store, &iter);
-			gtk_list_store_set (position_list_store, &iter, 0, _("staff") , 1, i, -1);
+			if (main_window->m_staff_contexts[i].m_staff_name != NULL && strlen(main_window->m_staff_contexts[i].m_staff_name->getText()) > 0) {
+				gtk_list_store_set (position_list_store, &iter, 0, main_window->m_staff_contexts[i].m_staff_name->getText() , 1, i, -1);
+			}
+			else {
+				gtk_list_store_set (position_list_store, &iter, 0, _("staff") , 1, i, -1);
+			}
 		}
 
 		m_position_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(position_list_store));
@@ -303,6 +332,7 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	signature_name_vbox = gtk_vbox_new(FALSE, 2);
 	gtk_box_pack_start(GTK_BOX(signature_name_vbox), time_signature_frame, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(signature_name_vbox), staff_name_frame, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(signature_name_vbox), staff_short_name_frame, FALSE, FALSE, 0);
 
 	score_hbox = gtk_hbox_new(FALSE, 2);
 	midi_hbox = gtk_hbox_new(FALSE, 2);
@@ -314,6 +344,7 @@ m_staff_count(staff_count), m_staff_name(staff_name), m_muted(muted)  {
 	gtk_box_pack_start(GTK_BOX(midi_hbox), pan_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), chorus_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), reverb_frame, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(midi_hbox), play_transposed_frame, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), m_instrument_list, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), instrument_scroll, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(midi_hbox), mute_channel_vbox, FALSE, TRUE, 0);
@@ -346,10 +377,13 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	GList *selected_rows;
 	int newchannel, newvolume, newpgm;
 	int newnum, newdenom, newkeysig, newtempo;
-	int newpan, newchorus, newreverb, new_octave_shift;
-	const char *newname;
+	int newpan, newchorus, newplaytransposed, newreverb, new_octave_shift;
+	const char *newname, *newshortname;
+	bool diff;
 
 	NedStaffContextDialog *context_dialog = (NedStaffContextDialog *) data;
+	context_dialog->m_delete_staff = FALSE;
+	context_dialog->m_delete_system = FALSE;
 	switch (result) {
 		case GTK_RESPONSE_ACCEPT:
 			context_dialog->m_state = TRUE;
@@ -358,6 +392,10 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 			context_dialog->m_delete_staff = TRUE;
 			context_dialog->m_state = FALSE;
 			break;
+		case DELETE_SYSTEM_CMD:
+			context_dialog->m_delete_system = TRUE;
+			context_dialog->m_state = FALSE;
+			break;
 		default:
 			context_dialog->m_state = FALSE;
 		break;
@@ -408,6 +446,11 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 		context_dialog->m_chorus = newchorus;
 		context_dialog->m_config_changed = TRUE;
 	}
+	newplaytransposed = (int) gtk_range_get_value(GTK_RANGE(context_dialog->m_play_transposed_scale));
+	if (newplaytransposed != context_dialog->m_play_transposed) {
+		context_dialog->m_play_transposed = newplaytransposed;
+		context_dialog->m_config_changed = TRUE;
+	}
 	newreverb = (int) gtk_range_get_value(GTK_RANGE(context_dialog->m_reverb_scale));
 	if (newreverb != context_dialog->m_reverb) {
 		context_dialog->m_reverb = newreverb;
@@ -418,22 +461,40 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 		context_dialog->m_muted = newmute;
 		context_dialog->m_config_changed = TRUE;
 	}
+	context_dialog->m_do_adjust = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context_dialog->m_adjust_check_box));
 	newname = gtk_entry_get_text(GTK_ENTRY(context_dialog->m_staffname));
-	bool diff = (newname == NULL && context_dialog->m_staff_name != NULL) || 
+	if (strlen(newname) < 1) {
+		newname = NULL;
+	}
+	diff = (newname == NULL && context_dialog->m_staff_name != NULL) || 
 		(newname != NULL && context_dialog->m_staff_name == NULL);
 	if (newname  != NULL && context_dialog->m_staff_name != NULL) {
 		diff = diff || strcmp(newname, context_dialog->m_staff_name) != 0;
 	}
 	if (diff) {
-		if (context_dialog->m_staff_name != NULL) {
-			free(context_dialog->m_staff_name);
-			context_dialog->m_staff_name = NULL;
-		}
-		if (strlen(newname) > 0) {
-			context_dialog->m_staff_name = strdup(newname);
-		}
 		context_dialog->m_config_changed = TRUE;
 	}
+	if (newname != NULL && strlen(newname) > 0) {
+		context_dialog->m_staff_name = strdup(newname);
+	}
+	else {
+		context_dialog->m_staff_name = NULL;
+	}
+	newshortname = gtk_entry_get_text(GTK_ENTRY(context_dialog->m_staffshortname));
+	diff = (newshortname == NULL && context_dialog->m_staff_short_name != NULL) || 
+		(newshortname != NULL && context_dialog->m_staff_short_name == NULL);
+	if (newshortname  != NULL && context_dialog->m_staff_short_name != NULL) {
+		diff = diff || strcmp(newshortname, context_dialog->m_staff_short_name) != 0;
+	}
+	if (diff) {
+		context_dialog->m_config_changed = TRUE;
+	}
+	if (newshortname != NULL && strlen(newshortname) > 0) {
+		context_dialog->m_staff_short_name = strdup(newshortname);
+	}
+	else {
+		context_dialog->m_staff_short_name = NULL;
+	}
 	selection =  gtk_tree_view_get_selection (GTK_TREE_VIEW(context_dialog->m_instrument_list));
 	selected_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
 	if ( g_list_length(selected_rows) > 0) {
@@ -480,15 +541,18 @@ void NedStaffContextDialog::OnClose(GtkDialog *dialog, gint result, gpointer dat
 	gtk_widget_destroy (GTK_WIDGET(dialog));
 }
 
-void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, int *staff_pos, char **staff_name, int *clef_number, int *octave_shift, int *key_signature_number, int *numerator, int *demoninator, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *reverb, bool *config_changed, bool *muted) {
+void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, bool *delete_system, int *staff_pos, char **staff_name, char **staff_short_name, int *clef_number, int *octave_shift, int *key_signature_number, bool *ajdust_notes, int *numerator, int *demoninator, int *volume, int *midi_program, int *channel, int *tempo, int *pan, int *chorus, int *play_transposed, int *reverb, bool *config_changed, bool *muted) {
 	*delete_staff = m_delete_staff;
+	*delete_system = m_delete_system;
 	*state = m_state;
 	*staff_pos = m_staff_pos;
 	if (m_newclef != m_current_clef) {
 		m_current_clef = m_newclef;
 		m_config_changed = TRUE;
 	}
+	*ajdust_notes = m_do_adjust;
 	*staff_name = m_staff_name;
+	*staff_short_name = m_staff_short_name;
 	*clef_number = m_current_clef;
 	*key_signature_number = m_key_signature_number;
 	*numerator = m_numerator_value;
@@ -497,6 +561,7 @@ void NedStaffContextDialog::getValues(bool *state, bool *delete_staff, int *staf
 	*tempo = m_tempo;
 	*pan = m_pan;
 	*chorus = m_chorus;
+	*play_transposed = m_play_transposed;
 	*reverb = m_reverb;
 	*octave_shift = m_octave_shift;
 	*midi_program = m_midi_program_number;
