@@ -499,6 +499,7 @@ void NedTableInstrument::resort() {
 	GList *track;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		track = g_list_sort(track, (gint (*)(const void*, const void*)) NedTableNote::compare);
@@ -513,6 +514,7 @@ void NedTableInstrument::snapNotesToMeasures(bool onlyRest, NedInstrumentTable *
 	bool changed;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		lptr2 = g_list_first(track);
@@ -545,6 +547,7 @@ void NedTableInstrument::tripletRecognition(unsigned long long  base) {
 	GList *track;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -561,6 +564,7 @@ void NedTableInstrument::combinePossibleTripletChords() {
 	bool chord_found;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -614,11 +618,12 @@ void NedTableInstrument::buildTriplets(unsigned long long base, bool allow_singl
 	NedTableNote *tableNote;
 	NedTableTuplet *table_tuplet;
 	GList *tmembers;
-	unsigned long long t0, t3;
+	unsigned long long t0 = 0, t3 = 0;
 	bool avoid_tuplet, av_tuplet;
 
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -816,6 +821,7 @@ void NedTableInstrument::handleTuplets(unsigned long long  base) {
 	unsigned long long dist;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -924,6 +930,7 @@ void NedTableInstrument::freeUnhandledTuplets() {
 	bool bad;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -1078,6 +1085,7 @@ void NedTableInstrument::buildChords() {
 	unsigned long long dist1, dist2;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -1310,6 +1318,7 @@ void NedTableInstrument::distributeOverlapping(int voice_num) {
 	int i, next_voice_idx;
 	bool overlapping, test_next;
 
+	if (g_list_first(m_note_list) == NULL) return;
 	for (i = 0; i < VOICE_COUNT - 1; new_voices[i++] = NULL);
 	track = (GList *) g_list_first(m_note_list)->data;
 	current_voice = &track;
@@ -1426,14 +1435,14 @@ GList *NedTableInstrument::convertToNtEdNotes(int voice_nr, NedInstrumentTable *
 	GList *track, *lptr, *lptr2, *trackptr;
 	NedNote *note;
 	int dotcount, len2;
-	NedChordOrRest *chord_or_rest, *chord_or_rest2;
+	NedChordOrRest *chord_or_rest = NULL, *chord_or_rest2;
 	NedTableNote *table_note = NULL;
 	GList *ned_notes = NULL;
 	GList *pitches =  NULL;
 	unsigned long long delta_time, d1, current_midi_time = 0;
 	unsigned long long measure_start, measure_end;
 	bool has_tie_backward;
-	NedChordOrRest *this_first_chord = NULL, *this_last_chord, *previous_last_chord = NULL;
+	NedChordOrRest *this_first_chord = NULL, *this_last_chord = NULL, *previous_last_chord = NULL;
 	NedTableNote *this_table_note = NULL, *last_table_note = NULL;
 	int mnum = 1;
 	NedTableTemposig *temposig;
@@ -1686,11 +1695,11 @@ GList *NedTableInstrument::convertToNtEdNotes(int voice_nr, NedInstrumentTable *
 	return ned_notes;
 }
 
-NedInstrumentTable::NedInstrumentTable(bool with_triplets, bool with_single_triplets, int voice_num,
-	int tempo_change_density, int volume_change_density) :
-m_tempo_inverse(START_TEMPO_INVERSE), m_tempo_change_density(tempo_change_density), m_volume_change_density(volume_change_density),
+NedInstrumentTable::NedInstrumentTable(bool with_triplets, bool with_single_triplets, bool with_alto_clef, int voice_num,
+	int tempo_change_density, int volume_change_density, bool force_piano, bool force_one_staff) :
+m_tempo_inverse(START_TEMPO_INVERSE), m_tempo_change_density(tempo_change_density), m_volume_change_density(volume_change_density), m_force_piano(force_piano), m_force_one_staff(force_one_staff), 
 m_instrument_list(NULL), timesig_list(NULL), temposig_list(NULL), m_pending_keysig(-111), m_last_inserted(NULL),
-m_with_triplets(with_triplets), m_with_single_triplets(with_single_triplets), m_voice_num(voice_num) {
+m_with_triplets(with_triplets), m_with_single_triplets(with_single_triplets), m_with_alto_clef(with_alto_clef), m_voice_num(voice_num) {
 	m_pending_instrument_name[0] = '\0';
 }
 
@@ -1822,7 +1831,7 @@ void NedTableInstrument::determineRange(bool determine_volume /* = false */) {
 	}
 }
 
-void NedTableInstrument::determineBestClef() {
+void NedTableInstrument::determineBestClef(bool with_alto_clef) {
 	determineRange();
 	static int clef_mids[] = { 50 - 12, 50, 60, 67, 67 + 12};
 	int i, min_idx = 3;
@@ -1841,7 +1850,10 @@ void NedTableInstrument::determineBestClef() {
 	switch (min_idx) {
 		case 0: m_clef = BASS_CLEF; m_octave_shift = -1; break;
 		case 1: m_clef = BASS_CLEF; m_octave_shift = 0; break;
-		case 2: m_clef = ALTO_CLEF; m_octave_shift = 0; break;
+		case 2: if (with_alto_clef) {
+				m_clef = ALTO_CLEF; m_octave_shift = 0; 
+				break;
+			}
 		case 3: m_clef = TREBLE_CLEF; m_octave_shift = 0; break;
 		case 4: m_clef = TREBLE_CLEF; m_octave_shift = 1; break;
 	}
@@ -1849,7 +1861,7 @@ void NedTableInstrument::determineBestClef() {
 
 
 void NedTableInstrument::testUnclosed(int track_nr) {
-	GList *track, *lptr, *lptr2;
+	GList *track, *lptr2;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
 		track = (GList *) g_list_first(m_note_list)->data;	
@@ -1857,7 +1869,7 @@ void NedTableInstrument::testUnclosed(int track_nr) {
 			if (!((NedTableNote *) lptr2->data)->m_closed) {
 				NedResource::DbgMsg(DBG_TESTING, "0x%x unclosed: start = %llu(%llu), track = %d\n", lptr2->data,
 					((NedTableNote *) lptr2->data)->m_start, ((NedTableNote *) lptr2->data)->m_start / NOTE_4, track_nr);
-				lptr2 = g_list_previous(lptr);
+				lptr2 = g_list_previous(lptr2);
 				track = g_list_delete_link(track, lptr2);
 				if (lptr2 == NULL) {
 					lptr2 = g_list_first(track);
@@ -1872,6 +1884,7 @@ void NedTableInstrument::detectClashes() {
 	GList *track, *lptr2;
 
 	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+		if (g_list_first(m_note_list) == NULL) return;
 		track = (GList *) g_list_first(m_note_list)->data;	
 		if (track == NULL) return;
 		for (lptr2 = g_list_first(track); lptr2; lptr2 = g_list_next(lptr2)) {
@@ -1951,29 +1964,36 @@ void NedTableInstrument::testForRests() {
 	//}
 }
 
-GList *NedTableInstrument::splitRange() {
+GList *NedTableInstrument::splitRange(bool force_piano, bool with_alto_clef) {
 	GList *new_instruments = NULL;
 	NedTableInstrument *instrument[3];
-	GList *track, *lptr, *lptr2;
+	GList *track, *lptr2;
 	NedTableNote *table_note;
 	int split[4][2], i;
 	bool piano = false;
+	bool do_continue;
 
 	determineRange(true);
 	if (m_max_pitch - m_min_pitch < 35) {
-		determineBestClef();
+		determineBestClef(with_alto_clef);
 		return NULL;
 	}
 
-	if (m_min_pitch >= m_piano_splits[0]  && m_max_pitch < m_piano_splits[2]) {
+	if (force_piano) {
+		piano = true;
+		split[0][0] = 0; split[0][1] = m_piano_splits[1];
+		split[1][0] = m_piano_splits[1]; split[1][1] = 129;
+		m_staves = 2;
+	}
+	else if ((m_min_pitch >= m_piano_splits[0]  && m_max_pitch < m_piano_splits[2])) {
 		if (m_piano_slots[0] && m_piano_slots[1]) {
-			piano = true;
 			split[0][0] = m_piano_splits[0]; split[0][1] = m_piano_splits[1];
 			split[1][0] = m_piano_splits[1]; split[1][1] = m_piano_splits[2];
+			piano = true;
 			m_staves = 2;
 		}
 		else {
-			determineBestClef();
+			determineBestClef(with_alto_clef);
 			return NULL;
 		}
 	}
@@ -1987,7 +2007,7 @@ GList *NedTableInstrument::splitRange() {
 			}
 		}
 		if (m_staves == 1) {
-			determineBestClef();
+			determineBestClef(with_alto_clef);
 			return NULL;
 		}
 	}
@@ -2000,67 +2020,82 @@ GList *NedTableInstrument::splitRange() {
 		if (m_instrument_name[0] != '\0') {
 			strcpy(instrument[i]->m_instrument_name, m_instrument_name);
 		}
-		if (split[i+1][0] == m_splits[0]) {
-			instrument[i]->m_clef = BASS_CLEF;
-			instrument[i]->m_octave_shift = -1;
-		}
-		else if (split[i+1][0] == m_splits[1]) {
-			instrument[i]->m_clef = BASS_CLEF;
-		}
-		else if (split[i+1][0] == m_splits[2]) {
-			instrument[i]->m_clef = TREBLE_CLEF;
-		}
-		else {
-			instrument[i]->m_clef = TREBLE_CLEF;
-			instrument[i]->m_octave_shift = 1;
-		}
+		if (!piano) {
+			if (split[i+1][0] == m_splits[0]) {
+				instrument[i]->m_clef = BASS_CLEF;
+				instrument[i]->m_octave_shift = -1;
+			}
+			else if (split[i+1][0] == m_splits[1]) {
+				instrument[i]->m_clef = BASS_CLEF;
+			}
+			else if (split[i+1][0] == m_splits[2]) {
+				instrument[i]->m_clef = TREBLE_CLEF;
+			}
+			else {
+				instrument[i]->m_clef = TREBLE_CLEF;
+				instrument[i]->m_octave_shift = 1;
+			}
+		 }
 		new_instruments = g_list_insert(new_instruments, instrument[i], 0);
 	}
-	if (split[0][0] == m_splits[0]) {
+	if (piano) {
+		instrument[0]->m_clef = TREBLE_CLEF;
+		instrument[0]->m_octave_shift = 0;
 		m_clef = BASS_CLEF;
-		m_octave_shift = -1;
-	}
-	else if (split[0][0] == m_splits[1]) {
-		m_clef = BASS_CLEF;
-	}
-	else if (split[0][0] == m_splits[2]) {
-		m_clef = TREBLE_CLEF;
+		m_octave_shift = 0;
 	}
 	else {
-		m_clef = TREBLE_CLEF;
-		m_octave_shift = 1;
+		if (split[0][0] == m_splits[0]) {
+			m_clef = BASS_CLEF;
+			m_octave_shift = -1;
+		}
+		else if (split[0][0] == m_splits[1]) {
+			m_clef = BASS_CLEF;
+		}
+		else if (split[0][0] == m_splits[2]) {
+			m_clef = TREBLE_CLEF;
+		}
+		else {
+			m_clef = TREBLE_CLEF;
+			m_octave_shift = 1;
+		}
 	}
 
-	for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+	//for (lptr = g_list_first(m_note_list); lptr; lptr = g_list_next(lptr)) {
+	if (g_list_first(m_note_list) != NULL)  {
 		track = (GList *) g_list_first(m_note_list)->data;	
 		lptr2 = g_list_first(track);
 		while (lptr2) {
+			do_continue = false;
 			table_note = (NedTableNote *) lptr2->data;
 			for (i = 1; i < m_staves; i++) {
 				if (table_note->m_midi_pitch >= split[i][0] && table_note->m_midi_pitch < split[i][1]) {
-					instrument[i-1]->addNote(table_note);
 					track = g_list_delete_link(track, lptr2);
+					instrument[i-1]->addNote(table_note);
 					lptr2 = g_list_first(track);
-					continue;
+					do_continue = true;
+					break;
 				}
 			}
+			if (do_continue) continue;
 			lptr2 = g_list_next(lptr2);
 		}
 		g_list_first(m_note_list)->data = (void *) track;
 	}
-	if (piano) {
-		instrument[0]->m_clef = TREBLE_CLEF;
-		m_clef = BASS_CLEF;
-	}
-	else {
-		determineBestClef();
+
+	//}
+	if (!piano) {
+		determineBestClef(with_alto_clef);
 		for (i = 0; i < m_staves - 1; i++) {
-			instrument[i]->determineBestClef();
+			instrument[i]->determineBestClef(with_alto_clef);
 		}
 	}
 	return new_instruments;
 }
 
+void NedTableInstrument::setBestClef(bool with_alto_clef) {
+	determineBestClef(with_alto_clef);
+}
 			
 
 void NedInstrumentTable::prepareNotes() {
@@ -2087,22 +2122,30 @@ void NedInstrumentTable::prepareNotes() {
 		}
 	}
 	NedResource::DbgMsg(DBG_TESTING, "Stelle B\n");
-	for (lptr = g_list_first(m_instrument_list); lptr; lptr = g_list_next(lptr)) {
-		ori_list = g_list_append(ori_list, lptr->data);
+	if (!m_force_one_staff) {
+		for (lptr = g_list_first(m_instrument_list); lptr; lptr = g_list_next(lptr)) {
+			ori_list = g_list_append(ori_list, lptr->data);
+		}
+		for (lptr = g_list_first(ori_list); lptr; lptr = g_list_next(lptr)) {
+			instr = (NedTableInstrument *) lptr->data;
+			new_instruments = instr->splitRange(m_force_piano, m_with_alto_clef);
+			for (lptr2 = g_list_first(new_instruments); lptr2; lptr2 = g_list_next(lptr2)) {
+				if ((lptr3 = g_list_find(m_instrument_list, lptr->data)) == NULL) {
+					NedResource::Abort("NedInstrumentTable::prepareNotes");
+				}
+				m_instrument_list = g_list_insert_before(m_instrument_list, lptr3, lptr2->data);
+			}
+			g_list_free(new_instruments);
+		}
+		g_list_free(ori_list);
+		NedResource::DbgMsg(DBG_TESTING, "Stelle C\n");
 	}
-	for (lptr = g_list_first(ori_list); lptr; lptr = g_list_next(lptr)) {
-		instr = (NedTableInstrument *) lptr->data;
-		new_instruments = instr->splitRange();
-		for (lptr2 = g_list_first(new_instruments); lptr2; lptr2 = g_list_next(lptr2)) {
-			if ((lptr3 = g_list_find(m_instrument_list, lptr->data)) == NULL) {
-				NedResource::Abort("NedInstrumentTable::prepareNotes");
-			}
-			m_instrument_list = g_list_insert_before(m_instrument_list, lptr3, lptr2->data);
+	else {
+		for (lptr = g_list_first(m_instrument_list); lptr; lptr = g_list_next(lptr)) {
+			instr = (NedTableInstrument *) lptr->data;
+			instr->setBestClef(m_with_alto_clef);
 		}
-		g_list_free(new_instruments);
 	}
-	g_list_free(ori_list);
-	NedResource::DbgMsg(DBG_TESTING, "Stelle C\n");
 	for (lptr = g_list_first(m_instrument_list); lptr; lptr = g_list_next(lptr)) {
 		((NedTableInstrument *) lptr->data)->snapNotesToMeasures(false, this);
 	}
