@@ -26,6 +26,7 @@
 #include <sys/time.h>
 #include <gtk/gtk.h>
 #include <cairo.h>
+#include <cairo-features.h>
 #include "resource.h"
 #include "paths.h"
 
@@ -40,11 +41,13 @@ class NedFreeReplaceable;
 class NedFreeText;
 class NedImporter;
 class NedMusicXMLImport;
+class NedPangoCairoText;
 class ScoreInfo;
 struct paper_info_struct;
 
 struct staff_context_str {
-	char *m_staff_name;
+	NedPangoCairoText *m_staff_name;
+	NedPangoCairoText *m_staff_short_name;
 	int m_key_signature_number;
 	int m_clef_number;
 	int m_clef_octave_shift;
@@ -53,6 +56,7 @@ struct staff_context_str {
 	unsigned char m_midi_pan;
 	unsigned char m_midi_chorus;
 	unsigned char m_midi_reverb;
+	char m_play_transposed;
 	unsigned char m_midi_program;
 	unsigned char m_midi_channel;
 	unsigned int m_flags;
@@ -92,6 +96,8 @@ class SpecialMeasure {
 		unsigned int type;
 #define REPEAT_OPEN  (1 << REP_TYPE_START_BIT)
 #define REPEAT_CLOSE (2 << REP_TYPE_START_BIT)
+#define DOUBLE_BAR   4
+#define END_BAR      5
 #define REPEAT_OPEN_CLOSE 3
 #define REP1START (1 << START_TYPE_START_BIT)
 #define REP1END   (1 << END_TYPE_START_BIT)
@@ -99,6 +105,7 @@ class SpecialMeasure {
 #define REP2END   (2 << END_TYPE_START_BIT)
 #define TIMESIG   (1 << TIMESIG_TYPE_START_BIT)
 		int measure_number;
+		bool hide_following;
 		unsigned int measure_duration;
 		NedMeasure *measure;
 		unsigned short numerator, denominator;
@@ -114,15 +121,39 @@ class SpecialMeasure {
 
 #define MAX_STAFFS 100
 
+typedef struct {
+	struct chord_str *chord_ptr;
+	char chord_name_num;
+	unsigned int status;
+} chord_info_struct;
+
+typedef struct {
+	char *root_name;
+	char *up_name;
+	char *down_name;
+} chord_name_info_struct;
+
+typedef union {
+	int m_special_sub_type_int;
+	chord_info_struct m_chord_info;
+	chord_name_info_struct m_chordname_info;
+} t_special_subtype;
+	
+	
+
 class NedMainWindow {
 	public:
 		NedMainWindow();
+		~NedMainWindow();
 		void createLayout(char *fname, char *yelparg);
 		double getCurrentZoomFactor() {return m_current_zoom;}
 		int getCurrentZoomLevel() {return m_current_zoom_level;}
+		double getCurrentScale() {return (double) m_current_scale / (double) SCALE_GRANULARITY;}
 		unsigned int getCurrentLength();
 		int getCurrentNoteHead();
 		unsigned int getStatus();
+		unsigned int getMainWindowStatus();
+		bool getShowHidden();
 		bool getRestMode();
 		int getLyricsMode();
 		int getDotCount();
@@ -131,8 +162,10 @@ class NedMainWindow {
 		double getTopY();
 		NedNote *m_selected_note;
 		NedChordOrRest *m_selected_chord_or_rest;
+		NedChordOrRest *m_selected_stem;
 		NedFreeReplaceable *m_selected_free_replaceable;
 		SpecialMeasure *m_selected_spec_measure;
+		NedNote *m_selected_tie_forward;
 		GdkCursor *m_hand, *m_pointer, *m_pencil;
 		NedSystem *getNextSystem(NedPage *page, NedCommandList *command_list = NULL);
 		NedPage *getNextPage(NedPage *page, NedCommandList *command_list = NULL);
@@ -144,7 +177,9 @@ class NedMainWindow {
 		void setUnRedoButtons(bool execute_possible, bool unexecute_possible);
 		void repaint();
 		void repaintDuringReplay(NedNote *notes[], int num_notes);
-		void reposit(NedCommandList *command_list = NULL, NedPage *start_page = NULL, NedSystem *start_system = NULL, bool full = false,  bool with_progress_bar = false);
+		void reposit(NedCommandList *command_list = NULL, NedPage *start_page = NULL, NedSystem *start_system = NULL,
+			bool with_progress_bar = false);
+		void setNewMeasureSpread(double spread);
 		void renumberMeasures(NedSystem *start_system = NULL, int measure_number = 1, bool force = false);
 		staff_context_str m_staff_contexts[MAX_STAFFS];
 		int getNumerator() {return m_numerator;}
@@ -157,7 +192,7 @@ class NedMainWindow {
 		int getPreStaffCount() {return m_pre_staffcount;}
 		void stopReplay();
 		void setVisibleSystem(NedSystem *system);
-		void setVisiblePage(NedPage *page);
+		//void setVisiblePage(NedPage *page);
 		void setVisible(NedNote *note);
 		void setVisible(NedChordOrRest *chor_or_rest);
 		void setSelected(NedChordOrRest *chord_or_rest, NedNote *note);
@@ -170,19 +205,22 @@ class NedMainWindow {
 		int getStaffCount() {return m_staff_count;}
 		void do_remove_last_staff();
 		bool do_import_from_other(NedImporter *im);
-		void do_lily_export(FILE *fp);
+		void do_lily_export(FILE *fp, bool with_break, bool *selected_staves);
+		void do_transpose(int pitchdist, bool *staff_list, GList *selected_group);
+		void shiftNotes(unsigned long long start_time, int linedist, NedPage *start_page, NedSystem *start_system, int staff_number);
 		void deleteStaff(int staff_number);
 		void shiftStaff(int staff_number, int position);
 		void restoreStaff(int staff_number, staff_context_str *staff_context);
 		unsigned int determineVolume(NedChordOrRest *element, int staff_nr);
 		double determineTempoInverse(NedChordOrRest *element);
+		void setInternalPitches();
+		void adjustAccidentals();
 		GtkWidget *getDrawingArea() {return m_drawing_area;}
 		GtkWidget *getWindow() {return m_main_window;}
 		struct paper_info_struct *getCurrentPaper() {return m_current_paper;}
 		bool getPortrait() {return m_portrait;}
-		void reconfig_paper(bool do_reposit, struct paper_info_struct *paper_type, bool portrait);
+		void reconfig_paper(bool do_reposit, struct paper_info_struct *paper_type, bool portrait, int scale);
 		void resetPointerLastTouchedSystems() {m_last_touched_system = NULL;}
-		bool isLastTouchedSystem(NedSystem *system);
 		int getCurrentVoice();
 		bool doPaintColored() {return m_paint_colored;}
 		bool doDrawPostscript() {return m_draw_postscript;}
@@ -190,57 +228,78 @@ class NedMainWindow {
 		NedSystem *m_start_system_for_renumbering;
 		bool needsARepLine(int measure_number);
 		int getSpecialType() {return m_special_type;}
-		int getSpecialSubType() {return m_special_sub_type;}
+		int getDoAdjust() {return ((m_special_type & (0xef000000)) >> 24);}
+		int getSpecialSubTypeInt() {return m_special_sub_type.m_special_sub_type_int;}
+		int getSpecialSubTypeChordname() {return m_special_sub_type.m_special_sub_type_int;}
+		chord_info_struct getSpecialSubTypeChordInfo() {return m_special_sub_type.m_chord_info;}
+		chord_name_info_struct getSpecialSubTypeChordNameInfo() {return m_special_sub_type.m_chordname_info;}
 		unsigned int getSpecial(int meas_num);
 		SpecialMeasure *getSpecialMeasure(int meas_num);
-		void resetSpecialType() {m_special_type = m_special_sub_type = -1;}
+		void resetSpecialType() {m_special_type = m_special_sub_type.m_special_sub_type_int = -1;
+				m_special_sub_type.m_chord_info.chord_ptr = NULL;}
 		void setAndUpdateClefTypeAndKeySig();
+		double get2ndNettoIndent() {return m_2ndnetto_indent;}
+		double getNettoIndent() {return m_netto_indent;}
 		double getFirstSystemIndent() {return m_first_system_indent;}
+		double get2ndSystemIndent() {return m_2nd_system_indent;}
 		unsigned int getUpBeatInverse() {return m_upbeat_inverse;}
 		int getFileVersion() {return m_file_version;}
+		void turnOffKeyboardCtrlMode() { m_keyboard_ctrl_mode= false; }
+		void setAllUnpositioned();
 		NedFreeReplaceable *m_freetext;
+		void drawVisibleRectangle(cairo_t *cr, NedNote *note);
 	private: 
 		GList *m_special_measures;
-		int setSpecial(int measnum, int mtype);
+		int setSpecial(int measnum, int mtype, bool hide, bool *oldhide);
 		void setSpecialTimesig(int measnum, int num, int denom, int *oldnum, int *olddenom, bool do_reposit = true);
 		static gint compare_spec_measures_according_measnum(gconstpointer ptr1, gconstpointer ptr2);
 		void readSpecMeasures(FILE *fp, GList **new_spec_measures);
 		bool conflictWithOhterSpecMeasure(int measnum, int dir);
 		void testAlternativeState(NedCommandList *command_list, NedMeasure *measure, bool alter1, bool alter2);
 		void checkForElementsToSplit(NedCommandList *command_list);
-		void import_from_other(NedImporter *im, const char *filter_name, const char *pat1, const char *pat2, const char *label, char **last_dir);
+		void import_from_other(NedImporter *im, char* filename);
 		void deleteEmptyPagesAtEnd();
+		void deletePagesWithoutSystem();
+		void rezoomScoreInfo();
+		void adjustAfterZoom();
 		void find_new_cursor_pos(guint keyval, int *x, int *y);
+		bool find_new_cursor_pos_at_measure(guint keyval, int *x, int *y);
+		int computeLilySize();
 		void disconnectKeys();
+		NedChordOrRest *findAndBeamElementsOfCurrentVoice();
+		void zoomFreeReplaceables(double zoom, double scale);
 		bool getTiedMode();
 		void resetSomeButtons();
 		void resetLyricsMode();
 		void moveSpecMeasure(SpecialMeasure *spec, int dir);
-		bool deleteEnclosedBras();
+		bool deleteEnclosedBras(unsigned int starts, unsigned int ends);
 		bool deleteEnclosedConnections();
 		void computeSystemIndent();
-		unsigned int getMainWindowStatus();
-		void modeChange(unsigned int stat, GtkAction *action);;
+		void modeChange(unsigned int stat, GtkAction *action);
 		char *m_start_file_name;
 		int m_last_line_during_key_insertion;
 		NedStaff *m_last_staff_during_key_insertion;
 		double m_current_zoom;
+		int m_current_scale; // * 50!!! avoid 1.e-10
 		double m_leftx, m_topy;
 		double m_leftx0, m_topy0;
 		double m_mouse_x, m_mouse_y;
 		int m_current_zoom_level;
 		int m_special_type;
-		int m_special_sub_type;
+		t_special_subtype m_special_sub_type;
 		GtkWidget *m_drawing_area;
 		GtkWidget *m_page_selector;
 		static cairo_status_t writefunc(void *fdv,  const unsigned char *data, unsigned int length);
-		void do_export_ps(FILE *psfile);
+#define STREAM_FILE_TYPE_PS 1
+#define STREAM_FILE_TYPE_PDF 2
+		void do_export_to_stream(FILE *psfile, int stream_file_type);
 		GList *m_pages;
 		void updateRecentFiles();
 		int m_recentFileMergeId;
 		NedCommandHistory *m_command_history;
+		void realizeNewSpacement(double spacement);
 		void do_restore(FILE *fp, char *filename);
-		void do_staff_config(int staff_number);
+		void do_staff_config(int staff_number, NedSystem *system);
 		void setToPage(int page_nr);
 		void computeScoreTextExtends();
 		int m_numerator, m_denominator;
@@ -258,6 +317,7 @@ class NedMainWindow {
 		GtkWidget *m_gracebar;
 		GtkWidget *m_drumbar;
 		GtkWidget *m_more_acc_bar;
+		GtkWidget *m_customize_beam_button;
 		GtkAction *m_undo_action, *m_redo_action;
 		GtkToggleAction *m_sharp_action, *m_flat_action, *m_natural_action, *m_insert_erease_mode_action;
 		GtkToggleAction *m_stacc_action, *m_staccatissimo_action;
@@ -273,8 +333,10 @@ class NedMainWindow {
 		GtkToggleAction *m_dsharp_action, *m_dflat_action;
 		GtkToggleAction *m_shift_action, *m_keyboard_insert_action;
 		GtkToggleAction *m_pause_action, *m_dotted_action, *m_ddotted_action;
+		GtkToggleAction *m_show_hidden_action;
 		GtkToggleAction *m_tie_action;
 		GtkToggleAction *m_replay_action;
+		GtkToggleAction *m_show_measure_numbers_action;
 		GtkActionGroup *m_menu_action_group;
 		GtkActionGroup *m_open_recent_action_group;
 		GtkActionGroup *m_note_head_action_group;
@@ -284,7 +346,7 @@ class NedMainWindow {
 		GSList *m_note_length_buttons;
 		GSList *m_note_head_buttons;
 		GtkAction *m_recent_actions[MAX_RECENT_FILES];
-		GtkAction *remove_page_action;
+		GtkAction *m_remove_page_action;
 		GtkWidget *m_v1bu, *m_v2bu, *m_v3bu, *m_v4bu;
 		GtkWidget *m_main_vbox;
 		GtkWidget *m_extra_bar_hbox;
@@ -300,7 +362,7 @@ class NedMainWindow {
 
 		static const GtkActionEntry main_tools[];
 		static const GtkRadioActionEntry note_actions[];
-		static const GtkToggleActionEntry toogle_buttons[];
+		static const GtkToggleActionEntry toggle_buttons[];
 		static const GtkToggleActionEntry accessory_buttons[];
 		static const GtkToggleActionEntry toggle_mode_buttons[];
 		static const GtkToggleActionEntry insert_erease_button[];
@@ -316,8 +378,6 @@ class NedMainWindow {
 		void storeScore(FILE *fp);
 		GdkRectangle m_selection_rect;
 		GList *m_selected_group;
-		GList *m_main_clip_board;
-		int m_number_of_first_selected_staff, m_number_of_last_selected_staff;
 
 		static void zoom_in(GtkWidget *widget, void *data);
 		static void zoom_out(GtkWidget *widget, void *data);
@@ -337,13 +397,16 @@ class NedMainWindow {
 		static void set_connected_bar_line(GtkWidget *widget, void *data);
 		static void remove_system_delimiter(GtkWidget *widget, void *data);
 		static void remove_tuplet(GtkWidget *widget, void *data);
-		static void append_page(GtkWidget *widget, void *data);
-		static void insert_page(GtkWidget *widget, void *data);
+/*		static void append_page(GtkWidget *widget, void *data); 
+		static void insert_page(GtkWidget *widget, void *data); */
 		static void insert_keysig(GtkWidget *widget, void *data);
 		static void insert_lines(GtkWidget *widget, void *data);
 		static void insert_signs(GtkWidget *widget, void *data);
 		static void insert_slur(GtkWidget *widget, void *data);
-		static void insert_accelerato(GtkWidget *widget, void *data);
+		static void insert_spacer(GtkWidget *widget, void *data);
+		static void insert_chord(GtkWidget *widget, void *data);
+		static void insert_chord_name(GtkWidget *widget, void *data);
+		static void insert_accelerando(GtkWidget *widget, void *data);
 		static void insert_ritardando(GtkWidget *widget, void *data);
 		static void append_staff(GtkWidget *widget, void *data);
 		static void insert_ppp(GtkAction *widget, gpointer data);
@@ -365,10 +428,16 @@ class NedMainWindow {
 		static void set_lyrics_mode5(GtkAction *widget, gpointer data);
 		static void remove_page(GtkWidget *widget, void *data);
 		static void remove_empty_pages(GtkWidget *widget, void *data);
+		static void change_spacement(GtkWidget *widget, void *data);
+		static void change_scale(GtkWidget *widget, void *data);
+		static void popup_test_menu(GtkWidget *widget, void *data);
 		static void set_upbeat_measure(GtkWidget *widget, void *data);
 		static void set_upbeat_start(GtkWidget *widget, void *data);
 		static void edit_score_info(GtkWidget *widget, void *data);
 		static void mute_staves(GtkWidget *widget, void *data);
+		static void transpose(GtkWidget *widget, void *data);
+		static void hide_rests(GtkWidget *widget, void *data);
+		static void unhide_rests(GtkWidget *widget, void *data);
 		static void config_midi(GtkWidget *widget, void *data);
 		static void config_paper(GtkWidget *widget, void *data);
 		static void show_about(GtkWidget *widget, void *data);
@@ -384,12 +453,13 @@ class NedMainWindow {
 		static void copy_data(GtkWidget *widget, void *data);
 		static void paste_data(GtkWidget *widget, void *data);
 		static void delete_block(GtkWidget *widget, void *data);
+		static void append_system(GtkWidget *widget, void *data);
 		static void insert_block(GtkWidget *widget, void *data);
 		static void insert_clef(GtkWidget *widget, void *data);
 		static void reposit_all(GtkWidget *widget, void *data);
 		static void empty_block(GtkWidget *widget, void *data);
-		static void spec_meas_test(GtkWidget *widget, void *data);
 		static void toggle_insert_erease_mode(GtkAction *action, gpointer data);
+		static void prepare_keyboard_insertion(GtkAction *action, gpointer data);
 		static void set_rest_note_mode(GtkAction *action, gpointer data);
 		static void set_dotted_mode(GtkAction *action, gpointer data);
 		static void set_ddotted_mode(GtkAction *action, gpointer data);
@@ -411,9 +481,12 @@ class NedMainWindow {
 		static void set_mordent_mode(GtkAction *action, gpointer data);
 		static void set_fermata_mode(GtkAction *action, gpointer data);
 		static void set_arpeggio_mode(GtkAction *action, gpointer data);
+		static void customize_beam(GtkAction *action, gpointer data);
+		static void flip_stem(GtkAction *action, gpointer data);
 		static void set_tied_mode(GtkAction *action, gpointer data);
 		static void do_play(GtkAction *action, gpointer data);
-		static void set_colored(GtkAction *action, gpointer data);
+		static void initiate_repaint(GtkAction *action, gpointer data);
+		static void set_paint_colored(GtkAction *action, gpointer data);
 		static void toggle_shift_mode(GtkAction *action, gpointer data);
 		static void set_sharp_mode(GtkAction *action, gpointer data);
 		static void set_flat_mode(GtkAction *action, gpointer data);
@@ -421,6 +494,7 @@ class NedMainWindow {
 		static void show_graces(GtkAction *action, gpointer data);
 		static void show_drum_bar(GtkAction *action, gpointer data);
 		static void show_more_accs_bar(GtkAction *action, gpointer data);
+		static void toggle_show_hidden(GtkAction *action, gpointer data);
 		static void set_note_length(GtkAction *action, GtkRadioAction *current, void *data);
 		static void set_drum_note(GtkAction *action, GtkRadioAction *current, void *data);
 		static void size_change_handler(GtkWidget *widget, GtkRequisition *requisition, gpointer data);
@@ -452,7 +526,17 @@ class NedMainWindow {
 	       			GdkEventMotion *event, gpointer data);
 
 		static void write_png(GtkWidget *widget, void *data);
+#ifdef CAIRO_HAS_SVG_SURFACE
+		static void write_svg(GtkWidget *widget, void *data);
+#endif
 		static void write_ps(GtkWidget  *widget, void *data);
+#ifdef CAIRO_HAS_PDF_SURFACE
+		static void write_pdf(GtkWidget  *widget, void *data);
+#endif
+		static void new_window(GtkWidget  *widget, void *data);
+		static void create_excerpt(GtkWidget  *widget, void *data);
+		static bool close_window2(GtkWidget *widget, GdkEvent  *event, gpointer data);
+		static void close_window(GtkWidget  *widget, void *data);
 		static void open_recent(GtkAction *action, gpointer user); 
 		double m_pointer_xpos, m_pointer_ypos;
 		struct paper_info_struct *m_current_paper;
@@ -460,8 +544,14 @@ class NedMainWindow {
 		bool m_portrait;
 		bool m_keyboard_ctrl_mode;
 		bool m_draw_postscript;
-		int last_full_reposit;
+		double m_netto_indent;
+		double m_2ndnetto_indent;
 		double m_first_system_indent;
+		double m_2nd_system_indent;
+		double m_global_spacement_fac;
+		gint m_last_cursor_x, m_last_cursor_y;
+		NedChordOrRest *m_last_obj;
+		NedSystem *m_last_y_adjustment;
 #ifdef YELP_PATH
 		char *m_docu;
 #endif
@@ -471,6 +561,7 @@ class NedMainWindow {
 	friend class NedInsertNewPageCommand;
 	friend class NedChangeMeasureTypeCommand;
 	friend class NedChangeMeasureTimeSignatureCommand;
+	friend class NedShiftStaffCommand;
 	friend class NedMoveSpecMeasureCommand;
 	friend class NedRemoveLastPageCommand;
 	friend class NedRemovePageCommand;
@@ -480,6 +571,8 @@ class NedMainWindow {
 	friend class NedMusicXMLImport;
 	friend class NedChangeUpbeadCommand;
 	friend class NedInstrumentTable;
+	friend class NedChangeSpacementCommand;
+	friend class NedDeleteStaffCommand;
 };
 
 #endif /* MAINWINDOW_H */
