@@ -25,7 +25,6 @@
 #include "system.h"
 #include "page.h"
 #include "voice.h"
-#include "resource.h"
 #include "system.h"
 #include "mainwindow.h"
 #include "commandlist.h"
@@ -34,10 +33,10 @@
 #include "untieforwardcommand.h"
 
 
-#define X_POS_PAGE_REL(p) ((getPage()->getContetXpos() + (p)) * zoom_factor - leftx)
+#define X_POS_PAGE_REL(p) ((getPage()->getContentXpos() + (p)) * zoom_factor - leftx)
 #define Y_POS_STAFF_REL(p) (((p) + getSystem()->getYPos() + getStaff()->getBottomPos()) * zoom_factor - topy)
-#define X_POS_INVERS_PAGE_REL(p) ((leftx + (p)) / zoom_factor - getPage()->getContetXpos())
-#define Y_POS_INVERS_STAFF_REL(p) ((topy + (p)) / zoom_factor - (getSystem()->getYPos() + getStaff()->getBottomPos()))
+#define X_POS_INVERS_PAGE_REL(p) ((leftx + (p) / current_scale) / zoom_factor - getPage()->getContentXpos())
+#define Y_POS_INVERS_STAFF_REL(p) ((topy + (p) / current_scale) / zoom_factor - (getSystem()->getYPos() + getStaff()->getBottomPos()))
 
 #define X_PS_POS(p) ((DEFAULT_BORDER + LEFT_RIGHT_BORDER + (p)) * PS_ZOOM)
 #define Y_PS_POS_STAFF_REL(p) ((height - ((p) + getSystem()->getYPos() + getStaff()->getBottomPos())) * PS_ZOOM)
@@ -77,8 +76,9 @@ int NedNote::signToBase[][2] = {
 int NedNote::diaDists[] = {-1, 0, 2, 4, 5, 7, 9, 11, 12, 14};
 
 NedNote::NedNote(NedChordOrRest *chord_or_rest, int line, int head, unsigned int status) :
-m_active(FALSE), m_acc_dist(ACCIDENTAL_BASE_DIST), m_line(line), m_tmp_line(UNKNOWN_LINE), m_status(status), m_head(head), m_chord_or_rest(chord_or_rest),
-m_tie_forward(NULL), m_tie_backward(NULL){
+m_active(FALSE), m_acc_dist(0.0), m_line(line), m_tmp_line(UNKNOWN_LINE), m_status(status), m_head(head), m_chord_or_rest(chord_or_rest),
+m_tie_forward(NULL), m_tie_backward(NULL), m_tie_y_offs1(0.0), m_tie_y_offs2(0.0), m_tie_xpos0(0.0), m_tie_xpos1(0.0), m_tie_back_xpos0(0.0),
+m_tie_back_xpos1(0.0), m_tie_offs(0.0) {
 	m_ypos = - line * LINE_DIST/2.0;
 	/*
 	NedResource::DbgMsg(DBG_TESTING, "keysig = 0, pitch = 62, pitchup = %d, pitchdown = %d\n", findPitchUpDown(0, 62, true),  findPitchUpDown(0, 62, false));
@@ -105,9 +105,14 @@ void NedNote::draw(cairo_t *cr) {
 	cairo_glyph_t glyph;
 	double yl;
 	double yy = (m_tmp_line == UNKNOWN_LINE) ? m_ypos : m_tmp_ypos;
-	double px0, px1;
 	int line;
-	double tie_y_offs1, tie_y_offs2;
+	/*
+	extern  bool xxx;
+	if (xxx) {
+		getMainWindow()->drawVisibleRectangle(cr, this);
+		xxx = false;
+	}
+	*/
 
 	if (getMainWindow()->m_selected_note == this || m_active) {
 		if (getMainWindow()->doPaintColored()) {
@@ -153,8 +158,23 @@ void NedNote::draw(cairo_t *cr) {
 			glyph.index = BASE + 25; break;
 		case TRIAG_NOTE2:
 			glyph.index = BASE + 26; break;
+		case CROSS_NOTE3:
+			glyph.index = BASE + 49; break;
+		case CROSS_NOTE4:
+			glyph.index = BASE + 50; break;
+		case TRIAG_NOTE3:
+			glyph.index = BASE + 51; break;
+		case QUAD_NOTE2:
+			glyph.index = BASE + 52; break;
+		case GUITAR_NOTE_STEM:
+			if (m_chord_or_rest->getLength() > NOTE_4) {
+				glyph.index = BASE + 54; break;
+			}
+			glyph.index = BASE + 53; break;
+		case GUITAR_NOTE_NO_STEM:
+			glyph.index = BASE + 53; break;
 	}
-	glyph.x = X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPos());
+	glyph.x = X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts());
 	glyph.y = Y_POS_STAFF_REL(yy);
 
 
@@ -163,23 +183,23 @@ void NedNote::draw(cairo_t *cr) {
 
 	switch (m_status & ACCIDENTAL_MASK) {
 		case STAT_DFLAT: glyph.index = BASE + 29;
-				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist);
+				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist /*+ ((m_line < -1 || m_line > 9) ? AUX_LINE_OVER: 0.0) */);
 				cairo_show_glyphs(cr, &glyph, 1);
 				break;
 		case STAT_FLAT: glyph.index = BASE + 16;
-				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist);
+				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist /*+ ((m_line < -1 || m_line > 9) ? AUX_LINE_OVER: 0.0) */);
 				cairo_show_glyphs(cr, &glyph, 1);
 				break;
-		case STAT_SHARP: glyph.index = BASE + 0;
-				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist);
+		case STAT_SHARP: glyph.index = BASE - 1;
+				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist /*+ ((m_line < -1 || m_line > 9) ? AUX_LINE_OVER: 0.0) */);
 				cairo_show_glyphs(cr, &glyph, 1);
 				break;
 		case STAT_DSHARP: glyph.index = BASE + 30;
-				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist);
+				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist /*+ ((m_line < -1 || m_line > 9) ? AUX_LINE_OVER: 0.0) */);
 				cairo_show_glyphs(cr, &glyph, 1);
 				break;
 		case STAT_NATURAL: glyph.index = BASE + 17;
-				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist);
+				glyph.x = X_POS_PAGE_REL(m_chord_or_rest->getXPos() - m_acc_dist /*+ ((m_line < -1 || m_line > 9) ? AUX_LINE_OVER: 0.0) */);
 				cairo_show_glyphs(cr, &glyph, 1);
 				break;
 	}
@@ -191,9 +211,9 @@ void NedNote::draw(cairo_t *cr) {
 		line *= 2;
 		for (; line < -1; line += 2) {
 			yl = - line * LINE_DIST/2.0;
-			cairo_move_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_xpos - HEAD_THICK_2 - AUX_LINE_OVER),
+			cairo_move_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPosAndMicroshifts() + m_xpos - HEAD_THICK_2 - AUX_LINE_OVER),
 				Y_POS_STAFF_REL(yl));
-			cairo_line_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_xpos + HEAD_THICK_2 + AUX_LINE_OVER),
+			cairo_line_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPosAndMicroshifts() + m_xpos + HEAD_THICK_2 + AUX_LINE_OVER),
 				Y_POS_STAFF_REL(yl));
 		}
 		cairo_stroke(cr);
@@ -205,9 +225,9 @@ void NedNote::draw(cairo_t *cr) {
 		line *= 2;
 		for (; line > 9; line -= 2) {
 			yl = - line * LINE_DIST/2.0;
-			cairo_move_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_xpos - HEAD_THICK_2 - AUX_LINE_OVER),
+			cairo_move_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPosAndMicroshifts() + m_xpos - HEAD_THICK_2 - AUX_LINE_OVER),
 				Y_POS_STAFF_REL(yl));
-			cairo_line_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPos() + m_xpos + HEAD_THICK_2 + AUX_LINE_OVER),
+			cairo_line_to(cr, X_POS_PAGE_REL(m_chord_or_rest->getXPosAndMicroshifts() + m_xpos + HEAD_THICK_2 + AUX_LINE_OVER),
 				Y_POS_STAFF_REL(yl));
 		}
 		cairo_stroke(cr);
@@ -215,13 +235,25 @@ void NedNote::draw(cairo_t *cr) {
 
 	if (m_chord_or_rest->getDotCount() > 0) {
 		cairo_new_path(cr);
-		cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPos() + HEAD_THICK + DOT1POS), Y_POS_STAFF_REL(yy),
+		if ((m_line % 2) == 0 && m_line >= 0 && m_line <= 8) {
+			cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT1POS), Y_POS_STAFF_REL(yy - LINE_DIST / 2.0),
 				zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
+		}
+		else {
+			cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT1POS), Y_POS_STAFF_REL(yy),
+				zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
+		}
 		cairo_fill(cr);
 		if (m_chord_or_rest->getDotCount() > 1) {
 			cairo_new_path(cr);
-			cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPos() + HEAD_THICK + DOT2POS), Y_POS_STAFF_REL(yy),
+			if ((m_line % 2) == 0 && m_line >= 0 && m_line <= 8) {
+				cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT2POS), Y_POS_STAFF_REL(yy - LINE_DIST / 2.0),
 					zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
+			}
+			else {
+				cairo_arc(cr, X_POS_PAGE_REL(m_xpos + m_chord_or_rest->getXPosAndMicroshifts() + HEAD_THICK + DOT2POS), Y_POS_STAFF_REL(yy),
+					zoom_factor * DOT_RAD, 0, 2.0 * M_PI);
+			}
 			cairo_fill(cr);
 		}
 			
@@ -231,64 +263,44 @@ void NedNote::draw(cairo_t *cr) {
 	}
 	if (m_tie_forward != NULL) {
 		if (m_chord_or_rest->getVoice() == m_tie_forward->getVoice()) {
-			if (getVoice()->getStemDir() == STEM_DIR_NONE) {
-				if (!m_chord_or_rest->hasUpDir() && !m_tie_forward->getChord()->hasUpDir()) {
-					tie_y_offs1 = -TIE_Y_OFFS1;
-					tie_y_offs2 = -TIE_Y_OFFS2;
-				}
-				else  {
-					tie_y_offs1 = TIE_Y_OFFS1;
-					tie_y_offs2 = TIE_Y_OFFS2;
-				}
+			if (getMainWindow()->m_selected_tie_forward == this) {
+				cairo_set_source_rgb (cr, 1.0, 0.0, 0.0);
 			}
-			else {
-				if (getVoice()->getStemDir() == STEM_DIR_UP) {
-					tie_y_offs1 = -TIE_Y_OFFS1;
-					tie_y_offs2 = -TIE_Y_OFFS2;
-				}
-				else  {
-					tie_y_offs1 = TIE_Y_OFFS1;
-					tie_y_offs2 = TIE_Y_OFFS2;
-				}
-			}
-			px0 = m_chord_or_rest->getXPos() + m_xpos + TIE_X_START;
-			px1 = m_tie_forward->m_chord_or_rest->getXPos() + m_tie_forward->m_xpos - TIE_X_START;
 			cairo_new_path(cr);
-			cairo_move_to(cr, X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos));
-			cairo_curve_to(cr,  X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos),
-						X_POS_PAGE_REL(px0 + (px1 - px0)/ 2.0), Y_POS_STAFF_REL(m_ypos + tie_y_offs1),
-						X_POS_PAGE_REL(px1), Y_POS_STAFF_REL(m_ypos));
-			cairo_curve_to(cr,  X_POS_PAGE_REL(px1), Y_POS_STAFF_REL(m_ypos),
-						X_POS_PAGE_REL(px0 + (px1 - px0)/ 2.0), Y_POS_STAFF_REL(m_ypos + tie_y_offs2),
-						X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos));
+			cairo_move_to(cr, X_POS_PAGE_REL(m_tie_xpos0), Y_POS_STAFF_REL(m_ypos));
+			cairo_curve_to(cr,  X_POS_PAGE_REL(m_tie_xpos0), Y_POS_STAFF_REL(m_ypos),
+						X_POS_PAGE_REL(m_tie_xpos0 + (m_tie_xpos1 - m_tie_xpos0)/ 2.0), Y_POS_STAFF_REL(m_ypos + m_tie_y_offs1),
+						X_POS_PAGE_REL(m_tie_xpos1), Y_POS_STAFF_REL(m_ypos));
+			cairo_curve_to(cr,  X_POS_PAGE_REL(m_tie_xpos1), Y_POS_STAFF_REL(m_ypos),
+						X_POS_PAGE_REL(m_tie_xpos0 + (m_tie_xpos1 - m_tie_xpos0)/ 2.0), Y_POS_STAFF_REL(m_ypos + m_tie_y_offs2),
+						X_POS_PAGE_REL(m_tie_xpos0), Y_POS_STAFF_REL(m_ypos));
 			cairo_fill(cr);
+			if (getMainWindow()->m_selected_tie_forward == this) {
+				cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+			}
 		}
 		else {
-			px1 = getPage()->getContentWidth() + TIE_X_RIGHT_OVER;
-			px0 = m_chord_or_rest->getXPos() + m_xpos + TIE_X_START;
 			cairo_new_path(cr);
-			cairo_move_to(cr, X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos));
-			cairo_curve_to(cr,  X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos),
-						X_POS_PAGE_REL(px0 + (px1 - px0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS1),
-						X_POS_PAGE_REL(px1), Y_POS_STAFF_REL(m_ypos));
-			cairo_curve_to(cr,  X_POS_PAGE_REL(px1), Y_POS_STAFF_REL(m_ypos),
-						X_POS_PAGE_REL(px0 + (px1 - px0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS2),
-						X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos));
+			cairo_move_to(cr, X_POS_PAGE_REL(m_tie_xpos0), Y_POS_STAFF_REL(m_ypos));
+			cairo_curve_to(cr,  X_POS_PAGE_REL(m_tie_xpos0), Y_POS_STAFF_REL(m_ypos),
+						X_POS_PAGE_REL(m_tie_xpos0 + (m_tie_xpos1 - m_tie_xpos0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS1),
+						X_POS_PAGE_REL(m_tie_xpos1), Y_POS_STAFF_REL(m_ypos));
+			cairo_curve_to(cr,  X_POS_PAGE_REL(m_tie_xpos1), Y_POS_STAFF_REL(m_ypos),
+						X_POS_PAGE_REL(m_tie_xpos0 + (m_tie_xpos1 - m_tie_xpos0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS2),
+						X_POS_PAGE_REL(m_tie_xpos0), Y_POS_STAFF_REL(m_ypos));
 			cairo_fill(cr);
 		}
 			
 	}
 	if (m_tie_backward != NULL && m_chord_or_rest->getVoice() != m_tie_backward->getVoice()) {
-		px0 = getSystem()->m_system_start - TIE_X_OFFS;
-		px1 = m_chord_or_rest->getXPos() + m_xpos - TIE_X_START;
 		cairo_new_path(cr);
-		cairo_move_to(cr, X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos));
-		cairo_curve_to(cr,  X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos),
-					X_POS_PAGE_REL(px0 + (px1 - px0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS1),
-					X_POS_PAGE_REL(px1), Y_POS_STAFF_REL(m_ypos));
-		cairo_curve_to(cr,  X_POS_PAGE_REL(px1), Y_POS_STAFF_REL(m_ypos),
-					X_POS_PAGE_REL(px0 + (px1 - px0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS2),
-					X_POS_PAGE_REL(px0), Y_POS_STAFF_REL(m_ypos));
+		cairo_move_to(cr, X_POS_PAGE_REL(m_tie_back_xpos0), Y_POS_STAFF_REL(m_ypos));
+		cairo_curve_to(cr,  X_POS_PAGE_REL(m_tie_back_xpos0), Y_POS_STAFF_REL(m_ypos),
+					X_POS_PAGE_REL(m_tie_back_xpos0 + (m_tie_back_xpos1 - m_tie_back_xpos0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS1),
+					X_POS_PAGE_REL(m_tie_back_xpos1), Y_POS_STAFF_REL(m_ypos));
+		cairo_curve_to(cr,  X_POS_PAGE_REL(m_tie_back_xpos1), Y_POS_STAFF_REL(m_ypos),
+					X_POS_PAGE_REL(m_tie_back_xpos0 + (m_tie_back_xpos1 - m_tie_back_xpos0)/ 2.0), Y_POS_STAFF_REL(m_ypos + TIE_Y_OFFS2),
+					X_POS_PAGE_REL(m_tie_back_xpos0), Y_POS_STAFF_REL(m_ypos));
 		cairo_fill(cr);
 	}
 };
@@ -343,6 +355,13 @@ bool NedNote::testForTiesToDelete(NedCommandList *command_list, unsigned int dir
 	}
 	return deleted_ties;
 }
+double NedNote::getYpos() {
+#define GUITAR_YTOP_DISTANCE ( 1.6 * LINE_DIST )
+	if (m_head == GUITAR_NOTE_STEM || m_head == GUITAR_NOTE_NO_STEM) {
+		return m_ypos + GUITAR_YTOP_DISTANCE;
+	}
+	return (m_tmp_line == UNKNOWN_LINE) ? m_ypos : m_tmp_ypos;
+}
 
 void NedNote::correctTieForward() {
 	if (m_tie_forward != NULL) {
@@ -350,36 +369,37 @@ void NedNote::correctTieForward() {
 	}
 }
 
-void NedNote::setOffset(char offs_array[115]) {
-	int offs;
-	switch (getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_clef_number) {
-		case NEUTRAL_CLEF1:
-		case TREBLE_CLEF: offs = TREBLE_PITCH_OFFS; break;
-		case NEUTRAL_CLEF2:
-		case BASS_CLEF: offs = BASS_PITCH_OFFS; break;
-		case ALTO_CLEF: offs = ALTO_PITCH_OFFS; break;
-		case TENOR_CLEF: offs = TENOR_PITCH_OFFS; break;
-		case SOPRAN_CLEF: offs = SOPRAN_PITCH_OFFS; break;
-	}
+void NedNote::setOffset(char offs_array[OFFS_ARRAY_SIZE]) {
 	switch (m_status & ACCIDENTAL_MASK) {
-		case STAT_DFLAT: offs_array[m_line+offs] = -2;
+		case STAT_DFLAT: offs_array[m_line + OFFS_ARRAY_HALF] = -2;
 				break;
-		case STAT_FLAT: offs_array[m_line+offs] = -1;
+		case STAT_FLAT: offs_array[m_line + OFFS_ARRAY_HALF] = -1;
 				break;
-		case STAT_SHARP: offs_array[m_line+offs] = 1;
+		case STAT_SHARP: offs_array[m_line + OFFS_ARRAY_HALF] = 1;
 				break;
-		case STAT_DSHARP: offs_array[m_line+offs] = 2;
+		case STAT_DSHARP: offs_array[m_line + OFFS_ARRAY_HALF] = 2;
 				break;
-		case STAT_NATURAL: offs_array[m_line+offs] = VIRTUAL_NATURAL_OFFS;
+		case STAT_NATURAL: offs_array[m_line + OFFS_ARRAY_HALF] = VIRTUAL_NATURAL_OFFS;
 				break;
 	}
 }
 
 int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int *offs) {
 	int oct = 0;
-	int line;
+	int line = 0;
+	int x_oct_shift= 0;
 	*offs = 0;
 	int clef_offs = 0;
+	switch (octave_shift) {
+		case  12: x_oct_shift =  7; break;
+		case -12: x_oct_shift = -7; break;
+		case   1: x_oct_shift =  7; break;
+		case  -1: x_oct_shift = -7; break;
+		case   0: x_oct_shift =  0; break;
+		default: printf("octave_shift = %d\n", octave_shift);
+			NedResource::Abort("NedNote::pitchToLine");
+			break;
+	}
 
 	while (pitch < 60) {
 		pitch += 12; oct--;
@@ -430,13 +450,13 @@ int NedNote::pitchToLine(int pitch, int clef, int keysig, int octave_shift, int
 		}
 	}
 
-	return line + 7 * (oct - octave_shift) + clef_offs;
+	return line + 7 * oct - x_oct_shift + clef_offs;
 }
 
 int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bool *flat) {
 	int p;
-	int offs;
-	char offs_array[115];
+	int offs = 0;
+	char offs_array[OFFS_ARRAY_SIZE];
 	*flat = false;
 
 	if (m_tie_backward != 0) {
@@ -454,7 +474,9 @@ int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bo
 	}
 	p = lineToPitch[m_line+offs].pitch; 
 	memset(offs_array, 0, sizeof(offs_array));
-	getStaff()->findAccidentals(offs_array, getSystem()->getMeasure(m_chord_or_rest->getMidiTime()), m_chord_or_rest->getMidiTime());
+	//getStaff()->findAccidentals(offs_array, getSystem()->getMeasure(m_chord_or_rest->getMidiTime()), m_chord_or_rest->getMidiTime());
+	getStaff()->findAccidentals(offs_array, m_chord_or_rest->m_measure, m_chord_or_rest->getMidiTime());
+	
 	switch (m_status & ACCIDENTAL_MASK) {
 		case STAT_DFLAT: p -= 2; *flat = true;
 				break;
@@ -466,32 +488,32 @@ int NedNote::getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bo
 				break;
 		case STAT_NATURAL: break;
 		default:
-			if (offs_array[m_line+offs] == VIRTUAL_NATURAL_OFFS) { //natural
+			if (offs_array[m_line + OFFS_ARRAY_HALF] == VIRTUAL_NATURAL_OFFS) { //natural
 				break;
 			}
 			if (keysig < 0) {
-				if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+offs] == 0) {
+				if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+OFFS_ARRAY_HALF] == 0) {
 					p--;
 					*flat = true;
 					break;
 				}
 			}
-			if (lineToPitch[m_line+offs].min_sharp <= keysig && offs_array[m_line+offs] == 0) {
+			if (lineToPitch[m_line+offs].min_sharp <= keysig && offs_array[m_line+OFFS_ARRAY_HALF] == 0) {
 				p++;
 				break;
 			}
-			p += offs_array[m_line+offs];
-			if (offs_array[m_line+offs] < 0) *flat = true;
+			p += offs_array[m_line + OFFS_ARRAY_HALF];
+			if (offs_array[m_line + OFFS_ARRAY_HALF] < 0) *flat = true;
 			break;
 	} 
-	return p + octave_shift + pitch_offs;
+	return p + octave_shift + pitch_offs + getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_play_transposed;
 }
 
 	
 void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 	int p;
-	int offs;
-	char offs_array[115];
+	int offs = 0;
+	char offs_array[OFFS_ARRAY_SIZE];
 
 	switch (clef) {
 		case NEUTRAL_CLEF1:
@@ -504,36 +526,36 @@ void NedNote::removeUnneededAccidental(int clef, int keysig, int octave_shift) {
 	}
 	p = lineToPitch[m_line+offs].pitch + octave_shift; 
 	memset(offs_array, 0, sizeof(offs_array));
-	getStaff()->findAccidentals(offs_array, getSystem()->getMeasure(m_chord_or_rest->getMidiTime()), m_chord_or_rest->getMidiTime());
+	getStaff()->findAccidentals(offs_array, m_chord_or_rest->m_measure, m_chord_or_rest->getMidiTime());
 	switch (m_status & ACCIDENTAL_MASK) {
-		case STAT_FLAT: if (offs_array[m_line+offs] == -1) {
+		case STAT_FLAT: if (offs_array[m_line+OFFS_ARRAY_HALF] == -1) {
 					 m_status &= (~(ACCIDENTAL_MASK));
 					 break;
 				}
-				if (offs_array[m_line+offs] != VIRTUAL_NATURAL_OFFS) {
+				if (offs_array[m_line+OFFS_ARRAY_HALF] != VIRTUAL_NATURAL_OFFS) {
 					if (keysig < 0) {
-						if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+offs] != 2 /* natural */) {
+						if (lineToPitch[m_line+offs].min_flat <= -keysig && offs_array[m_line+OFFS_ARRAY_HALF] != 2 /* natural */) {
 							m_status &= (~(ACCIDENTAL_MASK));
 						}
 					}
 				}
 				break;
-		case STAT_SHARP: if (offs_array[m_line+offs] == 1) {
+		case STAT_SHARP: if (offs_array[m_line+OFFS_ARRAY_HALF] == 1) {
 					m_status &= (~(ACCIDENTAL_MASK));
 					break;
 				 }
-				if (offs_array[m_line+offs] != VIRTUAL_NATURAL_OFFS) {
-					if (lineToPitch[m_line+offs].min_sharp <= keysig && offs_array[m_line+offs] != 2 /* natural */) {
+				if (offs_array[m_line+OFFS_ARRAY_HALF] != VIRTUAL_NATURAL_OFFS) {
+					if (lineToPitch[m_line+offs].min_sharp <= keysig && offs_array[m_line+OFFS_ARRAY_HALF] != 2 /* natural */) {
 						m_status &= (~(ACCIDENTAL_MASK));
 					}
 				}
 				break;
 		case STAT_NATURAL: 
-				if (offs_array[m_line+offs] == 2) { //natural
+				if (offs_array[m_line+OFFS_ARRAY_HALF] == 2) { //natural
 					m_status &= (~(ACCIDENTAL_MASK));
 					break;
 				}
-				if (offs_array[m_line+offs] != 0) break;
+				if (offs_array[m_line+OFFS_ARRAY_HALF] != 0) break;
 				if (keysig < 0) {
 					if (lineToPitch[m_line+offs].min_flat > -keysig) {
 						m_status &= (~(ACCIDENTAL_MASK));
@@ -610,8 +632,30 @@ unsigned int NedNote::determineState(int pitch, int line, int half_offs, char of
 	return 0;
 }
 
+void NedNote::setInternalPitch(int pitchdist, int clef, int keysig, int octave_shift) {
+	bool dummy;
+
+	m_pitch = getPitch(clef, keysig, octave_shift, 0, &dummy) + pitchdist;
+}
+
+void NedNote::adjustAccidental(int clef, int keysig, int octave_shift) {
+	int offs;
+
+	m_line = pitchToLine(m_pitch, clef, keysig, octave_shift, &offs);
+	m_ypos = - m_line * LINE_DIST/2.0;
+
+	m_status &= (~(ACCIDENTAL_MASK));
+
+	switch (offs) {
+		case -1: m_status |= STAT_FLAT; break;
+		case  1: m_status |= STAT_SHARP; break;
+		default: m_status |= STAT_NATURAL; break;
+	}
+}
+
+
 void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int keysig, int octave_shift, int pitch_offs, 
-		 unsigned int grace_time, bool is_grace, bool is_stacc) {
+		 unsigned int grace_time, bool is_grace, bool is_stacc, unsigned short segno_sign) {
 	int pitch, pitch2, tt;
 	unsigned int i;
 	bool dummy;
@@ -619,60 +663,60 @@ void NedNote::prepareReplay(unsigned long long midi_start_time, int clef, int ke
 	pitch = getPitch(clef, keysig, octave_shift, pitch_offs, &dummy);
 	if (is_grace) {
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			midi_start_time - grace_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+			midi_start_time - grace_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else if (is_stacc && m_chord_or_rest->getLength() > NOTE_32 && grace_time == 0) {
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() / 2, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() / 2, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_TRILL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
 		pitch2 = findPitchUpDown(keysig, pitch, true);
 		for (tt = 0, i = 0; i < m_chord_or_rest->getDuration() / NOTE_32; i++, tt = 1 - tt) {
 			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			midi_start_time + i * NOTE_32, SND_SEQ_EVENT_NOTEON, tt ? pitch : pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+			midi_start_time + i * NOTE_32, SND_SEQ_EVENT_NOTEON, tt ? pitch : pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 			NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime() + (i + 1) * NOTE_32, SND_SEQ_EVENT_NOTEOFF, tt ? pitch : pitch2, 0, this);
+			m_chord_or_rest->getMidiTime() + (i + 1) * NOTE_32, SND_SEQ_EVENT_NOTEOFF, tt ? pitch : pitch2, 0, segno_sign, this);
 		}
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_PRALL) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
 		pitch2 = findPitchUpDown(keysig, pitch, true);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, this);
+		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else if (grace_time == 0 && (m_chord_or_rest->getStatus() & STAT_MORDENT) != 0 && (m_chord_or_rest->getStatus() & (STAT_STACC | STAT_STACCATISSIMO)) == 0) {
 		pitch2 = findPitchUpDown(keysig, pitch, false);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		midi_start_time,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+		m_chord_or_rest->getMidiTime() +  NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		midi_start_time + NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch2, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, this);
+		m_chord_or_rest->getMidiTime() + 2 * NOTE_32, SND_SEQ_EVENT_NOTEOFF, pitch2, 0, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+		midi_start_time + 2 * NOTE_32,  SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+		m_chord_or_rest->getMidiTime() +  m_chord_or_rest->getDuration(), SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 	else {
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, this);
+			midi_start_time, SND_SEQ_EVENT_NOTEON, pitch, getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_current_midi_volume, segno_sign, this);
 		NedResource::addMidiEvent(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()].m_midi_channel,
-			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() - grace_time, SND_SEQ_EVENT_NOTEOFF, pitch, 0, this);
+			m_chord_or_rest->getMidiTime() + m_chord_or_rest->getDuration() - grace_time, SND_SEQ_EVENT_NOTEOFF, pitch, 0, segno_sign, this);
 	}
 }
 
@@ -745,11 +789,12 @@ bool NedNote::trySelect(double x, double y) {
 	double leftx = getMainWindow()->getLeftX();
 	double topy = getMainWindow()->getTopY();
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+	double current_scale = getMainWindow()->getCurrentScale();
 	cairo_text_extents_t *extention = &(NedResource::fontextentions[zoom_level][4]);
 
 	double xl = X_POS_INVERS_PAGE_REL(x);
 	double yl = Y_POS_INVERS_STAFF_REL(y);
-	double xnote = m_xpos + m_chord_or_rest->getXPos();
+	double xnote = m_xpos + m_chord_or_rest->getXPosAndMicroshifts();
 
 	if (m_ypos + extention->y_bearing / zoom_factor < yl &&
 	    m_ypos + (extention->y_bearing  +  extention->height) / zoom_factor > yl &&
@@ -758,12 +803,28 @@ bool NedNote::trySelect(double x, double y) {
 		getMainWindow()->m_selected_note = this;
 		return TRUE;
 	}
+	if (m_tie_forward != NULL) {
+		if (xl > m_tie_xpos0 && xl < m_tie_xpos1) {
+			if (m_tie_y_offs1 > 0.0) {
+				if (yl > m_ypos && yl < m_ypos +  m_tie_y_offs1) {
+					getMainWindow()->m_selected_tie_forward = this;
+				}
+			}
+			else {
+				if (yl < m_ypos && yl > m_ypos + m_tie_y_offs1) {
+					getMainWindow()->m_selected_tie_forward = this;
+				}
+			}
+		}
+	}
 	return FALSE;
 }
 
 bool NedNote::testYShift(double y) {
+	if (m_head == GUITAR_NOTE_STEM  || m_head == GUITAR_NOTE_NO_STEM) return false;
 	double topy = getMainWindow()->getTopY();
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+	double current_scale = getMainWindow()->getCurrentScale();
 
 	double yl = Y_POS_INVERS_STAFF_REL(y);
 	int old_line = m_tmp_line;
@@ -795,19 +856,32 @@ void NedNote::moveUpDown(int line) {
 	m_line = line;
 	m_ypos = - m_line * LINE_DIST/2.0;
 	m_chord_or_rest->sortNotes();
-	getMainWindow()->reposit(NULL, getPage());
-	context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 	if (NedResource::m_midi_echo) {
+		context = &(getMainWindow()->m_staff_contexts[getStaff()->getStaffNumber()]);
 		NedResource::playImmediately(context->m_midi_channel, context->m_midi_program, getPitch(clef, keysig, octave_shift, 0, &dummy), context->m_midi_volume);
 	}
 	m_tmp_line = UNKNOWN_LINE;
 }
 
+void NedNote::shiftNoteLine(int linedist) {
+	m_line += linedist;
+	m_ypos = - m_line * LINE_DIST/2.0;
+}
+
 
 bool NedNote::testRelativeMove(int dist) {
 	return m_chord_or_rest->noConflict(m_line + dist);
 }
 
+bool NedNote::test_tie_increment(double incr) {
+	if (m_tie_forward == NULL) return false;
+	if (m_tie_offs + incr > MAX_TIE_INCR) return false;
+	if (m_tie_offs + incr < MIN_TIE_INCR) return false;
+	return true;
+}
+
+
+
 void NedNote::shiftNote(bool shift) {
 	switch(m_chord_or_rest->getLength()) {
 		case WHOLE_NOTE:
@@ -822,22 +896,80 @@ void NedNote::shiftNote(bool shift) {
 	}
 }
 
-void NedNote::placeAccidental(bool all_shifted, int acc_places[4]) {
+void NedNote::computeTie() {
+	if (m_tie_forward != NULL) {
+		if (m_chord_or_rest->getVoice() == m_tie_forward->getVoice()) {
+			if (m_chord_or_rest->getStemDir() == STEM_DIR_NONE) {
+				if (!m_chord_or_rest->hasUpDir() && !m_tie_forward->getChord()->hasUpDir()) {
+					m_tie_y_offs1 = -TIE_Y_OFFS1 + m_tie_offs;
+					m_tie_y_offs2 = -TIE_Y_OFFS2 + m_tie_offs;
+				}
+				else  {
+					m_tie_y_offs1 = TIE_Y_OFFS1 + m_tie_offs;
+					m_tie_y_offs2 = TIE_Y_OFFS2 + m_tie_offs;
+				}
+			}
+			else {
+				if (m_chord_or_rest->getStemDir() == STEM_DIR_UP) {
+					m_tie_y_offs1 = -TIE_Y_OFFS1 + m_tie_offs;
+					m_tie_y_offs2 = -TIE_Y_OFFS2 + m_tie_offs;
+				}
+				else  {
+					m_tie_y_offs1 = TIE_Y_OFFS1 + m_tie_offs;
+					m_tie_y_offs2 = TIE_Y_OFFS2 + m_tie_offs;
+				}
+			}
+			m_tie_xpos0 = m_chord_or_rest->getXPosAndMicroshifts() + m_xpos + TIE_X_START;
+			m_tie_xpos1 = m_tie_forward->m_chord_or_rest->getXPosAndMicroshifts() + m_tie_forward->m_xpos - TIE_X_START;
+		}
+		else {
+			m_tie_xpos1 = getPage()->getContentWidth() + TIE_X_RIGHT_OVER;
+			m_tie_xpos0 = m_chord_or_rest->getXPosAndMicroshifts() + m_xpos + TIE_X_START;
+		}
+			
+	}
+	if (m_tie_backward != NULL && m_chord_or_rest->getVoice() != m_tie_backward->getVoice()) {
+		m_tie_back_xpos0 = getSystem()->m_system_start - TIE_X_OFFS;
+		m_tie_back_xpos1 = m_chord_or_rest->getXPosAndMicroshifts() + m_xpos - TIE_X_START;
+	}
+}
+
+
+void NedNote::collectNotesWithAccidentals(unsigned int *num_notes_with_accidentals, NedNote **note_field) {
+	if (*num_notes_with_accidentals  >= MAX_NOTES_PER_WITH_ACCIDENTALS_STAFF) return;
+	if ((m_status & ACCIDENTAL_MASK) == 0) return;
+	note_field[*num_notes_with_accidentals] = this;
+	(*num_notes_with_accidentals)++;
+}
+
+int NedNote::compare_note_lines(NedNote **n1, NedNote **n2) {
+	if ((*n1)->m_line < (*n2)->m_line) return -1;
+	return 1;
+}
+
+double NedNote::placeAccidental(bool all_shifted, int acc_places[4], bool has_arpeggio, bool out) {
 	bool placed;
 	int  i;
 	if ((m_status & ACCIDENTAL_MASK) != 0) {
 		placed = false;
-		for (i = 0; !placed && i < 4; i++) {
-			if (m_line - acc_places[i] > 3) {
+		i = (has_arpeggio ?  1 : 0);
+		for (;!placed && i < 4; i++) {
+			if (m_line - acc_places[i] > 4) {
 				placed = true;
-				m_acc_dist = ACCIDENTAL_BASE_DIST + i * NedResource::m_acc_bbox.width - (all_shifted ?  HEAD_THICK : 0);
+				m_acc_dist = ACCIDENTAL_BASE_DIST + i * NedResource::m_acc_bbox.width + HEAD_THICK;
 				acc_places[i] = m_line;
 			}
 		}
 		if (!placed) {
 			NedResource::Warning("Cannot place accidental");
 		}
+		/*
+		if (m_line < -1 || m_line > 9) {
+			m_acc_dist += 2* AUX_LINE_OVER;
+		}
+		*/
 	}
+	return m_acc_dist;
 }
 
 void NedNote::computeBounds(int len, double *minx, double *miny, double *maxx, double *maxy) {
@@ -869,6 +1001,7 @@ void NedNote::computeBounds(int len, double *minx, double *miny, double *maxx, d
 		xx1 += AUX_LINE_OVER;
 	}
 
+#ifdef XXX
 	if ((m_status & ACCIDENTAL_MASK) != 0) {
 		if (-m_acc_dist /* + NedResource::m_acc_bbox.x */ < xx0) xx0 = -m_acc_dist /* + NedResource::m_acc_bbox.x */;
 		if (m_ypos + NedResource::m_acc_bbox.y < yy0) yy0 = m_ypos + NedResource::m_acc_bbox.y;
@@ -876,9 +1009,10 @@ void NedNote::computeBounds(int len, double *minx, double *miny, double *maxx, d
 			if (yy0 + NedResource::m_acc_bbox.height > yy1) yy1 = yy0 + NedResource::m_acc_bbox.height;
 		}
 	}
+#endif
 	switch (m_chord_or_rest->getDotCount()) {
-		case 1: if (xx1 < m_xpos + HEAD_THICK + DOT1POS) {xx1 =  m_xpos + HEAD_THICK +  DOT1POS;} break;
-		case 2: if (xx1 < m_xpos + HEAD_THICK + DOT2POS) {xx1 =  m_xpos + HEAD_THICK +  DOT2POS;} break;
+		case 1: if (xx1 < m_xpos + HEAD_THICK + DOT1POS) {xx1 =  m_xpos + HEAD_THICK + DOT1POS + 2 * DOT_RAD;} break;
+		case 2: if (xx1 < m_xpos + HEAD_THICK + DOT2POS) {xx1 =  m_xpos + HEAD_THICK + DOT2POS + 2 * DOT_RAD;} break;
 	}
 	if (xx0 < *minx)  *minx = xx0;
 	if (yy0 < *miny)  *miny = yy0;
@@ -922,7 +1056,7 @@ void NedNote::saveTies(FILE *fp, bool *ties_written) {
 		}
 		marker1 = NedResource::getMarkerOfAddress(m_tie_backward);
 		marker2 = NedResource::getMarkerOfAddress(this);
-		fprintf(fp, "( %d, %d )\n", marker1, marker2);
+		fprintf(fp, "( %d, %d, %f )\n", marker1, marker2, m_tie_backward->m_tie_offs);
 	}
 }
 
