@@ -27,6 +27,7 @@
 #include <gtk/gtk.h>
 #include <cairo.h>
 #include "chordorrest.h"
+#include "resource.h"
 
 class NedMainWindow;
 class NedPage;
@@ -56,20 +57,24 @@ class NedNote {
 		void handleRelease() {m_tmp_line = UNKNOWN_LINE;}
 		bool noConflict();
 		void shiftNote(bool shift);
-		void placeAccidental(bool all_shifted, int acc_places[4]);
+		double placeAccidental(bool all_shifted, int acc_places[4], bool has_arpeggio, bool out);
+		void collectNotesWithAccidentals(unsigned int *num_notes_with_accidentals, NedNote **note_field);
+		static int compare_note_lines(NedNote **n1, NedNote **n2);
 		bool trySelect (double x, double y);
-		double getYpos() {return (m_tmp_line == UNKNOWN_LINE) ? m_ypos : m_tmp_ypos;}
+		double getYpos();
 		bool testYShift(double y);
 		void moveUpDown(int line);
 		bool testRelativeMove(int dist);
 		int getTempLine() {return (m_tmp_line == m_line) ?  UNKNOWN_LINE : m_tmp_line;}
 		int getLine() {return m_line;}
+		void shiftNoteLine(int linedist);
 		unsigned int getStatus() {return m_status;}
 		void setStatus(unsigned int status);
 		void computeBounds(int len, double *minx, double *miny, double *maxx, double *maxy);
 		void do_tie(NedNote *other_note);
 		void setTieSimple(NedNote *other_note);
 		void setTieIfSamePitch(NedNote *other_note);
+		void computeTie();
 		bool isTheSame(NedNote *other_note);
 		void removeBackwardTie();
 		void removeForwardTie();
@@ -81,14 +86,17 @@ class NedNote {
 		void correctTieForward();
 		void saveNote(FILE *fp);
 		void saveTies(FILE *fp, bool *ties_written);
+		bool test_tie_increment(double incr);
+		void setInternalPitch(int pitchdist, int clef, int keysig, int octave_shift);
+		void adjustAccidental(int clef, int keysig, int octave_shift);
 		void prepareReplay(unsigned long long midi_start_time, int clef, int keysig, 
-			 int octave_shift,  int pitch_offs, unsigned int grace_time, bool is_grace, bool is_stacc);
+			 int octave_shift,  int pitch_offs, unsigned int grace_time, bool is_grace, bool is_stacc, unsigned short segno_sign);
 		bool m_active;
 		int getPitch(int clef, int keysig, int octave_shift, int pitch_offs, bool *flat);
 		void removeUnneededAccidental(int clef, int keysig, int octave_shift);
 		int getNoteHead() {return m_head;}
 		void setNoteHead(int head) {m_head = head;}
-		void setOffset(char offs_array[115]);
+		void setOffset(char offs_array[OFFS_ARRAY_SIZE]);
 		static void setTies(NedNote *n1, NedNote *n2);
 		void adjust_pointers(struct addr_ref_str *addrlist);
 		void exportLilyPond(FILE *fp, int *last_line, bool *tie);
@@ -96,6 +104,7 @@ class NedNote {
 		static int pitchToLine(int pitch, int clef, int keysig, int octave_shift, int *offs);
 		static unsigned int determineState(int pitch, int line, int half_offs, char offs_array[115], int keysig);
 		bool isFirstNote();
+		void setTieOffset(double offs) {m_tie_offs = offs;}
 	private:
 		int findPitchUpDown(int key, int pitch, bool up);
 		static pitch_descr lineToPitch[];
@@ -109,6 +118,15 @@ class NedNote {
 		int m_head;
 		NedChordOrRest *m_chord_or_rest;
 		NedNote *m_tie_forward, *m_tie_backward;
+		double m_tie_y_offs1, m_tie_y_offs2;
+		double m_tie_xpos0, m_tie_xpos1;
+		double m_tie_back_xpos0, m_tie_back_xpos1;
+		double m_tie_offs;
+		int m_pitch; // used temporarily
+
+	friend class NedChangeTieCommand;
+	friend class NedTieNotesCommand;
+	friend class NedMoveNoteRelativeCommand;
 };
 
 #endif /* NOTE_H */
