@@ -46,16 +46,20 @@
 #include "octavation.h"
 #include "acceleration.h"
 #include "freesign.h"
+#include "freechord.h"
+#include "freespacer.h"
+#include "pangocairotext.h"
+#include "freechordname.h"
 
 
-#define X_POS_INVERS(p) ((leftx + (p)) / zoom_factor - m_xpos)
-#define Y_POS_INVERS(p) ((topy + (p)) / zoom_factor)
-#define X_POS_INVERS_PAGE_REL(p) ((leftx + (p)) / zoom_factor - getContetXpos())
+#define X_POS_INVERS(p) ((leftx + (p) / current_scale) / zoom_factor - m_xpos)
+#define Y_POS_INVERS(p) ((topy + (p) / current_scale) / zoom_factor)
+#define X_POS_INVERS_PAGE_REL(p) (((leftx + (p) / current_scale) / zoom_factor - getContentXpos()))
 
 #define X_POS(p) (((p) + m_xpos) * zoom_factor - leftx)
 #define Y_POS(p) ((p) * zoom_factor - topy)
 
-#define X_POS_PAGE_REL(p) (((p) + getContetXpos()) * zoom_factor - leftx)
+#define X_POS_PAGE_REL(p) (((p) + getContentXpos()) * zoom_factor - leftx)
 
 #define DEFAULT_SYSTEM_DIST (2 * 5 * LINE_DIST)
 NedPage::NedPage(NedMainWindow *main_window, double width, double height, int nr, unsigned int start_measure_number, bool start) :
@@ -65,11 +69,15 @@ m_width(width), m_height(height), default_border(DEFAULT_BORDER), m_page_number(
 	int i;
 	double system_pos = TOP_BOTTOM_BORDER;
 	NedSystem *system;
-	GList *lptr;
+
 
 	if (start) {
 		i = 0;
 		m_system_diff = 100.0;
+		m_systems = g_list_append(m_systems, system = new NedSystem(this, system_pos,
+			m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), i, start_measure_number, TRUE));
+		placeStaffs(true, 0);
+		/*
 		do {
 			m_systems = g_list_append(m_systems, system = new NedSystem(this, system_pos,
 				m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), i, start_measure_number, TRUE));
@@ -84,6 +92,7 @@ m_width(width), m_height(height), default_border(DEFAULT_BORDER), m_page_number(
 			m_systems = g_list_delete_link(m_systems, lptr);
 			placeStaffs(0);
 		}
+		*/
 	}
 }
 
@@ -97,6 +106,26 @@ NedPage::~NedPage() {
 	m_systems = NULL;
 }
 
+NedPage *NedPage::clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedMainWindow *p_main_window, bool *staves) {
+	GList *lptr;
+	NedPage *page = new NedPage(p_main_window, m_width, m_height, m_page_number, ((NedSystem *)g_list_first(m_systems)->data)->getNumberOfFirstMeasure(), false);
+
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		page->m_systems = g_list_append(page->m_systems, ((NedSystem *) lptr->data)->clone(addrlist, slurlist, page, staves));
+	}
+
+	return page;
+}
+
+void NedPage::adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist) {
+	GList *lptr;
+
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->adjust_pointers(addrlist, slurlist);
+	}
+}
+
+
 double NedPage::getTopPos() {return (2 * default_border +  m_height) * m_main_window->getCurrentZoomFactor();}
 double NedPage::getLeftPos() {return (( m_xpos + 2*default_border +  m_width)) * m_main_window->getCurrentZoomFactor();}
 
@@ -108,11 +137,18 @@ NedSystem *NedPage::appendSystem() {
 	m_systems = g_list_append(m_systems, system = new NedSystem(this, 0,
 	                                m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), system_nr, 1, TRUE));
 
-	placeStaffs(0);
+	placeStaffs(false, 0);
 
 	return system;
 }
 
+void NedPage::insertSystem(NedSystem *system) {
+	
+	m_systems = g_list_prepend(m_systems, system);
+	system->changePageInfo(this);
+}
+
+
 void NedPage::removeSystem(NedSystem *system) {
 	GList *lptr;
 	int system_nr;
@@ -127,10 +163,10 @@ void NedPage::removeSystem(NedSystem *system) {
 	}
 }
 
-void NedPage::insertSystem(NedSystem *system) {
-	
-	m_systems = g_list_prepend(m_systems, system);
-	system->changePageInfo(this);
+void NedPage::appendSystem(NedCommandList *command_list) {
+	NedAppendSystemCommand *append_sys_cmd = new NedAppendSystemCommand(this);
+	append_sys_cmd->execute();
+	command_list->addCommand(append_sys_cmd);
 }
 
 bool NedPage::isPageOverflow() {
@@ -139,31 +175,13 @@ bool NedPage::isPageOverflow() {
 	return m_system_diff <= MIN_SYSTEM_Y_DIST;
 }
 
-void NedPage::fill_up () {
-	int system_nr;
-	GList *lptr;
-
-	system_nr = g_list_length(m_systems);
-	do {
-		m_systems = g_list_append(m_systems, new NedSystem(this, 0,
-	                                m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), system_nr, 1, TRUE));
-		system_nr++;
-		placeStaffs(0);
-	}
-	while (m_system_diff > MIN_SYSTEM_Y_DIST);
-	if (g_list_length(m_systems) > 1) {
-		lptr = g_list_last(m_systems);
-		delete (NedSystem *) lptr->data;
-		m_systems = g_list_delete_link(m_systems, lptr);
-	}
-	for (system_nr = 0, lptr = g_list_first(m_systems); lptr; system_nr++, lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->setSystemNumber(system_nr);
-	}
-}
 
 unsigned int NedPage::getNumberOfLastMeasure() {
 	GList *lptr;
 	lptr = g_list_last(m_systems);
+	if (lptr == NULL) {
+		NedResource::Abort("NedPage::getNumberOfLastMeasure");
+	}
 	return ((NedSystem *) g_list_last(m_systems)->data)->getNumberOfLastMeasure();
 }
 
@@ -205,9 +223,14 @@ bool NedPage::isLast(NedSystem *system) {
 	return (lptr == g_list_last(m_systems));
 }
 
+NedSystem *NedPage::getLastSystem() {
+	return (NedSystem *) g_list_last(m_systems)->data;
+}
+
 
 bool NedPage::find_staff_and_line(int x, int y, NedStaff **last_staff, int *last_line) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double current_scale = m_main_window->getCurrentScale();
 	double leftx = m_main_window->getLeftX();
 	double bottom;
 
@@ -223,18 +246,20 @@ bool NedPage::find_staff_and_line(int x, int y, NedStaff **last_staff, int *last
 
 
 
-void NedPage::draw(cairo_t *cr, double main_width, double main_height) {
+void NedPage::draw(cairo_t *cr, bool show_measure_numbers, double main_width, double main_height) {
 	double leftx = m_main_window->getLeftX();
 	double topy = m_main_window->getTopY();
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double scale = m_main_window->getCurrentScale();
 	ScoreInfo *score_info = NULL;
-	bool score_info_present;
 	double xp, yp;
 	bool first = true;
+	bool freetext_or_lyrics_present;
+	cairo_scale(cr, m_main_window->getCurrentScale(), m_main_window->getCurrentScale());
 
 	if (!m_main_window->doDrawPostscript()) {
-		if (m_xpos * zoom_factor - leftx > main_width) return;
-		if ((m_xpos + (DEFAULT_BORDER + m_width)) * zoom_factor - leftx  < 0) return;
+		if ((m_xpos * zoom_factor - leftx) * scale > main_width) {cairo_identity_matrix(cr); return;}
+		if (((m_xpos  + (DEFAULT_BORDER + m_width)) * zoom_factor - leftx)  * scale < 0) {cairo_identity_matrix(cr); return;}
 	
 	
 		cairo_set_source_rgb (cr, 0.3, 0.3, 0.3);
@@ -259,73 +284,51 @@ void NedPage::draw(cairo_t *cr, double main_width, double main_height) {
 			DEFAULT_BORDER * zoom_factor,
 		 	m_width * zoom_factor, m_height * zoom_factor);
 		cairo_fill(cr);
+		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 		xp = 0.0;
 	}
-	GList *lptr;
-	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->draw(cr, m_page_number == 0 && first);
-		first = false;
-	}
 	if (m_page_number == 0) {
+		cairo_identity_matrix(cr);
 		yp = 0.0;
 		score_info = getMainWindow()->getScoreInfo();
 		if (m_main_window->getScoreInfo()->title != NULL) {
-			score_info_present = true;
-			cairo_new_path(cr);
-			cairo_select_font_face(cr, TITLE_FONT, TITLE_FONT_SLANT, TITLE_FONT_WEIGHT);
-			cairo_set_font_size(cr, TITLE_FONT_SIZE * zoom_factor);
-			cairo_move_to(cr, (xp + DEFAULT_BORDER + (m_width - score_info->title_extends.width) / 2.0) * zoom_factor -leftx,
-				(STAFF_TOP_DIST + score_info->title_extends.height) * zoom_factor - topy);
-			cairo_show_text(cr, score_info->title);
-			cairo_stroke(cr);
-			yp += score_info->title_extends.height + SCORE_INFO_EXTRA_DIST;
+			score_info->title->draw(cr, ((xp + DEFAULT_BORDER + (m_width - score_info->title_extends.width) / 2.0) * zoom_factor -leftx ) * scale,
+				(STAFF_TOP_DIST + score_info->title_extends.height) * zoom_factor - topy, zoom_factor, scale);
+			yp += score_info->title_extends.height + 4.0 * SCORE_INFO_EXTRA_DIST;
 		}
 		if (m_main_window->getScoreInfo()->subject != NULL) {
-			score_info_present = true;
-			cairo_new_path(cr);
-			cairo_select_font_face(cr, SUBJECT_FONT, SUBJECT_FONT_SLANT, SUBJECT_FONT_WEIGHT);
-			cairo_set_font_size(cr, SUBJECT_FONT_SIZE * zoom_factor);
-			cairo_move_to(cr, (xp + DEFAULT_BORDER + (m_width - score_info->subject_extends.width) / 2.0) * zoom_factor -leftx,
-				(STAFF_TOP_DIST + yp + score_info->subject_extends.height) * zoom_factor - topy);
-			cairo_show_text(cr, score_info->subject);
-			cairo_stroke(cr);
+			score_info->subject->draw(cr, ((xp + DEFAULT_BORDER + (m_width - score_info->subject_extends.width) / 2.0) * zoom_factor -leftx) * scale,
+				(STAFF_TOP_DIST + yp + score_info->subject_extends.height) * zoom_factor - topy, zoom_factor, scale);
 			yp += score_info->subject_extends.height;
 		}
 		if (m_main_window->getScoreInfo()->composer != NULL) {
-			score_info_present = true;
-			cairo_new_path(cr);
-			cairo_select_font_face(cr, COMPOSER_FONT, COMPOSER_FONT_SLANT, COMPOSER_FONT_WEIGHT);
-			cairo_set_font_size(cr, COMPOSER_FONT_SIZE * zoom_factor);
-			cairo_move_to(cr, (xp + DEFAULT_BORDER + (m_width - score_info->composer_extends.width - DEFAULT_BORDER - LEFT_RIGHT_BORDER)) * zoom_factor -leftx,
-				(STAFF_TOP_DIST + yp + score_info->composer_extends.height) * zoom_factor - topy);
-			cairo_show_text(cr, score_info->composer);
-			cairo_stroke(cr);
+			score_info->composer->draw(cr, ((xp + DEFAULT_BORDER + (m_width - score_info->composer_extends.width - DEFAULT_BORDER - LEFT_RIGHT_BORDER)) * zoom_factor -leftx) * scale,
+				(STAFF_TOP_DIST + yp + score_info->composer_extends.height) * zoom_factor - topy, zoom_factor, scale);
 			yp += score_info->composer_extends.height + SCORE_INFO_EXTRA_DIST;
 		}
 		if (m_main_window->getScoreInfo()->arranger != NULL) {
-			score_info_present = true;
-			cairo_new_path(cr);
-			cairo_select_font_face(cr, ARRANGER_FONT, ARRANGER_FONT_SLANT, ARRANGER_FONT_WEIGHT);
-			cairo_set_font_size(cr, ARRANGER_FONT_SIZE * zoom_factor);
-			cairo_move_to(cr, (xp + DEFAULT_BORDER + (m_width - score_info->arranger_extends.width - DEFAULT_BORDER - LEFT_RIGHT_BORDER)) * zoom_factor -leftx,
-				(STAFF_TOP_DIST + yp + score_info->arranger_extends.height) * zoom_factor - topy);
-			cairo_show_text(cr, score_info->arranger);
-			cairo_stroke(cr);
+			score_info->arranger->draw(cr, ((xp + DEFAULT_BORDER + (m_width - score_info->arranger_extends.width - DEFAULT_BORDER - LEFT_RIGHT_BORDER)) * zoom_factor -leftx) * scale,
+				(STAFF_TOP_DIST + yp + score_info->arranger_extends.height) * zoom_factor - topy, zoom_factor, scale);
 			yp += score_info->arranger_extends.height + SCORE_INFO_EXTRA_DIST;
 		}
 		if (m_main_window->getScoreInfo()->copyright != NULL) {
-			score_info_present = true;
-			cairo_new_path(cr);
-			cairo_select_font_face(cr, COPYRIGHT_FONT, COPYRIGHT_FONT_SLANT, COPYRIGHT_FONT_WEIGHT);
-			cairo_set_font_size(cr, COPYRIGHT_FONT_SIZE * zoom_factor);
-			cairo_move_to(cr, (xp + DEFAULT_BORDER + (m_width - score_info->copyright_extends.width - DEFAULT_BORDER - LEFT_RIGHT_BORDER)) * zoom_factor -leftx,
-				(STAFF_TOP_DIST + yp + score_info->copyright_extends.height) * zoom_factor - topy);
-			cairo_show_text(cr, score_info->copyright);
-			cairo_stroke(cr);
+			score_info->copyright->draw(cr, ((xp + DEFAULT_BORDER + (m_width - score_info->copyright_extends.width - DEFAULT_BORDER - LEFT_RIGHT_BORDER)) * zoom_factor -leftx) * scale,
+				(STAFF_TOP_DIST + yp + score_info->copyright_extends.height) * zoom_factor - topy, zoom_factor, scale);
 		}
-		if (score_info_present) {
-			cairo_set_font_face(cr, NedResource::getFontFace());
-			cairo_set_font_matrix(cr,  NedResource::getFontMatrix(getMainWindow()->getCurrentZoomLevel()));
+		cairo_scale(cr, m_main_window->getCurrentScale(), m_main_window->getCurrentScale());
+	}
+	GList *lptr;
+	freetext_or_lyrics_present = false;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->draw(cr, m_page_number == 0 && first, show_measure_numbers, &freetext_or_lyrics_present);
+		first = false;
+	}
+	cairo_identity_matrix(cr);
+	if (freetext_or_lyrics_present) {
+		first = true;
+		for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+			((NedSystem *) lptr->data)->drawTexts(cr, m_page_number == 0 && first, scale);
+			first = false;
 		}
 	}
 	cairo_show_page(cr);
@@ -336,29 +339,38 @@ int NedPage::getNumberOfStaffs() {
 	return ((NedSystem *) g_list_first(m_systems)->data)->getNumberOfStaffs();
 }
 
-double  NedPage::getContetXpos() {
+double  NedPage::getContentXpos() {
 	if (m_main_window->doDrawPostscript()) {
 		return LEFT_RIGHT_BORDER;
 	}
 	return m_xpos + LEFT_RIGHT_BORDER;
 }
 
-bool NedPage::reposit(int pass, NedCommandList *command_list /* = NULL */) {
+bool NedPage::isLastPage() {
+	return m_main_window->getLastPage() == this;
+}
+
+bool NedPage::isLastSystem(NedSystem *system) {
 	GList *lptr;
-	bool ch, changes = FALSE;
-	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		ch = ((NedSystem *) lptr->data)->reposit(pass, -1, command_list);
-		changes = changes || ch;
+
+	if (m_main_window->getLastPage() != this) return false;
+	if ((lptr = g_list_last(m_systems)) == NULL) {
+		NedResource::Abort("NedPage::isLastSystem");
 	}
-	return changes;
+	return (((NedSystem *) lptr->data) == system);
 }
 
 void NedPage::setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first) {
 	GList *lptr;
-	bool first_system = true;
-	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array, first && first_system);
-		first_system = false;
+	double indent = first ? m_main_window->getFirstSystemIndent() : m_main_window->get2ndSystemIndent();
+
+	if ((lptr = g_list_first(m_systems)) == NULL) return;
+	((NedSystem *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array, indent, first);
+
+	indent = m_main_window->get2ndSystemIndent();
+
+	for (lptr = g_list_next(lptr); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->setAndUpdateClefTypeAndKeySig(clef_and_key_array, indent, false);
 	}
 }
 
@@ -396,6 +408,14 @@ void NedPage::cutEmptyVoices() {
 	}
 }
 
+void NedPage::handleEmptyMeasures() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->handleEmptyMeasures();
+	}
+}
+
 bool NedPage::hasOnlyRests() {
 	GList *lptr;
 	
@@ -405,6 +425,23 @@ bool NedPage::hasOnlyRests() {
 	return true;
 }
 
+void NedPage::setAllUnpositioned() {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->m_is_positioned = false;
+	}
+}
+
+void NedPage::zoomFreeReplaceables(double zoom, double scale) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->zoomFreeReplaceables(zoom, scale);
+	}
+}
+
+
 void NedPage::testTies() {
 	GList *lptr;
 	
@@ -414,11 +451,22 @@ void NedPage::testTies() {
 }
 
 bool NedPage::exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map) {
+	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames) {
 	GList *lptr;
 	
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->exportLilyPond(fp, staff_nr, voice_nr, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map);
+		((NedSystem *) lptr->data)->exportLilyPond(fp, staff_nr, voice_nr, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map, with_break, chordnames);
+		if ((NedSystem *) lptr->data == last_system) return false;
+	}
+	return true;
+}
+
+bool NedPage::exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+	NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
+	GList *lptr;
+	
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->exportLilyChordnames(fp, staff_nr, last_line, midi_len, last_system, end_time, in_alternative, with_break);
 		if ((NedSystem *) lptr->data == last_system) return false;
 	}
 	return true;
@@ -441,6 +489,53 @@ void NedPage::handleStaffElements() {
 	}
 }
 
+void NedPage::setInternalPitches() {
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->setInternalPitches();
+	}
+}
+
+void NedPage::adjustAccidentals() {
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->adjustAccidentals();
+	}
+}
+
+void NedPage::transpose(int pitchdist, bool *staff_list, GList *selected_group) {
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->transpose(pitchdist, staff_list, selected_group);
+	}
+}
+
+void NedPage::hideRests(NedCommandList *command_list, bool unhide, int staff_nr, int voice_nr) {
+	GList *lptr;
+	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
+		((NedSystem *) lptr->data)->hideRests(command_list, unhide, staff_nr, voice_nr);
+	}
+}
+
+bool NedPage::shiftNotes(unsigned long long start_time, int linedist, NedSystem *start_system, int staff_number) {
+	GList *lptr;
+
+	if (start_system == 0) {
+		lptr = g_list_first(m_systems);
+	}
+	else {
+		if ((lptr = g_list_find(m_systems, start_system)) == NULL) {
+			NedResource::Abort("NedPage::shiftNotes");
+		}
+	}
+	for (; lptr; lptr = g_list_next(lptr)) {
+		if (!((NedSystem *) lptr->data)->shiftNotes(start_time, linedist, staff_number)) {
+			return false;
+		}
+	}
+	return true;
+}
+
 void NedPage::removeUnneededAccidentals() {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
@@ -462,11 +557,10 @@ void NedPage::prepareReplay() {
 void NedPage::reconfig_paper(double width, double height) {
 	GList *lptr;
 
-	m_xpos = m_page_number * (width + DEFAULT_BORDER);
+	m_xpos = m_page_number * (width + DEFAULT_BORDER) * m_main_window->getCurrentScale();
 	m_width = width; m_height = height;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
 		((NedSystem *) lptr->data)->setWidth(m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER));
-		((NedSystem *) lptr->data)->resetUntouched();
 	}
 
 }
@@ -480,20 +574,24 @@ void NedPage::getLinePoints(NedStaff *start_staff, NedStaff *end_staff, GList **
 
 
 	if (start_staff->getPage() != this) {
-		if ((lptr = g_list_find(next_page->m_systems, start_staff->getSystem())) == NULL) {
+		if ((next_page = m_main_window->getNextPage(this)) == NULL) {
 			NedResource::Abort("NedPage::getLinePoints: error 1");
 		}
+		if ((lptr = g_list_find(next_page->m_systems, start_staff->getSystem())) == NULL) {
+			NedResource::Abort("NedPage::getLinePoints: error 2");
+		}
 	}
-
-	if ((lptr = g_list_find(m_systems, start_staff->getSystem())) == NULL) {
-		NedResource::Abort("NedPage::getLinePoints: error 2");
+	else {
+		if ((lptr = g_list_find(m_systems, start_staff->getSystem())) == NULL) {
+			NedResource::Abort("NedPage::getLinePoints: error 3");
+		}
 	}
 
 
 	for (i = 0; i < 2; i++) {
 		for (; lptr; lptr = g_list_next(lptr)) {
 			if ((staff = ((NedSystem *) lptr->data)->getStaff(staff_nr)) == NULL) {
-				NedResource::Abort("NedPage::getLinePoints: error 3");
+				NedResource::Abort("NedPage::getLinePoints: error 4");
 			}
 			*staves = g_list_append(*staves, staff);
 			if (staff == end_staff) return;
@@ -501,16 +599,16 @@ void NedPage::getLinePoints(NedStaff *start_staff, NedStaff *end_staff, GList **
 	
 	
 		if ((next_page = end_staff->getPage()) == NULL) {
-			NedResource::Abort("NedPage::getLinePoints: error 4");
+			NedResource::Abort("NedPage::getLinePoints: error 6");
 		}
 		if ((lptr = g_list_first(next_page->m_systems)) == NULL) {
-			NedResource::Abort("NedPage::getLinePoints: error 5");
+			NedResource::Abort("NedPage::getLinePoints: error 7");
 		}
 	}
 
 	NedResource::DbgMsg(DBG_CRITICAL, "gesucht wurde in staff %d\n", staff_nr);
 
-	NedResource::Abort("NedPage::getLinePoints: error 6");
+	NedResource::Abort("NedPage::getLinePoints: error 8");
 }
 
 
@@ -527,7 +625,7 @@ void NedPage::savePage(FILE *fp) {
 	}
 }
 
-void NedPage::restorePage(FILE *fp, unsigned int start_measure_number) {
+void NedPage::restorePage(FILE *fp) {
 	int i, system_nr;
 	char buffer[128];
 	NedSystem *system;
@@ -554,16 +652,18 @@ void NedPage::restorePage(FILE *fp, unsigned int start_measure_number) {
 			NedResource::m_error_message = "SYSTEM number expected";
 			return;
 		}
+		/*
 		if (system_nr != i) {
 			NedResource::m_error_message = "bad system_nr";
 			return;
 		}
+		*/
 		if (!NedResource::readWord(fp, buffer) || strcmp(buffer, ":")) {
 			NedResource::m_error_message = ": expected";
 			return;
 		}
 
-		system = new NedSystem(this, system_pos, m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), system_nr, start_measure_number, FALSE);
+		system = new NedSystem(this, system_pos, m_width - 2 * (LEFT_RIGHT_BORDER - DEFAULT_BORDER), system_nr, 0 /* dummy is reset during "renumberMeasures()" at the end of "reposit()" */, FALSE);
 		m_systems = g_list_append(m_systems, system);
 
 		system->restoreSystem(fp);
@@ -591,19 +691,20 @@ bool NedPage::tryChangeLength(NedChordOrRest *chord_or_rest) {
 bool NedPage::trySelect(double x, double y, bool only_free_placeables /* = false */) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double leftx = m_main_window->getLeftX();
+	double current_scale = m_main_window->getCurrentScale();
 
 	double xl = X_POS_INVERS(x);
 
 
-	if (xl < 0.0 || xl > m_width) return FALSE;
+	if (xl < 0.0 || xl > m_width) return false;
 
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
 		if (((NedSystem *) lptr->data)->trySelect(x, y, only_free_placeables)) {
-			return TRUE;
+			return true;
 		}
 	}
-	return FALSE;
+	return false;
 }
 
 NedChordOrRest *NedPage::findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *ydist) {
@@ -704,10 +805,10 @@ void NedPage::testForPageBackwardTies(NedCommandList *command_list) {
 	((NedSystem *) g_list_first(m_systems)->data)->testForPageBackwardTies(command_list);
 }
 
-void NedPage::checkForElementsToSplit(NedCommandList *command_list) {
+void NedPage::checkForElementsToSplit(NedCommandList *command_list, int *measure_number) {
 	GList *lptr;
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-		((NedSystem *) lptr->data)->checkForElementsToSplit(command_list);
+		((NedSystem *) lptr->data)->checkForElementsToSplit(command_list, measure_number);
 	}
 }
 	
@@ -715,6 +816,7 @@ void NedPage::checkForElementsToSplit(NedCommandList *command_list) {
 
 NedStaff *NedPage::findStaff(double x, double y, NedMeasure **measure) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double current_scale = m_main_window->getCurrentScale();
 	double leftx = m_main_window->getLeftX();
 	NedStaff *staff;
 
@@ -742,6 +844,7 @@ double NedPage::convertX(double xpos) {
 
 bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*  = NULL */, bool force_rest /* = false */) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double current_scale = m_main_window->getCurrentScale();
 	double leftx = m_main_window->getLeftX();
 	NedChordOrRest *element, *element1, *element2;
 	double dummy, dummy0;
@@ -773,11 +876,11 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 		element = findNearestElement(-1, xx, yy, NULL, &dummy);
 		command_list = new NedCommandList(m_main_window);
 		if (m_main_window->getSpecialType() == TYPE_DYNAMIC) {
-			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedVolumeSign(m_main_window->getSpecialSubType()), element, xx, yy));
+			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedVolumeSign(m_main_window->getSpecialSubTypeInt()), element, xx, yy));
 		}
 		else {
-			kind = (m_main_window->getSpecialSubType() & 0xffff) * FACTOR;
-			tempo = ((m_main_window->getSpecialSubType() >> 16) & 0xffff);
+			kind = (m_main_window->getSpecialSubTypeInt() & 0xffff) * FACTOR;
+			tempo = ((m_main_window->getSpecialSubTypeInt() >> 16) & 0xffff);
 			command_list->addCommand(new NedInsertFreePlaceableCommand(new NedTempoSign(kind, tempo), element, xx, yy));
 		}
 		command_list->execute();
@@ -790,11 +893,52 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 		yy = Y_POS_INVERS(y);
 		element = findNearestElement(-1, xx, yy, NULL, &dummy);
 		command_list = new NedCommandList(m_main_window);
-		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSign(m_main_window->getSpecialSubType()), element, xx, yy));
+		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSign(m_main_window->getSpecialSubTypeInt()), element, xx, yy));
 		command_list->execute();
 		m_main_window->getCommandHistory()->addCommandList(command_list);
 		m_main_window->resetSpecialType();
 		return TRUE;
+	case TYPE_GUITAR_CHORD:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		yy = Y_POS_INVERS(y);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list = new NedCommandList(m_main_window);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeChord(m_main_window->getSpecialSubTypeChordInfo().chord_ptr,
+				m_main_window->getSpecialSubTypeChordInfo().chord_name_num, 
+				m_main_window->getSpecialSubTypeChordInfo().status, m_main_window), element, xx, yy));
+		command_list->execute();
+		m_main_window->setAllUnpositioned();
+		m_main_window->reposit(command_list);
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		return TRUE;
+	case TYPE_SPACER:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		yy = Y_POS_INVERS(y);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list = new NedCommandList(m_main_window);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedFreeSpacer(m_main_window), element, xx, yy));
+		command_list->execute();
+		m_main_window->setAllUnpositioned();
+		m_main_window->reposit(command_list);
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		return TRUE;
+	case TYPE_GUITAR_CHORDNAME:
+		topy = m_main_window->getTopY();
+		xx = X_POS_INVERS_PAGE_REL(x);
+		yy = Y_POS_INVERS(y);
+		element = findNearestElement(-1, xx, yy, NULL, &dummy);
+		command_list = new NedCommandList(m_main_window);
+		command_list->addCommand(new NedInsertFreePlaceableCommand(new NedChordName(m_main_window->getDrawingArea(),m_main_window->getSpecialSubTypeChordNameInfo().root_name, m_main_window->getSpecialSubTypeChordNameInfo().up_name,m_main_window->getSpecialSubTypeChordNameInfo().down_name), element, xx, yy));
+		command_list->execute();
+		m_main_window->setAllUnpositioned();
+		m_main_window->reposit(command_list);
+		m_main_window->getCommandHistory()->addCommandList(command_list);
+		m_main_window->resetSpecialType();
+		return TRUE;
 	case TYPE_FREE_TEXT:
 		topy = m_main_window->getTopY();
 		xx = X_POS_INVERS_PAGE_REL(x);
@@ -802,6 +946,7 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 		element = findNearestElement(-1, xx, yy, NULL, &dummy);
 		command_list = new NedCommandList(m_main_window);
 		command_list->addCommand(new NedInsertFreePlaceableCommand(m_main_window->m_freetext, element, xx, yy));
+		m_main_window->m_freetext->setZoom(zoom_factor, current_scale);
 		command_list->execute();
 		m_main_window->m_freetext = NULL;
 		m_main_window->getCommandHistory()->addCommandList(command_list);
@@ -842,16 +987,16 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 		element = element->getStaff()->findNearestElement(xx + LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
 		command_list->addCommand(new NedInsertFreePlaceableCommand(lp1 = new NedLinePoint(NULL, 1), element, xx + LINE_START_X_DIST, yy));
 		command_list->execute();
-		switch (m_main_window->getSpecialSubType()) {
+		switch (m_main_window->getSpecialSubTypeInt()) {
 			case LINE_CRESCENDO:
 			case LINE_DECRESCENDO:
-					new NedCrescendo(lp0, lp1, m_main_window->getSpecialSubType() == LINE_DECRESCENDO);
+					new NedCrescendo(lp0, lp1, m_main_window->getSpecialSubTypeInt() == LINE_DECRESCENDO);
 					break;
 			case LINE_OCTAVATION1:
 			case LINE_OCTAVATION_1:
 			case LINE_OCTAVATION2:
 			case LINE_OCTAVATION_2:
-					new NedOctavation(lp0, lp1, m_main_window->getSpecialSubType());
+					new NedOctavation(lp0, lp1, m_main_window->getSpecialSubTypeInt());
 					break;
 					
 		}
@@ -874,7 +1019,7 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 		element = element->getStaff()->findNearestElement(xx + 2 * LINE_START_X_DIST, yy, NULL, &dummy0, &dummy);
 		command_list->addCommand(new NedInsertFreePlaceableCommand(lp2 = new NedLinePoint(NULL, 1), element, xx + 2 * LINE_START_X_DIST, yy));
 		command_list->execute();
-		new NedAcceleration(lp0, lp1, lp2, m_main_window->getSpecialSubType() == LINE_RITARDANDO);
+		new NedAcceleration(lp0, lp1, lp2, m_main_window->getSpecialSubTypeInt() == LINE_RITARDANDO);
 		m_main_window->getCommandHistory()->addCommandList(command_list);
 		m_main_window->resetSpecialType();
 		m_main_window->m_selected_free_replaceable = lp0;
@@ -899,6 +1044,7 @@ bool NedPage::tryInsertOrErease(double x, double y, NedChordOrRest **newObj /*
 bool NedPage::findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff) {
 	double zoom_factor = m_main_window->getCurrentZoomFactor();
 	double leftx = m_main_window->getLeftX();
+	double current_scale = m_main_window->getCurrentScale();
 
 	double xl = X_POS_INVERS(x);
 
@@ -920,6 +1066,65 @@ bool NedPage::findLine(double x, double y, double *ypos, int *line, double *bott
 	return FALSE;
 }
 
+bool NedPage::findXposInOtherMeasure(guint keyval, double x, double y, double *newx, double *newy) {
+	double zoom_factor = m_main_window->getCurrentZoomFactor();
+	double current_scale = m_main_window->getCurrentScale();
+	double leftx = m_main_window->getLeftX();
+	NedChordOrRest *element;
+	NedStaff *staff;
+	NedSystem *system;
+
+	double xl = X_POS_INVERS(x);
+	*newy = y;
+
+	if (xl < 0.0 || xl > m_width) return false;
+	GList *lptr;
+	double mindist, d;
+	lptr = g_list_first(m_systems);
+	NedSystem *nearest_system = (NedSystem *) lptr->data;
+	mindist = nearest_system->computeMidDist(y);
+	for (lptr = g_list_next(lptr); lptr; lptr = g_list_next(lptr)) {
+		if ((d = ((NedSystem *) lptr->data)->computeMidDist(y)) < mindist) {
+			nearest_system = (NedSystem *) lptr->data;
+			mindist = d;
+		}
+	}
+	if (nearest_system->findElement(keyval, x, y, &element, &staff)) {
+		m_main_window->setVisible(element);
+		leftx = m_main_window->getLeftX();
+		*newx = X_POS_PAGE_REL(element->getXPos());
+		return true;
+	}
+	else {
+		if ((lptr = g_list_find(m_systems, nearest_system)) == NULL) {
+			NedResource::Abort("NedPage::findXposInOtherMeasure");
+		}
+		switch (keyval) {
+			case GDK_Right: lptr = g_list_next(lptr);
+					if (lptr == NULL) return false;
+					system = (NedSystem *) lptr->data;
+					lptr = system->getFirstChordOrRest(staff->getStaffNumber(), 0, 0, false, false);
+					if (lptr == NULL) return false;
+					element = (NedChordOrRest *) lptr->data;
+					m_main_window->setVisible(element);
+					*newx = X_POS_PAGE_REL(element->getXPos());
+					*newy = element->getStaff()->getRealYPosOfLine(-3);
+					return true;
+			case GDK_Left:  lptr = g_list_previous(lptr);
+					if (lptr == NULL) return false;
+					system = (NedSystem *) lptr->data;
+					lptr = system->getLastChordOrRest(staff->getStaffNumber(), 0, 0, false, false);
+					if (lptr == NULL) return false;
+					element = (NedChordOrRest *) lptr->data;
+					m_main_window->setVisible(element);
+					*newx = X_POS_PAGE_REL(element->getXPos());
+					*newy = element->getStaff()->getRealYPosOfLine(-3);
+					return true;
+		}
+	}
+	return false;
+}
+
 void NedPage::renumberSystems() {
 	int i;
 	GList *lptr;
@@ -937,16 +1142,6 @@ NedPage *NedPage::getPreviousPage() {
 	return m_main_window->getPreviousPage(this);
 }
 
-void NedPage::findLastTouchedSystem(NedSystem **start_of_systems) {
-	GList *lptr;
-
-	for (lptr = g_list_last(m_systems); lptr && *start_of_systems == NULL; lptr = g_list_previous(lptr)) {
-		if (!((NedSystem *) lptr->data)->isUntouched()) {
-			*start_of_systems = (NedSystem *) lptr->data;
-			return;
-		}
-	}
-}
 
 bool NedPage::placeStaffs(int pass, NedCommandList *command_list /* = NULL */) {
 	GList *lptr;
@@ -956,108 +1151,99 @@ bool NedPage::placeStaffs(int pass, NedCommandList *command_list /* = NULL */) {
 	bool changed = FALSE;
 
 	if (m_page_number == 0) {
-		offs += getMainWindow()->getFirstPageYOffs();
+		offs += getMainWindow()->getFirstPageYOffs() / getMainWindow()->getCurrentScale();
 	}
 
 	if (pass == 0) m_ping_pong = FALSE;
 	do_place_staffs(offs);
-	if (pass == 0 && m_system_diff < MIN_SYSTEM_Y_DIST) {
+	if (pass == 0 && m_system_diff < 0) {
 		if (command_list != NULL && g_list_length(m_systems) > 1) {
 			lptr = g_list_last(m_systems);
-			if (((NedSystem *) lptr->data)->isUntouched()) {
-				NedDeleteSystemCommand *command = new NedDeleteSystemCommand(this, &m_systems, lptr);
-				command_list->addCommand(command);
-				command->execute(); // the commandlist is not executed
+			if (m_main_window->getLastPage() && hasOnlyRests())  {
+				NedDeleteSystemCommand *del_system_command = new NedDeleteSystemCommand((NedSystem *) lptr->data);
+				del_system_command->execute();
+				command_list->addCommand(del_system_command);
 			}
 			else {
 				other_page = m_main_window->getNextPage(this, command_list);
-				NedMoveSystemCommand *move_system_command = new NedMoveSystemCommand(lptr, this, &m_systems, other_page, &(other_page->m_systems));
+				NedMoveSystemCommand *move_system_command = new NedMoveSystemCommand(this, (NedSystem *) lptr->data, other_page);
 				move_system_command->execute(); // the commandlist is not executed
 				command_list->addCommand(move_system_command);
-				//other_page->placeStaffs(command_list);
 			}
 			changed = TRUE;
 			do_place_staffs(offs);
 		}
 	}
-	if (pass == 1 && !isEmpty() && !m_ping_pong && command_list != NULL && m_system_diff > MAX_SYSTEM_Y_DIST) {
-		if ((other_page = m_main_window->getNextPage(this)) == NULL || other_page->isEmpty()) {
-			NedAppendSystemCommand *command = new NedAppendSystemCommand(this, m_width, &m_systems, getNumberOfLastMeasure() + 1);
-			command->execute(); // the commandlist is not executed ; performs placeStaffs();
-			if (m_system_diff < MIN_SYSTEM_Y_DIST) {
-				command->unexecute();
-				m_ping_pong = TRUE;
-				delete command;
-			}
-			else {
-				command_list->addCommand(command);
-			}
+#define MAX_Y_BORDER 1.0
+	if (pass == 1 && !isEmpty() && !m_ping_pong && command_list != NULL && m_system_diff > MAX_Y_BORDER) {
+		other_page = m_main_window->getNextPage(this);
+		if (other_page == NULL) return FALSE;
+		if ((lptr = g_list_first(other_page->m_systems)) == NULL) {
+			NedRemovePageCommand *rem_page_command = new NedRemovePageCommand(m_main_window, other_page);
+			rem_page_command->execute();
+			command_list->addCommand(rem_page_command);
+			return FALSE;
+		}
+		NedGetSystemFromNextPageCommand *get_system_from_next_page_command =
+			new NedGetSystemFromNextPageCommand(lptr, this, other_page);
+		get_system_from_next_page_command->execute(); // the commandlist is not executed; performs placeStaffs()
+		if (m_system_diff < MIN_SYSTEM_Y_DIST) {
+			m_ping_pong = TRUE;
+			get_system_from_next_page_command->unexecute();
+			delete get_system_from_next_page_command;
 		}
 		else {
-			if ((lptr = g_list_first(other_page->m_systems)) == NULL) {
-				NedResource::Abort("NedPage::placeStaffs");
-			}
-			NedGetSystemFromNextPageCommand *get_system_from_next_page_command =
-				new NedGetSystemFromNextPageCommand(lptr, this, other_page);
-			get_system_from_next_page_command->execute(); // the commandlist is not executed; performs placeStaffs()
-			if (m_system_diff < MIN_SYSTEM_Y_DIST) {
-				m_ping_pong = TRUE;
-				get_system_from_next_page_command->unexecute();
-				delete get_system_from_next_page_command;
-			}
-			else {
-				command_list->addCommand(get_system_from_next_page_command);
-				//other_page->placeStaffs(command_list);
-				changed = TRUE;
-				/*
-				if (other_page->isEmpty()) {
-					rem_last_page_cmd = new NedRemovePageCommand(m_main_window, other_page);
-					command_list->addCommand(rem_last_page_cmd);
-				}
-				*/
-			}
-			if (!isEmpty()) {
-				do_place_staffs(offs);
-			}
+			command_list->addCommand(get_system_from_next_page_command);
+			changed = TRUE;
+		}
+		if (!isEmpty()) {
+			do_place_staffs(offs);
 		}
 	}
 	return changed;
 }
 
+int NedPage::getSystemCount() {
+	return g_list_length(m_systems);
+}
+
 void NedPage::do_place_staffs(double offs) {
 	GList *lptr;
-	double staffpos = TOP_BOTTOM_BORDER;
-
-	int system_count = g_list_length(m_systems);
+	double staffpos = TOP_BOTTOM_BORDER + offs;
 
 	for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
 		staffpos = ((NedSystem *) lptr->data)->placeStaffs(staffpos);
 	}
-	m_system_diff = m_height - 2 * TOP_BOTTOM_BORDER - offs + STAFF_TOP_DIST - staffpos;
-	if (m_system_diff > 0.0 && g_list_length(m_systems) > 0 ) {
-		if (system_count > 1) {
-			m_system_diff /= system_count - 1;
-		}
-		for (lptr = g_list_first(m_systems); lptr; lptr = g_list_next(lptr)) {
-			((NedSystem *) lptr->data)->shiftY(offs);
-			offs += m_system_diff;
-		}
-	}
+	m_system_diff = m_height - offs - staffpos;
 }
 
-NedSystem *NedPage::getNextSystem(NedSystem *system, NedCommandList *command_list /* = NULL */ ) {
+NedSystem *NedPage::getNextSystem(NedSystem *system, NedCommandList *command_list /* = NULL */) {
 	GList *lptr;
+	NedAppendSystemCommand *app_sys_cmd;
+	NedSystem *system_on_next_page;
+	bool append_system_if_necessary = ((m_main_window->getLastPage() == this) && (system == ((NedSystem *) g_list_last(m_systems)->data)));
+
 
 	if (isEmpty()) return NULL;
 
-
 	if ((lptr = g_list_find(m_systems, system)) == NULL) {
 		NedResource::Abort("NedPage::getNextSystem");
 	}
 
 	if ((lptr = g_list_next(lptr)) == NULL) {
-		return m_main_window->getNextSystem(this, command_list);
+		system_on_next_page = m_main_window->getNextSystem(this, NULL);
+		if (system_on_next_page == NULL) {
+			if (command_list != NULL && append_system_if_necessary) {
+				app_sys_cmd = new NedAppendSystemCommand(this);
+				app_sys_cmd->execute();
+				command_list->addCommand(app_sys_cmd);
+				return app_sys_cmd->getSystem();
+			}
+			return NULL;
+		}
+		return system_on_next_page;
 	}
+
 	return (NedSystem *) lptr->data;
 }
 
@@ -1082,6 +1268,7 @@ NedSystem *NedPage::getFirstSystem() {
 	return (NedSystem *) lptr->data;
 }
 
+
 void NedPage::appendStaff(NedCommandList *command_list, int p_staff_nr /* = -1 */) {
 	GList *lptr;
 
