@@ -41,14 +41,16 @@ class NedPage {
 	public:
 		NedPage(NedMainWindow *main_window, double width, double height, int nr, unsigned int start_measure_number, bool start);
 		~NedPage();
+		NedPage *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedMainWindow *p_main_window, bool *staves);
+		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
 		NedSystem *appendSystem();
 		void removeSystem(NedSystem *system);
 		void insertSystem(NedSystem *system);
+		void appendSystem(NedCommandList *command_list);
 		bool isPageOverflow();
 		bool isEmpty() {return g_list_length(m_systems) == 0;}
 		bool hasOnlyRests();
-		void fill_up();
-		void draw(cairo_t *cr, double main_width, double main_height);
+		void draw(cairo_t *cr, bool show_measure_numbers, double main_width, double main_height);
 		double getTopPos();
 		double getLeftPos();
 		double getWidth() {return m_width;}
@@ -56,17 +58,25 @@ class NedPage {
 		double getXPos() {return m_xpos;}
 		unsigned int getNumberOfLastMeasure();
 		void setPageNumber(int num) {m_page_number = num; m_xpos = num * (m_width + DEFAULT_BORDER);}
-		double getContetXpos();
+		double getContentXpos();
 		double getContentWidth() {return m_width - 2 * LEFT_RIGHT_BORDER;}
 		NedMainWindow *getMainWindow() {return m_main_window;}
 		void appendElementsOfMeasureLength(part *parts, unsigned int meas_duration);
 		void handleStaffElements();
+		void adjustAccidentals();
+		void setInternalPitches();
+		void transpose(int pitchdist, bool *staff_list, GList *selected_group);
+		void hideRests(NedCommandList *command_list, bool unhide, int staff_nt, int voice_nr);
+		bool shiftNotes(unsigned long long start_time, int linedist, NedSystem *start_system, int staff_number);
 		void removeUnneededAccidentals();
 		bool trySelect(double x, double y, bool only_free_placeables = false);
 		NedChordOrRest *findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *ydist);
 		bool tryInsertOrErease(double x, double y, NedChordOrRest **newObj = NULL, bool force_rest = false);
+		void setAllUnpositioned();
+		void zoomFreeReplaceables(double zoom, double scale);
 		double convertX(double xpos);
 		bool findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff);
+		bool findXposInOtherMeasure(guint keyval, double x, double y, double *newx, double *newy);
 		int getPageNumber() {return m_page_number;}
 		bool findTimeOfMeasure(int meas_num, unsigned long long *meas_time, unsigned long long *system_offs);
 		void resetCollision();
@@ -84,15 +94,16 @@ class NedPage {
 		NedPage *getPreviousPage();
 		void renumberSystems();
 		bool isFirst(NedSystem *system);
+		bool isLastPage();
 		bool isLast(NedSystem *system);
+		bool isLastSystem(NedSystem *system);
+		NedSystem *getLastSystem();
 		bool tryChangeLength(NedChordOrRest *chord_or_rest);
 		bool placeStaffs(int pass, NedCommandList *command_list = NULL);
-		bool reposit(int pass, NedCommandList *command_list = NULL);
 		int getNumberOfStaffs();
-		void findLastTouchedSystem(NedSystem **start_of_systems);
 		NedStaff *findStaff(double x, double y, NedMeasure **measure);
 		void savePage(FILE *fp);
-		void restorePage(FILE *fp, unsigned int start_measure_number);
+		void restorePage(FILE *fp);
 		void prepareReplay();
 		void appendStaff(NedCommandList *command_list, int p_staff_nr = -1);
 		void removeLastStaff();
@@ -107,7 +118,7 @@ class NedPage {
 		void removeNotesFromTo(NedCommandList *command_list, GList *items, bool is_first_page, bool is_last_page,
 			NedSystem *min_sys, NedSystem *max_sys);
 		void testForPageBackwardTies(NedCommandList *command_list);
-		void checkForElementsToSplit(NedCommandList *command_list);
+		void checkForElementsToSplit(NedCommandList *command_list, int *measure_number);
 		void setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first);
 		GList **getSystemsList() {return &m_systems;}
 		bool find_staff_and_line(int x, int y, NedStaff **last_staff, int *last_line);
@@ -115,9 +126,13 @@ class NedPage {
 		void determineTempoInverse(NedChordOrRest *element, NedSystem* system, double *tempoinverse, bool *found);
 		void detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem **last_system, unsigned long long *end_time);
 		void cutEmptyVoices();
+		void handleEmptyMeasures();
 		void testTies();
+		int getSystemCount();
 		bool exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, NedSlur **lily_slur, unsigned int *lyrics_map);
+			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames);
+		bool exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+			NedSystem *last_system, unsigned long long end_time, bool *in_alernative, bool with_break);
 		bool exportLilyLyrics(FILE *fp, int staff_nr, int voice_nr, int line_nr, NedSystem *last_system, unsigned long long end_time, int *sil_count);
 	private:
 		void do_place_staffs(double offs);
@@ -129,6 +144,10 @@ class NedPage {
 		int m_page_number;
 		bool m_ping_pong;
 		NedMainWindow *m_main_window;
+
+	friend class NedDeleteSystemCommand;
+	friend class NedAppendSystemCommand;
+	friend class NedMoveSystemCommand;
 };
 
 #endif /* PAGE_H */
