@@ -28,13 +28,13 @@
 #include <cairo.h>
 #include <alsa/asoundlib.h>
 
-#define FONTFILE "nted/ntedfont.pfa"
+#define FONTFILE "ntedfont.pfa"
 
 #define WIDTH 1023
 #define HEIGHT 767
 
-#define ZOOMLEVELS 15
-#define MAXELEMENTS  21
+#define ZOOMLEVELS 20
+#define MAXELEMENTS  30
 #define DEFAULT_BORDER 0.2
 #define LEFT_RIGHT_BORDER 1.5
 #define TOP_BOTTOM_BORDER 1.5
@@ -53,12 +53,14 @@
 #define MEASURE_LINE_THICK 0.03
 #define BEAM_THICK 0.1
 
-#define STEM_HEIGHT (3.6 * LINE_DIST)
+#define STEM_DEFAULT_HEIGHT (3.4 * LINE_DIST)
 #define STEM_THICK 0.04
 #define HEAD_THICK (1.12 * LINE_DIST)
 #define HEAD_THICK_HALF (1.09 * HEAD_THICK)
 #define FLAG_DIST (0.87 * LINE_DIST)
 
+#define STEM_TOP_DIST 0.09
+
 #define HEAD_THICK_2 (HEAD_THICK / 2.0)
 #define HEAD_THICK_HALF_2 (HEAD_THICK_HALF / 2.0)
 
@@ -67,6 +69,11 @@
 #define HEAD_HIGHT_2 (HEAD_HIGHT / 2.0)
 #define HEAD_HIGHT_HALF_2 (HEAD_HIGHT_HALF / 2.0)
 
+
+#define BRACE_SPACE 0.05
+#define BRACKET_INDENT 0.2
+#define NESTED_BRACE_INDENT 0.18
+
 #define VOICE_COUNT 4
 
 #define COUNT_OF_PARALLEL_ELEMENTS 1024
@@ -101,7 +108,7 @@
 #define WHOLE_NOTE_Y_OFFS (-0.217 * LINE_DIST)
 #define HALF_NOTE_Y_OFFS (0.2 * LINE_DIST)
 
-#define ACCIDENTAL_BASE_DIST (2.6 * LINE_DIST)
+#define ACCIDENTAL_BASE_DIST (1.3 * LINE_DIST)
 #define MIN_SHIFT_X_DIFF (0.87 * LINE_DIST)
 
 #define START_ZOOM_LEVEL (ZOOMLEVELS / 2)
@@ -123,6 +130,9 @@
 #define TIE_Y_OFFS2 (2.174 * LINE_DIST)
 #define TIE_X_RIGHT_OVER (3.48 * LINE_DIST)
 #define TIE_X_OFFS 0.5
+#define D_TIE 0.1
+#define MAX_TIE_INCR 1.5
+#define MIN_TIE_INCR -1.5
 
 #define SORTING_NONE 0
 #define SORTING_LESS 1
@@ -133,43 +143,54 @@
 #define STEM_DIR_DOWN 2
 
 #define LYRICS_FONT_SIZE 0.4
-#define LYRICS_FONT "Helvetica"
-#define LYRICS_FONT_SLANT CAIRO_FONT_SLANT_NORMAL
-#define LYRICS_FONT_WEIGHT CAIRO_FONT_WEIGHT_NORMAL
+#define LYRICS_FONT "Sans"
+#define LYRICS_FONT_SLANT PANGO_STYLE_NORMAL
+#define LYRICS_FONT_WEIGHT PANGO_WEIGHT_NORMAL
+
 
 #define TITLE_FONT_SIZE 0.8
-#define TITLE_FONT "Times"
-#define TITLE_FONT_SLANT CAIRO_FONT_SLANT_NORMAL
-#define TITLE_FONT_WEIGHT CAIRO_FONT_WEIGHT_BOLD
+#define TITLE_FONT "Serif"
+#define TITLE_FONT_SLANT PANGO_STYLE_NORMAL
+#define TITLE_FONT_WEIGHT PANGO_WEIGHT_BOLD
 
 #define SUBJECT_FONT_SIZE 0.5
-#define SUBJECT_FONT "Times"
-#define SUBJECT_FONT_SLANT CAIRO_FONT_SLANT_ITALIC
-#define SUBJECT_FONT_WEIGHT CAIRO_FONT_WEIGHT_NORMAL
+#define SUBJECT_FONT "Serif"
+#define SUBJECT_FONT_SLANT PANGO_STYLE_ITALIC
+#define SUBJECT_FONT_WEIGHT PANGO_WEIGHT_NORMAL
 
 #define COMPOSER_FONT_SIZE 0.5
-#define COMPOSER_FONT "Helvetica"
-#define COMPOSER_FONT_SLANT CAIRO_FONT_SLANT_NORMAL
-#define COMPOSER_FONT_WEIGHT CAIRO_FONT_WEIGHT_NORMAL
+#define COMPOSER_FONT "Sans"
+#define COMPOSER_FONT_SLANT PANGO_STYLE_NORMAL
+#define COMPOSER_FONT_WEIGHT PANGO_WEIGHT_BOLD
 
 #define ARRANGER_FONT_SIZE 0.5
-#define ARRANGER_FONT "Helvetica"
-#define ARRANGER_FONT_SLANT CAIRO_FONT_SLANT_NORMAL
-#define ARRANGER_FONT_WEIGHT CAIRO_FONT_WEIGHT_NORMAL
+#define ARRANGER_FONT "Sans"
+#define ARRANGER_FONT_SLANT PANGO_STYLE_NORMAL
+#define ARRANGER_FONT_WEIGHT PANGO_WEIGHT_NORMAL
 
 #define COPYRIGHT_FONT_SIZE 0.4
-#define COPYRIGHT_FONT "Helvetica"
-#define COPYRIGHT_FONT_SLANT CAIRO_FONT_SLANT_NORMAL
-#define COPYRIGHT_FONT_WEIGHT CAIRO_FONT_WEIGHT_NORMAL
+#define COPYRIGHT_FONT "Sans"
+#define COPYRIGHT_FONT_SLANT PANGO_STYLE_NORMAL
+#define COPYRIGHT_FONT_WEIGHT PANGO_WEIGHT_NORMAL
+
+#define STAFF_NAME_FONT_SIZE 0.4
+#define STAFF_NAME_FONT "Serif"
+#define STAFF_NAME_FONT_SLANT PANGO_STYLE_NORMAL
+#define STAFF_NAME_FONT_WEIGHT PANGO_WEIGHT_NORMAL
+
+#define STAFF_SHORT_NAME_FONT_SIZE 0.4
+#define STAFF_SHORT_NAME_FONT "Serif"
+#define STAFF_SHORT_NAME_FONT_SLANT PANGO_STYLE_NORMAL
+#define STAFF_SHORT_NAME_FONT_WEIGHT PANGO_WEIGHT_NORMAL
 
-#define STAFF_NAME_FONT_SIZE 0.5
-#define STAF_NAME_FONT "Times"
-#define STAFF_NAME_FONT_SLANT CAIRO_FONT_SLANT_ITALIC
-#define STAFF_NAME_FONT_WEIGHT CAIRO_FONT_WEIGHT_BOLD
 #define SYSTEM_INTENT_SEC_SPACE 0.2
 
 #define SCORE_INFO_EXTRA_DIST 0.1
 
+#define SCALE_GRANULARITY 50
+#define SCALE_MIN 0.6
+#define SCALE_MAX 1.4
+
 #define TEXT_Y_OFFS (- 2.0 * LINE_DIST)
 #define LYRICS_HEIGHT 0.35
 #define MAX_LYRICS_LINES 5
@@ -196,12 +217,20 @@
 #define RECT_NOTE2   4
 #define TRIAG_NOTE1  5
 #define TRIAG_NOTE2  6
+#define CROSS_NOTE3  7
+#define CROSS_NOTE4  8
+#define TRIAG_NOTE3  9
+#define QUAD_NOTE2  10 
+#define GUITAR_NOTE_STEM  11 
+#define GUITAR_NOTE_NO_STEM  12 
 
 #define STROKEN_GRACE (NOTE_64/2)
 #define GRACE_8 (STROKEN_GRACE/2)
 #define GRACE_16 (GRACE_8/2)
 #define GRACE_NOTE_TIME NOTE_32
 
+#define LENGH_FACTOR GRACE_16
+
 #define GRACE_HEIGHT1 0.0
 #define GRACE_HEIGHT2 0.1
 #define GRACE_HEIGHT3 0.5
@@ -238,6 +267,11 @@
 #define STAT_STACCATISSIMO (1 << 17)
 #define STAT_FERMATA (1 << 18)
 #define STAT_ARPEGGIO (1 << 19)
+#define STAT_BREAK_BEAM (1 << 20)
+#define STAT_FORCE_BEAM (1 << 21)
+#define STAT_USER_STEMDIR (1 << 22)
+#define STAT_USER_HIDDEN_REST  (1 << 23)
+#define STAT_USER_PLACED_REST  (1 << 24)
 
 #define VIRTUAL_NATURAL_OFFS -111
 
@@ -252,11 +286,23 @@
 //#define EXTRA_SPACE (0.2 * LINE_DIST)
 #define WHOLE_EXTRA_SPACE (2.6 * LINE_DIST)
 
+// real measure numbers
+#define MEASURE_NUMBER_X_POS -0.45
+#define MEASURE_NUMBER_Y_POS -0.45
+#define MEASURE_NUMBER_SLANT CAIRO_FONT_SLANT_ITALIC
+#define MEASURE_NUMBER_WEIGHT CAIRO_FONT_WEIGHT_NORMAL
+#define MEASNUMBER_SIZE 0.35
+
+// for testing
 #define MEASURE_WEIGHT NOTE_4
 #define MEASURE_NUMBER_Y_DIST 0.1
 #define MEASURE_NUMBER_X_DIST 0.1
 #define MEASURE_NUMBER_SIZE 0.3
-#define TIME_SIG_WIDTH 0.3
+#define TIME_SIG_WIDTH 0.5
+
+#define MEASURE_SPREAD_STEP 0.1
+#define MEASURE_MAX_SPREAD 2.0
+#define MEASURE_MIN_SPREAD 0.4
 
 #define MIN_NEAR_DIST 0.4
 #define MIN_STAFF_BORDER_DIST 0.3
@@ -266,6 +312,8 @@
 
 #define MAX_NON_FULL_REPOSITS 15
 
+#define MAX_NOTES_PER_WITH_ACCIDENTALS_STAFF 50
+
 #define TREBLE_CLEF 0
 #define BASS_CLEF 1
 #define ALTO_CLEF 2
@@ -295,18 +343,20 @@
 #define MAX_RECENT_FILES 8
 
 #define START_EXTRA_SPACE (1.5 * LINE_DIST)
-#define MINIMUM_EXTRA_SPACE (0.8 * LINE_DIST)
-#define MAXIMUM_EXTRA_SPACE (1.8 * LINE_DIST)
+#define MINIMUM_EXTRA_SPACE (2.0 * LINE_DIST)
+#define MAXIMUM_EXTRA_SPACE (10.0 * LINE_DIST)
 
 #define MIN_SYSTEM_Y_DIST (1.2 * LINE_DIST)
 #define MAX_SYSTEM_Y_DIST (1.8 * LINE_DIST)
 
 #define REP_DOTS_X_DIST (0.8 * LINE_DIST)
+#define REP_DOTS_X_EXTRA_DIST (1.0 * LINE_DIST)
 #define REP_DOTS_Y1DIST (1.4 * LINE_DIST)
 #define REP_DOTS_Y2DIST (2.4 * LINE_DIST)
 #define REP_DOTS_RADIUS (0.2 * LINE_DIST)
 #define REPEAT_MEASURE_LINE_THICK 0.1
 #define REP_THICK_LINE_DIST (0.6 * LINE_DIST)
+#define REP_DOUBLE_LINE_DIST (0.4 * LINE_DIST)
 
 #define V1RED 0xf800
 #define V1GREEN 0x8b00
@@ -327,18 +377,36 @@
 #define DCOLOR(x) ((double) (x) / (double) (1 << 16))
 #define DSCOLOR(x) (0.6 * (double) (x) / (double) (1 << 16))
 
+#define HIDD_RED     255
+#define HIDD_GREEN     0
+#define HIDD_BLUE    255
+
 
 #define TOLERANCE 0.05
 #define MIN_REST_Y_VOICE_DIST 0.1
 
-#define MIN_VOICE_ELEMS_DIST 0.1
+#define MIN_VOICE_ELEMS_DIST 0.04
 
 #define MAX_OVER_UNDER_LINES 12
 
+#define OFFS_ARRAY_SIZE 115
+#define OFFS_ARRAY_HALF (OFFS_ARRAY_SIZE / 2)
+
 #define DBG_NO 0
 #define DBG_TESTING 1
 #define DBG_CRITICAL 2
 
+#define SEGNO_SIGN           (1 << 0)
+#define SEGNO_D_S_AL_CODA    (1 << 1)
+#define SEGNO_D_S_AL_FINE    (1 << 2)
+#define SEGNO_D_CAPO_AL_CODA (1 << 3)
+#define SEGNO_D_CAPO_AL_FINE (1 << 4)
+#define SEGNO_FINE           (1 << 5) 
+#define SEGNO_FINE2	     (1 << 6)  // "valid" fine (during replay)
+#define SEGNO_CODA	     (1 << 7)
+#define SEGNO_AL_CODA	     (1 << 8)
+#define SEGNO_AL_CODA2	     (1 << 9)  // "valid" al coda (during replay)
+
 class NedNote;
 class NedChordOrRest;
 class NedMainWindow;
@@ -360,11 +428,11 @@ struct addr_ref_str {
 };
 
 struct MidiEventStruct {
-	int channel;
+	unsigned short channel, segno_sign;
 	unsigned long long midi_time;
 	unsigned int type;
-#define PSEUDO_FLAG (0x1 << 17)
-#define PSEUDO2_FLAG (0x1 << 18)
+#define PSEUDO_FLAG (0x1 << 17) // MidiEventStruct will be deleted before reply starts
+#define PSEUDO2_FLAG (0x1 << 18) // MidiEventStruct is kept during replay
 #define PSEUDO_MASK (0xffff << 16)
 #define NOTE_MASK (0xffff)
 #define PSEUDO_EVENT_MEASURE (PSEUDO_FLAG | 0x1)
@@ -391,6 +459,7 @@ class NedResource {
 		static void initialize();
 		static void cleanup();
 		static void setWindow(NedMainWindow *ned_main_window) {m_main_window = ned_main_window;}
+		static NedMainWindow *getMainWindow() {return m_main_window;}
 		static char *getFontFileName() {return m_font_file_name;}
 		static cairo_scaled_font_t *getScaledFont(int zoom_level);
 		static double getStartZoomScale() {return m_zoom_factors[START_ZOOM_LEVEL];}
@@ -443,6 +512,7 @@ class NedResource {
 		static GdkPixbuf *m_alto_clef_icon;
 
 		static GdkPixbuf *m_delete_staff_icon;
+		static GdkPixbuf *m_delete_system_icon;
 
 		static GdkPixbuf *m_color_icon;
 
@@ -456,6 +526,12 @@ class NedResource {
 		static GdkPixbuf *m_drum4_icon;
 		static GdkPixbuf *m_drum5_icon;
 		static GdkPixbuf *m_drum6_icon;
+		static GdkPixbuf *m_drum7_icon;
+		static GdkPixbuf *m_drum8_icon;
+		static GdkPixbuf *m_drum9_icon;
+		static GdkPixbuf *m_drum10_icon;
+		static GdkPixbuf *m_guitar_note_icon;
+		static GdkPixbuf *m_guitar_note_no_stem_icon;
 
 		static GdkPixbuf *m_dcross_icon;
 		static GdkPixbuf *m_dflat_icon;
@@ -467,6 +543,9 @@ class NedResource {
 		static GdkPixbuf *m_sforzando_icon;
 		static GdkPixbuf *m_bow_up_icon;
 		static GdkPixbuf *m_bow_down_icon;
+		static GdkPixbuf *m_groupbeams_icon;
+		static GdkPixbuf *m_isolate_icon;
+		static GdkPixbuf *m_flipstem_icon;
 
 		static GdkPixbuf *m_brace_icon;
 		static GdkPixbuf *m_bracket_icon;
@@ -486,6 +565,9 @@ class NedResource {
 		static GdkPixbuf *m_signs;
 		static int m_signs_buf_width, m_signs_buf_height;
 
+		static GdkPixbuf *m_background1;
+		static GdkPixbuf *m_background2;
+
 
 		static NedBbox m_acc_bbox;
 		static double getFlagWidth() {return m_flag_width;}
@@ -511,15 +593,16 @@ class NedResource {
 		static void SeqReset();
 
 		static void setAccelRange(unsigned int offset, unsigned long long duration, double tempodiff);
-		static void addMidiEvent(int channel, unsigned int offset, unsigned int type, int data1, int data2,
-			NedNote *note);
-		static void addMidiCtrlEvent(int channel, unsigned int offset, unsigned int ctrl_param, int val);
+		static void addMidiEvent(unsigned short channel, unsigned int offset, unsigned int type, int data1, int data2,
+			unsigned short segno_sign, NedNote *note);
+		static void addMidiCtrlEvent(unsigned short channel, unsigned int offset, unsigned int ctrl_param, int val);
 		static void addTempoChange(unsigned int offset, double tempoinverse);
 		static void addTempoChangeAbs(unsigned long long midi_time, double tempoinverse);
 		static void addMeasureEntry(NedMeasure *measure);
 		static void addTempoChange(double tempoinverse);
 		static void addFermata(unsigned int offset);
 		static void correctFermataTempo();
+		static void handleSegnos();
 		static void prepareMidiEventList(NedMainWindow *main_window, double tempo_inverse);
 		static void startReplay(NedNote *startNote, double tempo_inverse);
 		static void collectTempoEvents(GList **tempo_chords);
@@ -541,6 +624,7 @@ class NedResource {
 		static void addLastFile(char *filename);
 		static char *m_recent_files[MAX_RECENT_FILES];
 		static void addToRecentFiles(char *fname);
+		static int getClefDist(int oldclef, int old_oct_offs, int newclef, int new_oct_offs);
 		static void playImmediately(int channel, int pgm, int pitch, int vol);
 		static bool alsaSequencerOk() {return m_sequ != NULL;}
 		static bool m_avoid_immadiate_play;
@@ -549,6 +633,12 @@ class NedResource {
 		static int m_width;
 		static int m_height;
 		static int m_tuplet_tab[14];
+		static GList *m_main_clip_board;
+		static int m_number_of_first_selected_staff, m_number_of_last_selected_staff;
+#ifdef YELP_PATH
+		static char *m_yelparg;
+#endif
+		static GList *m_main_windows;
 		static void appendTempFileName(char *fname);
 		static const char *m_last_folder;
 		static char *m_last_xml_dir;
@@ -574,8 +664,12 @@ class NedResource {
 		static void printMidiEvent(MidiEventStruct *ev);
 		static void removeAllPseudosEvents();
 		static GList *findListAtMeasure(int meas_number, NedMeasure **measure);
-		static void addTimeOffsetAfter(GList *end_ptr, unsigned long long extra_offs);
+		static void addTimeOffsetAfter(GList *end_ptr, unsigned long long time_diff);
+		static void subTimeOffsetAfter(GList *end_ptr, unsigned long long time_diff);
 		static GList *cloneAllMidiEventsBetween(GList *start_ptr, GList *end_ptr, unsigned long long extra_offs);
+		static void deleteAllBetween(GList *start_ptr, GList *end_ptr /* , unsigned long long time_diff */);
+		static void handleDSalFine(bool da_capo);
+		static void handleDSalCoda(bool da_capo);
 		static struct paper_info_struct m_paper_info[];
 		static gboolean stopImmediate(void *data);
 		static GList *midi_devices;
@@ -625,6 +719,8 @@ class NedResource {
 		static double m_accel_start_tempoinverse;
 		static double m_accel_end_tempoinverse;
 		static GList *m_fermata_list;
+		static bool m_ds_al_coda_available, m_ds_al_fine_available;
+		static bool m_d_capo_al_fine_available, m_d_capo_al_coda_available;
 };
 
 
