@@ -28,17 +28,18 @@
 #include "system.h"
 #include "mainwindow.h"
 #include "clipboard.h"
+#include "pangocairotext.h"
 
 #define DEFAULT_STAFF_DIST (4 * LINE_DIST)
 
-#define X_POS_PAGE_REL(p) ((getPage()->getContetXpos() + (p)) * zoom_factor - leftx)
+#define X_POS_PAGE_REL(p) ((getPage()->getContentXpos() + (p)) * zoom_factor - leftx)
 #define Y_POS_SYS_REL(p) (((p) + getSystem()->getYPos())  * zoom_factor - topy)
 
 #define X_PS_POS(p) ((DEFAULT_BORDER + LEFT_RIGHT_BORDER + (p)) * PS_ZOOM)
 #define Y_PS_POS_SYS_REL(p) ((height - ((p) + getSystem()->getYPos())) * PS_ZOOM)
 
-#define Y_POS_INVERS_STAFF_REL(p) ((topy + (p)) / zoom_factor - (getSystem()->getYPos() + getBottomPos()))
-#define X_POS_INVERS_STAFF_REL(p) (((p) + leftx) / zoom_factor - (getPage()->getContetXpos()))
+#define Y_POS_INVERS_STAFF_REL(p) ((topy + (p) / current_scale) / zoom_factor - (getSystem()->getYPos() + getBottomPos()))
+#define X_POS_INVERS_STAFF_REL(p) (((p) / current_scale + leftx) / zoom_factor - (getPage()->getContentXpos()))
 #define Y_POS_STAFF_REL(p) (((p) + getSystem()->getYPos() + getBottomPos()) * zoom_factor - topy)
 
 const char NedStaff::m_sharpPos[7][7] = {
@@ -48,8 +49,7 @@ const char NedStaff::m_flatPos[7][7] = {
 
 NedStaff::NedStaff(NedSystem *system, double ypos, double width, int nr, bool start) :
 m_staffelems(NULL), m_ypos(ypos), m_width(width), m_top_y_border(0), m_lyrics_lines(0), m_bottom_y_border(4 * LINE_DIST), m_staff_number(nr), m_clef_type(TREBLE_CLEF),
-m_clef_octave_shift(0), m_keysig(0), m_volume_changes(NULL), m_system(system),
-m_real_multiple_voices(FALSE) {
+m_clef_octave_shift(0), m_keysig(0), m_volume_changes(NULL), m_system(system) {
 	int i;
 
 	if (start) {
@@ -73,6 +73,31 @@ NedStaff::~NedStaff() {
 	}
 }
 
+NedStaff *NedStaff::clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedSystem *p_system, int staff_number) {
+	GList * lptr;
+	int i;
+	NedStaff *staff = new NedStaff(p_system, m_ypos, m_width, staff_number, false);
+
+	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
+		staff->m_staffelems = g_list_append(staff->m_staffelems, ((NedChordOrRest *) lptr->data)->clone(NULL, NULL, m_voices[0]));
+	}
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		staff->m_voices[i] = m_voices[i]->clone(addrlist, slurlist, staff);
+	}
+
+	return staff;
+}
+
+void NedStaff::adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->adjust_pointers(addrlist, slurlist);
+	}
+}
+
+
 void NedStaff::appendWholeRest(NedCommandList *command_list /* = NULL */) {
 	int i;
 
@@ -81,6 +106,7 @@ void NedStaff::appendWholeRest(NedCommandList *command_list /* = NULL */) {
 	}
 }
 
+
 void NedStaff::setUpbeatRests(unsigned int upbeat_inverse) {
 	int i;
 
@@ -139,18 +165,18 @@ void NedStaff::determineTempoInverse(NedChordOrRest *element, unsigned long long
 	}
 }
 
-GList *NedStaff::getFirstChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required) {
+GList *NedStaff::getFirstChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required, bool note_required) {
 	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
 		NedResource::Abort("NedStaff::getFirstChordOrRest");
 	}
-	return m_voices[voice_nr]->getFirstChordOrRest(lyrics_line, lyrics_required);
+	return m_voices[voice_nr]->getFirstChordOrRest(lyrics_line, lyrics_required, note_required);
 }
 
-GList *NedStaff::getLastChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required) {
+GList *NedStaff::getLastChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required, bool note_required) {
 	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
 		NedResource::Abort("NedStaff::getLastChordOrRest");
 	}
-	return m_voices[voice_nr]->getLastChordOrRest(lyrics_line, lyrics_required);
+	return m_voices[voice_nr]->getLastChordOrRest(lyrics_line, lyrics_required, note_required);
 }
 
 
@@ -287,7 +313,7 @@ GList *NedStaff::getStaffElements(unsigned long long midi_time) {
 }
 
 
-void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent) {
+void NedStaff::draw(cairo_t *cr, double botpos, double botpos2, bool first_page, double indent, bool *freetexts_or_lyrics_present) {
 	GList *lptr;
 	double leftx = getMainWindow()->getLeftX();
 	double topy = getMainWindow()->getTopY();
@@ -388,7 +414,7 @@ void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent)
 	sig_xpos = CLEF_SPACE + indent;
 	if (m_keysig > 0) {
 		cairo_new_path(cr);
-		glyph[0].index = BASE + 0;
+		glyph[0].index = BASE - 1;
 		for (i = 0; i < m_keysig; i++) {
 			glyph[0].x = X_POS_PAGE_REL(sig_xpos);
 			glyph[0].y = Y_POS_STAFF_REL(-m_sharpPos[m_clef_type][i] * LINE_DIST/2.0);
@@ -445,29 +471,26 @@ void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent)
 		cairo_stroke(cr);
 	}
 	
+	m_freetexts_or_lyrics_present = false;
 	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->draw(cr);
+		m_voices[i]->draw(cr, &m_freetexts_or_lyrics_present);
+
 	}
 	int mcount = getSystem()->getMeasureCount();
 	bool rep_line_needed = getMainWindow()->needsARepLine(getSystem()->getMeasureNr(0)->getMeasureNumber());
-	getSystem()->getMeasureNr(0)->draw(cr, SPECIAL_POSITION_START, m_system->m_system_start + REP_THICK_LINE_DIST, getPage()->getContetXpos(), getSystem()->getYPos(), m_ypos, botpos, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed);
+	getSystem()->getMeasureNr(0)->draw(cr, SPECIAL_POSITION_START, m_system->m_system_start + REP_THICK_LINE_DIST - indent, getPage()->getContentXpos() + indent, getSystem()->getYPos(), m_ypos, botpos, botpos2, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed, false);
 	for (i = 1; i <= mcount; i++) {
 		rep_line_needed = getMainWindow()->needsARepLine(getSystem()->getMeasureNr(i)->getMeasureNumber());
-		getSystem()->getMeasureNr(i)->draw(cr, (i == mcount) ? SPECIAL_POSITION_END : SPECIAL_POSITION_NONE, m_width, getPage()->getContetXpos(),  getSystem()->getYPos(), m_ypos, botpos, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed);
+		getSystem()->getMeasureNr(i)->draw(cr, (i == mcount) ? SPECIAL_POSITION_END : SPECIAL_POSITION_NONE, m_width, getPage()->getContentXpos(),  getSystem()->getYPos(), m_ypos, botpos, botpos2, getBottomPos(), leftx, topy, m_top_y_border, zoom_factor, zoom_level, rep_line_needed, getSystem()->getSystemNumber() == 4);
 	}
 	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
 		((NedChordOrRest *) lptr->data)->draw(cr, &dummy);
 	}
-	if (first_page && getMainWindow()->m_staff_contexts[m_staff_number].m_staff_name != NULL) {
-		cairo_new_path(cr);
-		cairo_select_font_face(cr, STAF_NAME_FONT, STAFF_NAME_FONT_SLANT, STAFF_NAME_FONT_WEIGHT);
-		cairo_set_font_size(cr, STAFF_NAME_FONT_SIZE * zoom_factor);
-		cairo_move_to(cr, X_POS_PAGE_REL(0), Y_POS_SYS_REL(m_ypos + 2 * LINE_DIST));
-		cairo_show_text(cr, getMainWindow()->m_staff_contexts[m_staff_number].m_staff_name);
-		cairo_set_font_face(cr, NedResource::getFontFace());
-		cairo_set_font_matrix(cr,  NedResource::getFontMatrix(getMainWindow()->getCurrentZoomLevel()));
-		cairo_stroke(cr);
+	if (getMainWindow()->m_staff_contexts[m_staff_number].m_staff_name != NULL ||
+		getMainWindow()->m_staff_contexts[m_staff_number].m_staff_short_name != NULL) {
+			m_freetexts_or_lyrics_present = true;
 	}
+	if (m_freetexts_or_lyrics_present) *freetexts_or_lyrics_present = true;
 #ifdef SHOW_MASURE_LENGTH
 	int measure_count = getSystem()->getMeasureCount();
 	cairo_new_path(cr);
@@ -482,10 +505,39 @@ void NedStaff::draw(cairo_t *cr, double botpos, bool first_page, double indent)
 #endif
 };
 
+void NedStaff::drawTexts(cairo_t *cr, bool first_page, double scale) {
+	int i;
+
+	if (!m_freetexts_or_lyrics_present) return;
+
+	double leftx = getMainWindow()->getLeftX();
+	double topy = getMainWindow()->getTopY();
+	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+
+	if (first_page) {
+		if (getMainWindow()->m_staff_contexts[m_staff_number].m_staff_name != NULL) {
+		getMainWindow()->m_staff_contexts[m_staff_number].m_staff_name->draw(cr,
+		((getPage()->getContentXpos()) * zoom_factor - leftx) * scale +
+		(getMainWindow()->getNettoIndent() - getMainWindow()->m_staff_contexts[m_staff_number].m_staff_name->getInternalWidth()),
+		 Y_POS_SYS_REL(m_ypos + 2 * LINE_DIST) * scale, zoom_factor, scale);
+		}
+	}
+	else if (getMainWindow()->m_staff_contexts[m_staff_number].m_staff_short_name != NULL) {
+		getMainWindow()->m_staff_contexts[m_staff_number].m_staff_short_name->draw(cr,
+		((getPage()->getContentXpos()) * zoom_factor  - leftx) * scale +
+		(getMainWindow()->get2ndNettoIndent() - getMainWindow()->m_staff_contexts[m_staff_number].m_staff_short_name->getInternalWidth()),
+		 Y_POS_SYS_REL(m_ypos + 2 * LINE_DIST) * scale, zoom_factor, scale);
+	}
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->drawTexts(cr, scale);
+	}
+};
+
 void NedStaff::convertStaffRel(double x, double y, double *x_trans, double *y_trans) {
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
 	double leftx = getMainWindow()->getLeftX();
 	double topy = getMainWindow()->getTopY();
+	double current_scale = getMainWindow()->getCurrentScale();
 	*y_trans = Y_POS_INVERS_STAFF_REL(y);
 	*x_trans = X_POS_INVERS_STAFF_REL(x);
 }
@@ -562,6 +614,22 @@ void NedStaff::cutEmptyVoices() {
 	}
 }
 
+void NedStaff::handleEmptyMeasures() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->handleEmptyMeasures();
+	}
+}
+
+void NedStaff::zoomFreeReplaceables(double zoom, double scale) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->zoomFreeReplaceables(zoom, scale);
+	}
+}
+
 void NedStaff::testTies() {
 	int i;
 
@@ -651,6 +719,7 @@ void NedStaff::handleStaffElements() {
 
 void NedStaff::appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_duration, int meas_num) {
 	int i;
+
 	for (i = 0; i < VOICE_COUNT; i++) {
 		m_voices[i]->appendElementsOfMeasureLength(part_ptr, meas_duration, meas_num);
 	}
@@ -664,6 +733,7 @@ bool NedStaff::tryInsertOrErease(double x, double y, NedChordOrRest **newObj, bo
 bool NedStaff::findLine(double x, double y, double *ypos, int *line, double *bottom) {
 	double topy = getMainWindow()->getTopY();
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
+	double current_scale = getMainWindow()->getCurrentScale();
 	double yl = Y_POS_INVERS_STAFF_REL(y);
 	double yoffs = (yl >= 0.0) ? LINE_DIST / 4.0 : -LINE_DIST / 4.0;
 	*line = (int) ((yl + yoffs) / (LINE_DIST/2.0));
@@ -673,6 +743,14 @@ bool NedStaff::findLine(double x, double y, double *ypos, int *line, double *bot
 	return TRUE;
 }
 
+bool NedStaff::findFirstElementeGreaterThan(double x, NedChordOrRest **element) {
+	return m_voices[0]->findFirstElementeGreaterThan(x, element);
+}
+
+bool NedStaff::findLastElementeLessThan(double x, NedChordOrRest **element) {
+	return m_voices[0]->findLastElementeLessThan(x, element);
+}
+
 bool NedStaff::hasStaffElem(unsigned long long miditime) {
 	GList *lptr;
 
@@ -686,10 +764,19 @@ bool NedStaff::hasStaffElem(unsigned long long miditime) {
 
 void NedStaff::computeBeams() {
 	int i;
+	//bool topFix = false, botFix = false;
 	m_top_y_border = 0; m_bottom_y_border = 4 * LINE_DIST;
 	m_lyrics_lines = 0;
 	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->computeBeams(&m_lyrics_lines, &m_top_y_border, &m_bottom_y_border);
+		m_voices[i]->computeBeams(&m_lyrics_lines, &m_top_y_border, &m_bottom_y_border, /* &topFix, &botFix, */ m_staff_number * VOICE_COUNT + i);
+		//if (topFix && botFix) break;
+	}
+}
+
+void NedStaff::computeTies() {
+	int i;
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->computeTies();
 	}
 }
 
@@ -742,7 +829,7 @@ void NedStaff::insertStaffElement(NedChordOrRest *elem, bool redraw /* = true */
 	m_staffelems = g_list_insert_sorted(m_staffelems, elem, (gint (*)(const void*, const void*)) NedChordOrRest::compareMidiTimes);
 	if (redraw) {
 		getMainWindow()->setAndUpdateClefTypeAndKeySig();
-		getMainWindow()->reposit(NULL, NULL, NULL, true);
+		getMainWindow()->reposit(NULL, NULL, NULL);
 		getMainWindow()->repaint();
 	}
 }
@@ -755,31 +842,15 @@ void NedStaff::removeStaffElement(NedChordOrRest *elem) {
 	}
 	m_staffelems = g_list_delete_link(m_staffelems, lptr);
 	getMainWindow()->setAndUpdateClefTypeAndKeySig();
-	getMainWindow()->reposit(NULL, NULL, NULL, true);
+	getMainWindow()->reposit(NULL, NULL, NULL);
 	getMainWindow()->repaint();
 }
 
-void NedStaff::resetPositionPointers(bool reset_multiple_flag /* = TRUE */, bool resetRestYPos /* = FALSE */) {
+void NedStaff::assignMidiTimes() {
 	int i;
 	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->resetPositionPointer(reset_multiple_flag, resetRestYPos);
+		m_voices[i]->assignMidiTimes();
 	}
-	if (reset_multiple_flag) {
-		m_real_multiple_voices = FALSE;
-	}
-	m_position_ptr = g_list_first(m_staffelems);
-}
-
-void NedStaff::placeCurrentElementAt(double xpos) {
-	if (m_position_ptr == NULL) {
-		NedResource::Abort("NedStaff::placeCurrentElementAt");
-	}
-	((NedChordOrRest *) m_position_ptr->data)->setXPos(xpos);
-	m_position_ptr = g_list_next(m_position_ptr);
-}
-
-void NedStaff::incrElementPointer() {
-	m_position_ptr = g_list_next(m_position_ptr);
 }
 
 void NedStaff::deleteStaffElements(bool unexecute, GList *elems, unsigned long long reftime, unsigned long duration) {
@@ -791,7 +862,8 @@ void NedStaff::deleteStaffElements(bool unexecute, GList *elems, unsigned long l
 			NedResource::Abort("NedStaff::deleteStaffElements");
 		}
 		if (!unexecute) {
-			measure = m_system->getMeasure(((NedChordOrRest *) lptr2->data)->getMidiTime());
+			//measure = m_system->getMeasure(((NedChordOrRest *) lptr2->data)->getMidiTime());
+			measure = ((NedChordOrRest *) lptr2->data)->m_measure;
 			if (measure == NULL) {
 				NedResource::DbgMsg(DBG_TESTING, "measure = 0 fuer time %llu (%llu)\n", ((NedChordOrRest *) lptr2->data)->getMidiTime(),
 					((NedChordOrRest *) lptr2->data)->getMidiTime() / NOTE_4); 
@@ -841,19 +913,6 @@ void NedStaff::searchForBeamGroups(unsigned int midi_start_time) {
 	}
 }
 
-void NedStaff::shiftX(int pc, double shift) {
-	int i;
-	if (m_position_ptr != NULL) {
-		if (((NedChordOrRest *) m_position_ptr->data)->m_position == pc) {
-			((NedChordOrRest *) m_position_ptr->data)->shiftX(shift);
-			m_position_ptr = g_list_next(m_position_ptr);
-		}
-	}
-	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->shiftX(pc, shift);
-	}
-}
-
 void NedStaff::collectChordsAndRests(NedClipBoard *board, unsigned long long midi_start, unsigned long long midi_end) {
 	int i;
 	GList *lptr;
@@ -924,6 +983,7 @@ void NedStaff::prepareReplay() {
 	GList *lptr;
 
 	m_volume_changes = NULL;
+
 	for (i = 0; i < VOICE_COUNT; i++) {
 		m_voices[i]->collectVolumeChanges(&m_volume_changes);
 	}
@@ -954,11 +1014,69 @@ void NedStaff::findAccidentals(char offs_array[115], NedMeasure *meas_info, unsi
 	}
 }
 
-void NedStaff::removeUnneededAccidentals() {
+void NedStaff::setInternalPitches() {
 	int i;
 
 	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->removeUnneededAccidentals();
+		m_voices[i]->setInternalPitches();
+	}
+}
+
+void NedStaff::adjustAccidentals() {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->adjustAccidentals();
+	}
+}
+
+
+void NedStaff::transpose(int pitchdist, GList *selected_group) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->setInternalPitch(pitchdist, selected_group);
+	}
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->adjustAccidentals(pitchdist, selected_group);
+	}
+	removeUnneededAccidentals(selected_group);
+}
+
+void NedStaff::hideRests(NedCommandList *command_list, bool unhide, int voice_nr) {
+	if (voice_nr < 0  || voice_nr >= VOICE_COUNT) {
+		NedResource::Abort("NedStaff::hideRests");
+	}
+
+	m_voices[voice_nr]->hideRests(command_list, unhide);
+}
+
+bool NedStaff::shiftNotes(unsigned long long start_time, int linedist) {
+	int i;
+	GList *lptr;
+	unsigned long long stop_time = 0;
+	bool stop_at_clef = false;
+
+	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
+		if (((NedChordOrRest *) lptr->data)->getType() != TYPE_CLEF) continue;
+		if (((NedChordOrRest *) lptr->data)->getMidiTime() <= start_time) continue;
+		stop_time = ((NedChordOrRest *) lptr->data)->getMidiTime();
+		stop_at_clef = true;
+		break;
+	}
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->shiftNotes(start_time, linedist, stop_time, stop_at_clef);
+	}
+
+	return !stop_at_clef;
+}
+
+void NedStaff::removeUnneededAccidentals(GList *selected_group /* = NULL */) {
+	int i;
+
+	for (i = 0; i < VOICE_COUNT; i++) {
+		m_voices[i]->removeUnneededAccidentals(selected_group);
 	}
 }
 
@@ -974,11 +1092,16 @@ bool NedStaff::detectVoices(unsigned int *voice_mask, unsigned long long *e_time
 }
 
 void NedStaff::exportLilyPond(FILE *fp, int voice_nr, int *last_line, unsigned int *midi_len,
-		bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map) {
+		bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames) {
 	if (voice_nr < 0 || voice_nr >= VOICE_COUNT) {
 		NedResource::Abort("NedStaff::exportLilyPond");
 	}
-	m_voices[voice_nr]->exportLilyPond(fp, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map);
+	m_voices[voice_nr]->exportLilyPond(fp, last_line, midi_len, last_system, end_time, in_alternative, lily_slur, lyrics_map, with_break, chordnames);
+}
+
+void NedStaff::exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len,
+		bool last_system, unsigned long long end_time, bool *in_alternative, bool with_break) {
+	m_voices[0]->exportLilyChordnames(fp, last_line, midi_len, last_system, end_time, in_alternative, with_break);
 }
 
 void NedStaff::exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count) {
@@ -989,78 +1112,25 @@ void NedStaff::exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int li
 }
 	
 
-
-
-void NedStaff::collectElmentsAfter(unsigned long long midi_time, VoiceInfoContainer *containers, int *num_containers) {
-	int i;
-	for (; m_position_ptr && ((NedChordOrRest *) m_position_ptr->data)->getMidiTime() < midi_time; m_position_ptr = g_list_next(m_position_ptr));
-	if (m_position_ptr != NULL) {
-		if (((NedChordOrRest *) m_position_ptr->data)->getMidiTime() >= midi_time) {
-			containers[*num_containers].element = (NedChordOrRest *) m_position_ptr->data;
-			containers[*num_containers].voice = NULL;
-			containers[*num_containers].staff = this;
-			(*num_containers)++;
+int NedStaff::assignElementsToMeasures(NedMeasure *measures, int staff_offs, bool use_upbeat) {
+	int mesure_count = 0, mc;
+	GList *lptr;
+	int curr_measure = 0;
+	for (lptr = g_list_first(m_staffelems); lptr; lptr = g_list_next(lptr)) {
+		while (((NedChordOrRest *) lptr->data)->getMidiTime() > measures[curr_measure].midi_end) {
+			curr_measure++;
 		}
+		if (((NedChordOrRest *) lptr->data)->getMidiTime() > measures[curr_measure].midi_end) continue;
+		measures[curr_measure].m_position_array.insertStaffElement(
+			(NedChordOrRest *) lptr->data, staff_offs / VOICE_COUNT, measures[curr_measure].getMeasureNumber());
+		((NedChordOrRest *) lptr->data)->m_measure = &(measures[curr_measure]);
+		if (curr_measure > mesure_count) mesure_count = curr_measure;
 	}
-	for (i = 0; i < VOICE_COUNT; i++) {
-		if (*num_containers >= COUNT_OF_PARALLEL_ELEMENTS) {
-			NedResource::Abort("NedStaff::collectlementsAfter");
-		}
-		if ((containers[*num_containers].element  = m_voices[i]->getNextElementAfter(midi_time)) != NULL) {
-			containers[*num_containers].voice = m_voices[i];
-			containers[*num_containers].staff = this;
-			if (i > 0 && !containers[*num_containers].element->isRest()) {
-				m_real_multiple_voices = TRUE;
-			}
-			(*num_containers)++;
-		}
+	for (int i = 0; i < VOICE_COUNT; i++) {
+		mc = m_voices[i]->assignElementsToMeasures(measures, staff_offs + i, use_upbeat);
+		if (mc > mesure_count) mesure_count = mc;
 	}
-}
-
-
-void NedStaff::stemDirDecicion(int position_counter) {
-	int pc;
-	int i, j;
-	unsigned int bestvariant_idx, has_chords = 0;
-	NedChordOrRest *chords[VOICE_COUNT];
-	double costs[1 << VOICE_COUNT];
-	int numchords;
-	unsigned int variant;
-
-	for (i = 0; i < 1 << VOICE_COUNT; costs[i++] = 0.0);
-	resetPositionPointers(FALSE);
-
-	for (pc = 0; pc < position_counter; pc++) {
-		numchords = 0;
-		for (i = 0; i < VOICE_COUNT; i++) {
-			m_voices[i]->getElementAtPosition(pc, false, chords, &numchords, &has_chords, 1 << i);
-		}
-		if (numchords > 0) {
-			for (variant = 0; variant < (1 << VOICE_COUNT); variant++) {
-				for (i = 0; i < numchords - 1; i++) {
-					for (j = i + 1; j < numchords; j++) {
-						costs[variant] += chords[i]->computeStemDist(variant & (1 << i),
-							chords[j], variant & (1 << j));
-					}
-				}
-			}
-		}
-	}
-	bestvariant_idx = 0;
-	for (variant = 0; variant < (1 << VOICE_COUNT); variant++) {
-		if (((has_chords & variant) == has_chords) || (((~variant) & has_chords) == has_chords)) {
-			costs[variant] = 10000000.0;
-		}
-	}
-	for (variant = 0; variant < (1 << VOICE_COUNT); variant++) {
-		if (costs[variant] < costs[bestvariant_idx]) {
-			bestvariant_idx = variant;
-		}
-	}
-	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->setStemDir((bestvariant_idx & (1 << i)) ? STEM_DIR_UP : STEM_DIR_DOWN);
-	}
-		
+	return mesure_count;
 }
 
 double NedStaff::computeMidDist(double y) {
@@ -1068,71 +1138,13 @@ double NedStaff::computeMidDist(double y) {
 	double topy = getMainWindow()->getTopY();
 	double zoom_factor = getMainWindow()->getCurrentZoomFactor();
 
+	y /= getMainWindow()->getCurrentScale();
+
 	d = Y_POS_SYS_REL(m_ypos + 3 * LINE_DIST / 2.0) - y;
 	if (d < 0) return -d;
 	return d;
 }
 
-void NedStaff::placeRests(int position_counter) {
-	int pc;
-	int i, j;
-	double shift;
-	NedChordOrRest *rests[VOICE_COUNT];
-	int numrests;
-	unsigned int dummy;
-
-	resetPositionPointers(FALSE, TRUE);
-
-	for (pc = 0; pc < position_counter; pc++) {
-		numrests = 0;
-		for (i = 0; i < VOICE_COUNT; i++) {
-			m_voices[i]->getElementAtPosition(pc, true, rests, &numrests, &dummy, dummy);
-		}
-		if (numrests > 0) {
-			for (i = 0; i < numrests - 1; i++) {
-				if (rests[i]->isRest() && rests[i]->isHidden()) continue;
-				for (j = i+1; j < numrests; j++) {
-					if (!rests[j]->isRest()) continue;
-					if (rests[j]->isHidden()) continue;
-					shift = rests[i]->compute_y_shift(rests[j], rests[j]->getVoice()->getStemDir());
-					if (shift > 0.0) {
-						if (rests[j]->getVoice()->getStemDir() == STEM_DIR_DOWN) {
-							rests[j]->shiftY(shift);
-						}
-						else {
-							rests[j]->shiftY(-shift);
-						}
-					}
-				}
-			}
-			for (i = numrests - 1; i >= 1; i--) {
-				if (rests[i]->isRest() && rests[i]->isHidden()) continue;
-				for (j = i-1; j >= 0; j--) {
-					if (!rests[j]->isRest()) continue;
-					if (rests[j]->isHidden()) continue;
-					shift = rests[i]->compute_y_shift(rests[j], rests[j]->getVoice()->getStemDir());
-					if (shift > 0.0) {
-						if (rests[j]->getVoice()->getStemDir() == STEM_DIR_DOWN) {
-							rests[j]->shiftY(shift);
-						}
-						else {
-							rests[j]->shiftY(-shift);
-						}
-					}
-				}
-			}
-		}
-	}
-}
-
-void NedStaff::changeDirs() {
-	int i;
-
-	for (i = 0; i < VOICE_COUNT; i++) {
-		m_voices[i]->changeDirs();
-	}
-}
-
 void NedStaff::saveStaff(FILE *fp) {
 	int i;
 	GList *lptr;
