@@ -44,7 +44,10 @@ class NedStaff {
 	public:
 		NedStaff(NedSystem *system, double ypos, double width, int nr, bool start);
 		~NedStaff();
-		void draw(cairo_t *cr, double botpos, bool first_page, double indent);
+		NedStaff *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedSystem *p_system, int staff_number);
+		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
+		void draw(cairo_t *cr, double botpos, double botpos2, bool first_page, double indent, bool *freetexts_or_lyrics_present);
+		void drawTexts(cairo_t *cr, bool first_page, double scale);
 		NedMainWindow *getMainWindow();
 		NedPage *getPage();
 		NedSystem *getSystem() {return m_system;}
@@ -62,6 +65,8 @@ class NedStaff {
 		bool tryInsertOrErease (double x, double y, NedChordOrRest **newObj, bool force_rest);
 		bool tryErease (double x, double y);
 		bool findLine(double x, double y, double *ypos, int *line, double *bottom);
+		bool findFirstElementeGreaterThan(double x, NedChordOrRest **element);
+		bool findLastElementeLessThan(double x, NedChordOrRest **element);
 		double getBottomPos();
 		double getMidPos();
 		double getTopPos() {return m_ypos;}
@@ -72,17 +77,17 @@ class NedStaff {
 		void empty();
 		bool hasOnlyRests();
 		void cutEmptyVoices();
+		void handleEmptyMeasures();
+		void zoomFreeReplaceables(double zoom, double scale);
 		void computeBeams();
 		void computeTuplets();
+		void computeTies();
 		unsigned int getStaffDurationTotal();
-		void resetPositionPointers(bool reset_multiple_flag = TRUE, bool resetRestYPos = FALSE);
+		void assignMidiTimes();
 		double computeMidDist(double y);
-		void changeDirs();
-		void placeRests(int position_counter);
 		void collectChordsAndRests(NedClipBoard *board, unsigned long long midi_start, unsigned long long midi_end);
 		void collectDestinationVoices(NedClipBoard *board);
-		void collectElmentsAfter(unsigned long long midi_time, VoiceInfoContainer *containers, int *num_containers);
-		void shiftX(int pc, double shift);
+		int assignElementsToMeasures(NedMeasure *measures, int staff_offs, bool use_upbeat);
 		bool tryChangeLength(NedChordOrRest *chord_or_rest);
 		void searchForBeamGroups(unsigned int midi_start_time);
 		double getTopYBorder() {return m_top_y_border;}
@@ -95,14 +100,17 @@ class NedStaff {
 		int computeVolume(unsigned long long midi_time);
 		void prepareReplay();
 		void findAccidentals(char offs_array[115], NedMeasure *meas_info, unsigned long long midi_time);
-		void removeUnneededAccidentals();
+		void setInternalPitches();
+		void adjustAccidentals();
+		void transpose(int pitchdist, GList *selected_group);
+		void hideRests(NedCommandList *command_list, bool unhide, int voice_nr);
+		bool shiftNotes(unsigned long long start_time, int linedist);
+		void removeUnneededAccidentals(GList *selected_group = NULL);
 		void appendWholeRest(NedCommandList *command_list = NULL);
-		void stemDirDecicion(int position_counter);
 		void getCurrentClefAndKeysig(unsigned long long miditime, int *clef, int *keysig, int *octave_shift);
 		void appendAppropriateWholes(int start_measure);
 		void setUpbeatRests(unsigned int upbeat_inverse);
 		void testTies();
-		bool hasReallyMultipleVoices() {return m_real_multiple_voices;}
 		void collectSelectionRectangleElements(double xp, double yp, NedBbox *sel_rect, GList **sel_group,
 			bool is_first_selected, bool is_last_selected);
 		void findSelectedStaffs(double yp, NedBbox *sel_rect, int *number_of_first_selected_staff, int *number_of_last_selected_staff);
@@ -116,8 +124,6 @@ class NedStaff {
 		void insertBlocks(NedCommandList *command_list, int blockcount, unsigned long long midi_time);
 		void testForPageBackwardTies(NedCommandList *command_list);
 		void checkForElementsToSplit(NedCommandList *command_list);
-		void placeCurrentElementAt(double xpos);
-		void incrElementPointer();
 		void insertStaffElement(NedChordOrRest *elem, bool redraw = true);
 		void removeStaffElement(NedChordOrRest *elem);
 		void deleteStaffElements(bool unexecute, GList *elems, unsigned long long reftime, unsigned long duration);
@@ -127,19 +133,20 @@ class NedStaff {
 		void setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first);
 		double getSystemEnd();
 		double getSystemStart();
-		GList *getFirstChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required);
-		GList *getLastChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required);
+		GList *getFirstChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required, bool note_required);
+		GList *getLastChordOrRest(int voice_nr, int lyrics_line, bool lyrics_required, bool note_required);
 		void determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr);
 		void determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found);
 		bool detectVoices(unsigned int *voice_mask, unsigned long long *e_time);
 		void exportLilyPond(FILE *fp, int voice_nr, int *last_line, unsigned int *midi_len,
-				bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map);
+				bool last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames);
+		void exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len,
+				bool last_system, unsigned long long end_time, bool *in_alternative, bool with_break);
 		void exportLilyLyrics(FILE *fp, bool last_system, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count);
 		GList *getStaffElements(unsigned long long midi_time);
 		static const char m_sharpPos[7][7];
 		static const char m_flatPos[7][7];
 	private:
-		GList *m_position_ptr;
 		GList *m_staffelems;
 		NedVoice *m_voices[VOICE_COUNT];
 		double m_ypos;
@@ -151,9 +158,9 @@ class NedStaff {
 		int m_clef_type;
 		int m_clef_octave_shift;
 		int m_keysig;
+		bool m_freetexts_or_lyrics_present;
 		GList *m_volume_changes;
 		NedSystem *m_system;
-		bool m_real_multiple_voices;
 };
 
 #endif /* STAFF_H */
