@@ -39,7 +39,7 @@ class NedSlur;
 class part;
 
 #define MAX_POSITIONS 4096
-#define MAX_MEASURES 40
+#define MAX_MEASURES 80
 //#define WITH_WEIGHTS
 
 
@@ -47,41 +47,54 @@ class NedSystem {
 	public:
 		NedSystem(NedPage *page, double ypos, double width, int nr, unsigned int start_measure_number, bool start);
 		~NedSystem();
-		void draw(cairo_t *cr, bool first_page);
+		NedSystem *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedPage *p_page, bool *staves);
+		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
+		void draw(cairo_t *cr, bool first_page, bool show_measure_numbers, bool *freetexts_or_lyrics_present);
+		void drawTexts(cairo_t *cr, bool first_page, double scale);
 		NedMainWindow *getMainWindow();
 		NedPage *getPage() {return m_page;}
-		void resetPositionPointers();
+		void assignMidiTimes();
 		void prepareForImport();
-		bool testLineOverflow(double indent);
+		bool testLineOverflow();
 		void appendElementsOfMeasureLength(part *parts, unsigned int meas_duration, int meas_num);
 		void removeLastImported();
 		bool handleImportedTuplets();
 		void handleStaffElements();
+		void transpose(int pitchdist, bool *staff_list, GList *selected_group);
+		void hideRests(NedCommandList *command_list, bool unhide, int staff_nr, int voice_nr);
+		bool shiftNotes(unsigned long long start_time, int linedist, int staff_number);
+		void adjustAccidentals();
+		void setInternalPitches();
 		void removeUnneededAccidentals();
 		bool trySelect (double x, double y, bool only_free_placeables);
 		NedChordOrRest *findNearestElement(int staff_nr, double x, double y, NedStaff *oldstaff, double *mindist, double *ydist);
 		NedChordOrRest *getFirstObjectInNextSystem(int staff_nr);
 		bool tryInsertOrErease (double x, double y, NedChordOrRest **newObj, bool force_rest);
 		bool findLine(double x, double y, double *ypos, int *line, double *bottom, NedStaff **staff);
+		bool findElement(guint keyval, double x, double y, NedChordOrRest **element, NedStaff **staff);
 		double getHeight();
 		double getYPos() {return m_ypos;}
+		double getBottomPos();
 		int getSystemNumber() {return m_system_number;}
 		void setSystemNumber(int number) {m_system_number = number;}
 		void changePageInfo(NedPage *page) {m_page = page;}
 		int getNumberOfStaffs();
 		void appendStaff(NedCommandList *command_list, int p_staff_nr = -1);
 		void cutEmptyVoices();
+		void handleEmptyMeasures();
+		void zoomFreeReplaceables(double zoom, double scale);
 		void testTies();
 		void removeLastStaff();
 		void deleteStaff(int staff_number);
 		void restoreStaff(int staff_number);
 		void shiftStaff(int staff_number, unsigned int position);
-		bool reposit(int pass, int measure_number, NedCommandList *command_list = NULL, NedSystem **next_system  = NULL);
+		bool reposit(int measure_number, NedCommandList *command_list = NULL, NedSystem **next_system  = NULL);
 		void renumberMeasures(int *measure_number, GList *special_measures, bool force);
+		void setNewMeasureSpread(double spread);
 		int getNumberOfFirstMeasure();
 		int getNumberOfLastMeasure();
 		unsigned long long getSystemEndTime();
-		void checkForElementsToSplit(NedCommandList *command_list);
+		void checkForElementsToSplit(NedCommandList *command_list, int *measure_number);
 		void collectDestinationVoices(NedClipBoard *board);
 		bool collectFirstMeasure(NedClipBoard *board);
 		void findSelectedSystems(NedBbox *sel_rect, int *number_of_first_selected_staff, 
@@ -100,8 +113,9 @@ class NedSystem {
 		void insertBlocks(NedCommandList *command_list, int blockcount, unsigned long long midi_time);
 		void testForPageBackwardTies(NedCommandList *command_list);
 		double computeMidDist(double y);
-		NedMeasure *getMeasure(unsigned int midi_time);
+		NedMeasure *getMeasure(unsigned long long  midi_time);
 		NedMeasure *getMeasureNr(int m_number) {return &(m_measures[m_number]);}
+		NedMeasure *getLastMeasure() {return &(m_measures[m_measure_count]);}
 		bool findTimeOfMeasure(int meas_num, unsigned long long *meas_time);
 		int getMeasureCount() {return m_measure_count;}
 		bool tryChangeLength(NedChordOrRest *chord_or_rest);
@@ -114,45 +128,44 @@ class NedSystem {
 		void restoreSystem(FILE *fp);
 		void prepareReplay();
 		unsigned int getWholeMidiLength();
-		void resetUntouched() {m_isUntouched = FALSE;}
-		bool isUntouched() {return m_isUntouched;}
 		void setWidth(double w);
 		void do_remove_staff(NedStaff *staff);
 		void do_append_staff(NedStaff *staff);
-		void setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, bool first);
-		void fill_up(double indent, NedCommandList *command_list);
-		GList *getFirstChordOrRest(int staff_nr, int voice_nr, int lyrics_line, bool lyrics_required);
-		GList *getLastChordOrRest(int staff_nr, int voice_nr, int lyrics_line, bool lyrics_required);
+		void setAndUpdateClefTypeAndKeySig(int *clef_and_key_array, double indent, bool first);
+		void fill_up(NedCommandList *command_list);
+		GList *getFirstChordOrRest(int staff_nr, int voice_nr, int lyrics_line, bool lyrics_required, bool note_required);
+		GList *getLastChordOrRest(int staff_nr, int voice_nr, int lyrics_line, bool lyrics_required, bool note_required);
 		NedStaff *getStaff(int nr);
 		void determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr);
 		void determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found);
 		void detectVoices(int staff_nr, unsigned int *voice_mask, NedSystem **last_system, unsigned long long *end_time);
 		void exportLilyPond(FILE *fp, int staff_nr, int voice_nr, int *last_line, unsigned int *midi_len,
-			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map);
+			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordnames);
+		void exportLilyChordnames(FILE *fp, int staff_nr, int *last_line, unsigned int *midi_len,
+			NedSystem *last_system, unsigned long long end_time, bool *in_alternative, bool with_break);
 		void exportLilyLyrics(FILE *fp, bool last_system, int staff_nr, int voice_nr, int line_nr, unsigned long long end_time, int *sil_count);
 		double m_system_start;
+		bool m_is_positioned;
 	private:
 		void drawBrace (cairo_t *cr, double leftx, double indent, double topy, double zoom_factor, double height, double toppos);
 		void drawBracket (cairo_t *cr, double leftx, double indent, double topy, double zoom_factor, double height, double toppos);
-		bool m_isUntouched;
 		bool m_tempUntouched;
 		bool m_check_line_compression;
-		void do_reposit(bool use_upbeat, double indent);
-		void compute_extra_space(double indent);
+		void do_reposit(bool use_upbeat);
+		void compute_extra_space(bool use_upbeat);
 		GList *m_staffs;
 		GList *m_deleted_staffs;
 		double m_ypos;
 		double m_width;
 		int m_system_number;
-#ifdef WITH_WEIGHTS
-		unsigned int mylog(unsigned int x);
-#endif
 		NedPage *m_page;
 		NedMeasure m_measures[MAX_MEASURES];
 		int m_measure_count;
 		double m_ori,m_netto;
 		double m_extra_space;
 		bool m_only_whole_elements;
+		NedMeasure *m_endMeasure;
+		bool m_freetexts_or_lyrics_present;
 };
 
 #endif /* SYSTEM_H */
