@@ -39,6 +39,7 @@ class NedVoice;
 class NedBbox;
 class NedMeasure;
 class NedSlur;
+class SpecialMeasure;
 class part;
 
 struct VoiceInfoContainer {
@@ -54,7 +55,10 @@ class NedVoice {
 	public:
 		NedVoice(NedStaff *staff, int nr, bool start);
 		~NedVoice();
-		void draw(cairo_t *cr);
+		NedVoice *clone(struct addr_ref_str **addrlist, struct addr_ref_str **slurlist, NedStaff *p_staff);
+		void adjust_pointers(struct addr_ref_str *addrlist, struct addr_ref_str *slurlist);
+		void draw(cairo_t *cr, bool *freetexts_or_lyrics_present);
+		void drawTexts(cairo_t *cr, double scale);
 		int getBeamCount();
 		int getVoiceNumber() {return m_voice_nr;}
 		void setVoiceNumber(int nr) {m_voice_nr = nr;}
@@ -62,11 +66,12 @@ class NedVoice {
 		NedPage *getPage();
 		NedSystem *getSystem();
 		NedStaff *getStaff() {return m_staff;}
-		void changeDirs();
 		int getNumElements();
 		void prepareForImport() {m_start_of_last_imported = m_end_of_last_imported = NULL;}
 		void appendElementsOfMeasureLength(part *part_ptr, unsigned int meas_duration, int meas_num);
 		bool handleImportedTuplets();
+		void handleEmptyMeasures();
+		void zoomFreeReplaceables(double zoom, double scale);
 		void removeLastImported();
 		void handleStaffElements();
 		bool trySelect (double x, double y, bool only_free_placeables);
@@ -76,14 +81,13 @@ class NedVoice {
 		double findTimePos(unsigned int time);
 		void addBeam(NedBeam *beam);
 		void removeBeam(NedBeam *beam);
-		void setStemDir(int dir) {m_stem_direction = dir;}
-		int getStemDir() {return m_stem_direction;}
 		NedChordOrRest *getNextChordOrRest(NedChordOrRest *element);
 		NedChordOrRest *getPreviousChordOrRest(NedChordOrRest *element);
 		void selectNextChord(NedChordOrRest *chord, int line, bool lyrics_required);
 		void selectPreviousChord(NedChordOrRest *chord, int line, bool lyrics_required);
-		void computeBeams(int *lyrics_lines, double *topy, double *boty);
+		void computeBeams(int *lyrics_lines, double *topy, double *boty /*, bool *topFix, bool *botFix */, int staff_voice_offs);
 		void computeTuplets(double *topy, double *boty);
+		void computeTies();
 		void removeTuplet(NedChordOrRest *element, NedCommandList *cmd_list = NULL);
 		void insertIntoSystem(GList *chords_or_rests_to_move);
 		void deleteNoteGroup(GList *chords_or_rests_to_move);
@@ -105,14 +109,18 @@ class NedVoice {
 		void collectVolumeChanges(GList **volume_changes);
 		void prepareReplay();
 		void findAccidentals(char offs_array[115], NedMeasure *meas_info, unsigned long long midi_time);
-		void removeUnneededAccidentals();
-		void resetPositionPointer(bool reset_multiple_flag, bool resetRestYPos = FALSE);
-		NedChordOrRest *getNextElementAfter(unsigned long long midi_time);
-		void shiftX(int pc, double shift);
-		void placeCurrentElementAt(double xpos);
-		void incrElementPointer();
+		void removeUnneededAccidentals(GList *selected_group);
+		void setInternalPitches();
+		void hideRests(NedCommandList *command_list, bool unhide);
+		void adjustAccidentals();
+		void setInternalPitch(int pitchdist, GList *selected_group);
+		void adjustAccidentals(int pitchdist, GList *selected_group);
+		void shiftNotes(unsigned long long start_time, int linedist, unsigned long long stop_time, bool stop_at);
+		void assignMidiTimes();
+		int assignElementsToMeasures(NedMeasure *measures, int offs, bool use_upbeat);
 		void searchForBeamGroups(unsigned int midi_start_time, NedChordOrRest *new_chord_or_rest = NULL);
-		void getElementAtPosition(int pc, bool with_rests, NedChordOrRest **chords, int *numchords, unsigned int *has_chords, unsigned int marker);
+		bool findFirstElementeGreaterThan(double x, NedChordOrRest **element);
+		bool findLastElementeLessThan(double x, NedChordOrRest **element);
 		void collectChordsAndRests(NedClipBoard *board, unsigned long long midi_start, unsigned long long midi_end);
 		void collectDestinationVoices(NedClipBoard *board);
 		void collectSelectionRectangleElements(double xp, double yp, NedBbox *sel_rect, GList **sel_group, 
@@ -131,13 +139,17 @@ class NedVoice {
 		void insertBlocks(NedCommandList *command_list, int blockcount, unsigned long long midi_time);
 		void testForPageBackwardTies(NedCommandList *command_list);
 		void checkForElementsToSplit(NedCommandList *command_list);
-		GList *getFirstChordOrRest(int lyrics_line, bool lyrics_required);
-		GList *getLastChordOrRest(int lyrics_line, bool lyrics_required);
+		GList *getFirstChordOrRest(int lyrics_line, bool lyrics_required, bool note_required);
+		GList *getLastChordOrRest(int lyrics_line, bool lyrics_required, bool note_required);
+		void setBeamFlagsOf(NedCommandList *command_list, GList *selected_group, NedChordOrRest *start_elem);
+		void isolate(NedCommandList *command_list, NedChordOrRest *element);
 		void determineVolume(unsigned int *vol, NedChordOrRest *element, int staff_nr);
 		void determineTempoInverse(NedChordOrRest *element, unsigned long long till, double *tempoinverse, bool *found);
 		bool detectVoices(unsigned int *voice_mask, unsigned long long *e_time);
 		void exportLilyPond(FILE *fp, int *last_line, unsigned int *midi_len, bool last_system,
-				unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map);
+				unsigned long long end_time, bool *in_alternative, NedSlur **lily_slur, unsigned int *lyrics_map, bool with_break, bool *chordname);
+		void exportLilyChordnames(FILE *fp, int *last_line, unsigned int *midi_len, bool last_system,
+				unsigned long long end_time, bool *in_alternative, bool with_break);
 		void exportLilyLyrics(FILE *fp, bool last_system, int line_nr, unsigned long long end_time, int *sil_count);
 	private:
 		bool beamEndRequired(GList *beamlist_so_far, unsigned int beats, NedMeasure *measure);
@@ -148,15 +160,15 @@ class NedVoice {
 		void searchForBreakableTies(NedCommandList *command_list, unsigned int dir, unsigned long long start_time, unsigned long long end_time);
 		void testForTupletEnd(unsigned long long start_time, unsigned int duration, unsigned long long *end_time);
 		void testAllMeasuresForFillup(NedCommandList *command_list);
+		void exportLilypondSpecMeasure(FILE *fp, unsigned int special, SpecialMeasure *spec_measure, bool *in_alternative, int measure_number);
 		bool insertGrace(double x, double y);
 		GList *m_chord_or_rests;
 		GList *m_beam_list;
 		GList *m_tuplet_list;
 		int m_voice_nr;
 		NedStaff *m_staff;
-		GList *m_position_ptr;
-		int m_stem_direction;
 		GList *m_start_of_last_imported, *m_end_of_last_imported;
+		bool m_free_texts_or_lyrics_present;
 
 	friend class NedInsertChordOrRestCommand;
 	friend class NedEreaseChordOrRestCommand;
